/*******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
* $Element: /project/dsp568_sdk/sdk/src/dsp56838evm/nos/sys/arch.c $ 
* $Author: saa $ 
* $Revision: /main/2 $ 
* $VOB: /project/dsp568_sdk $ 
* $OS: solaris $ 
*
* Description:  Startup and standard dispatcher
*
* Notes:          
*
******************************************************************************/
#include "port.h"
#include "bsp.h"

#include "arch.h"
#include "arch_off.h"
#include "periph.h"

#define INCLUDE_PLL
#include "pll.h"

// internal oscillator calibration 
#define PLL_CALIBRATION_ADDRESS 		0xF41B 
UWord16 plldrvCalibrate(void);
                                       
// defined in linker.cmd file 
extern int  * _StackAddr;           
extern void * _Xbss_begin_addr;
extern void * _Xbss_length;

extern void * _Xdata_ROMtoRAM_length;      
extern void * _Xdata_begin_addr_in_RAM;    
extern void * _Xdata_begin_addr_in_ROM;   
extern void * _Pcode_end_addr_in_ROM; 

extern UWord16             RegState[];
extern arch_PeriphRegState peripheralReg[archoff_registers_num];

extern void configFinalize();
extern void configInitialize();

void archStart()
{
    extern void main(void);
 	int i = 0;
 	int j = 0;
 	int k = 0;

    asm(moveu.w  #0xFFFF, M01);     // set the m register to linear addressing 
    asm(bfclr    #0x01B0, OMR);     // set  CM(32-bit) XP(harvard architecture), 
                                    // R(Convergent rounding),  SA(Saturation disabled) 
    asm(moveu.w  hws, la);          // clear the hardware stack 
    asm(moveu.w  hws, la);
    asm(bfset #0x0300, SR);         // disable interrupts                     
    
	// stack and data section 
    asm(moveu.w #_StackAddr, SP);   // assign stack address 
    asm(alignsp);                   // set stack pointer to known location 
                
    asm(moveu.w  #_Xbss_length, N);        // 64K bss  
    asm(move.l   #>>_Xbss_begin_addr, R2); // clear internal ram to initialize all 

    asm(clr.w X0);                         // loop to initialize bss section 
    asm(rep   N);
    asm(move.w  X0, x:(R2)+);

    asm(moveu.w  #_Xdata_ROMtoRAM_length, N);      
    asm(move.l  #>>_Xdata_begin_addr_in_RAM, R2);    
    asm(move.l  #>>_Pcode_end_addr_in_ROM, R3);    
    
    asm(do  N, copy_data);                 // loop to copy data section 
    asm(move.w  P:(R3)+, X0);            
    asm(nop);
    asm(nop);
    asm(move.w  X0, X:(R2)+);
	asm(copy_data:);  


#ifdef DSP56832XEVM
	// calibrate the internal oscillator to 8MHz 
	plldrvCalibrate(); // system clock initialization 
#endif

	// keep the power on reset state for peripheral registers    
	for(i = 0; i < archoff_registers_num; i++)
	{
 		for(j = 0; j <  peripheralReg[i].length; j ++)
 		{
 			RegState[k] = periphMemRead((volatile UWord16 *) (peripheralReg[i].StartAddress + j));
 			k++;
 		}
 	} 
  
    asm(jsr configInitialize);   // initialize configurable hardware 
    asm(bfclr #0x0300,SR);       // enable interrupt 
    asm(jsr main);               // call the application 
    asm(jsr configFinalize);     // flush output, handle end of program 
    asm(forever:);
    asm(debughlt);
    asm(bra   forever);          // loop forever if return from user application 
}

void Dispatcher(void);

asm void Dispatcher()
{
    move.l  A2,x:(SP)+ 
    move.l  B2,x:(SP)+
    move.l  X0,x:(SP)+
    move.l  Y,x:(SP)+
    move.l  N3,x:(SP)+
    move.l  M01,x:(SP)+
    move.l  OMR,x:(SP)+

    move.l  C2,x:(SP)+
    move.l  D2,x:(SP)+
    move.l  A10,x:(SP)+
    move.l  B10,x:(SP)+
    move.l  N,x:(SP)+
    move.l  R1,x:(SP)+
    move.l  R2,x:(SP)+
    move.l  R3,x:(SP)+

    // save hardware loop stack and registers 
    move.l  la,x:(SP)+
    move.l  la2,x:(SP)+
    move.l  lc2,x:(SP)+
    move.l  lc,x:(SP)+
    move.l  hws,x:(SP)+
    move.l  hws,x:(SP)+

    // set consistent state for critical registers
    bfclr   #$0130,OMR  // convergent rounding, no saturation mode, 36-bit compares
    move.l  R4,x:(SP)   // save last register in context. Moved here to eliminate nop.
    moveu.w #0xFFFF,m01 // linear addressing       

    jsr (R0)
    
FDispatchRestore:

    move.l  x:(SP)-,R4

    // restore hardware stack
    move.w  hws,A       // clear HWS to ensure reload
    move.w  hws,A
    move.l  x:(SP)-,hws
    move.l  x:(SP)-,hws
    
    // restore all saved registers
    move.l  x:(SP)-,lc
    move.l  x:(SP)-,lc2
    move.l  x:(SP)-,la2
    move.l  x:(SP)-,la

    move.l  x:(SP)-,R3
    move.l  x:(SP)-,R2
    move.l  x:(SP)-,R1
    move.l  x:(SP)-,N
    move.l  x:(SP)-,B
    move.l  x:(SP)-,A

    move.l  x:(SP)-,D2
    move.l  x:(SP)-,C2
    move.l  x:(SP)-,OMR 
    move.l  x:(SP)-,M01
    move.l  x:(SP)-,N3
    move.l  x:(SP)-,Y
    move.l  x:(SP)-,X0
    move.l  x:(SP)-,B2
    move.l  x:(SP)-,A2
    move.l  x:(SP)-,R0   // R0 saved in prolog procedure 
    move.l  x:(SP)-,SP
    nop        
    nop
    
    rti
}

/*******************************************************************************
*
* Module: plldrvCalibrate()
*
* Description: 
*     Set the internal oscillator control register (IOSCTL) to the factory 
*     calibration value. This value is calibrated to 8MHz at room temperature
*     at the factory. If the value is invalid (0xFFFF), the IOSCTL will not 
*     be changed.
*
* Returns: Factory Calibration value.
*
* Arguments:    None.
*
* Range Issues: None.
*
* Special Issues:
*     The calibration value is stored at the end of the Data Flash information
*     block (0xF41B). This address is RESERVED but may be overwritten by the user  
*     to force a different calibration value.
*
*******************************************************************************/
UWord16 plldrvCalibrate(void)
{
	UWord16 trimval;

	// write calibration value
	trimval = *(UWord16 *) PLL_CALIBRATION_ADDRESS;
	
	trimval = trimval & 0x03FF;
	trimval = trimval | (periphMemRead(&ArchIO.Pll.OscControlReg) & (~(0x03FF)));
	periphMemWrite(trimval, &ArchIO.Pll.OscControlReg);

   	return (trimval);
}