/*******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
* $Element:  $ 
* $Author:  $ 
* $Revision:  $ 
* $VOB:  $ 
* $OS:  $ 
*
* Description:       Flash driver for DSP5680x
*
* Notes:  
*
******************************************************************************/

#include "port.h"
#include "arch.h"
#include "arch_off.h"
#include "bsp.h"
#include "io.h"
#include "flashdrv.h"
#include "periph.h"
#include "string.h"


unsigned int UpdatePageContex( handle_t hndl, size_t n_bytes );
void mainFlashConfig();
char* copyFromBuf( unsigned int off, char* buf, unsigned int len );
char* compareWithBuf( unsigned int off, char* buf, unsigned int len );
char* copyToBuf( unsigned int off, char* buf, unsigned int len );  

// char* copyToBuf( struct sflashDesc* hndl, char* buf, unsigned int len );


union s_flash_tmp_buf
{
    unsigned char  buf8[9];
    unsigned int buf16[PAGE_SIZE_IN_WORDS];  /* temporary buffer */
    unsigned int bufp16[PAGE_SIZE_P_IN_WORDS];  /* for p: */
    unsigned long buf32[1];
};
union s_flash_tmp_buf flash_tmp_buf= {0};
/* define temporary buffer for flash operations */

ssize_t flashPCompare( handle_t hndl, const void * pBuffer, size_t n_bytes )
{
#if __m56800E_lmm__ 
    asm(move.w A0,A);
    asm(move.w  A1,Y0);
#else
	asm(nop);
	asm(nop);
#endif
    asm( move.w Y0, N );
    asm( move.l X:(R2 + sFlash_offset_seek), A ); /* P byte address */
    asm( asr A );
    asm( move.l  A10, R1 );
    asm( bcc  skip_odd_first );
        asm( move.w  P:(R1)+, X0 );
        asm( asrr.w   #8, X0 );
        asm( dec.w  Y0 );               /* update length */
        asm( moveu.bp X:(R3)+, Y1 );
        asm( cmp.b X0, Y1 );
        asm( bne no_compare );
skip_odd_first:
    asm( asr  Y0 );                 /* shift off to first words */
    asm( rol.w  A );                /* C ->B1:0 */
    asm( do Y0, enddo );
        asm( moveu.bp  X:(R3)+, X0 );
        asm( moveu.bp  X:(R3)+, Y1 );
        asm( asll.w   #8, Y1 );
        asm( or.w   X0, Y1 );
        asm( move.w  P:(R1)+, X0 );
        asm( cmp.w X0, Y1 );
        asm( beq continue_compare );
            asm( enddo );
            asm( bra  no_compare );
continue_compare:
        asm( nop ); /* 1 nop needed ? */
        asm( nop );
enddo:
    asm( brclr #0x0001, A1, skip_odd_last );
        asm( move.w P:(R1)+, X0 );
        asm( moveu.bp X:(R3)+, Y1 );
        asm( cmp.b X0, Y1 );
        asm( bne no_compare );
skip_odd_last:
    asm( move.w N, Y0 );

#if __m56800E_lmm__ 
    asm(move.w Y0,A0);
    asm(clr.w  A1);
#else
    asm(nop);
	asm(nop);
#endif

    asm( rts);

no_compare:

#if __m56800E_lmm__ 
    asm(move.l  #0xFFFFFFFF,A);
    asm(clr.w  A1);
#else
    asm( move.w #0xFFFF, Y0 );
	asm(nop);
#endif

}


/*****************************************************************************
* Module:         ioctlFLASH_RAW_WRITE_PMEM
* Description:    write page from internal buffer
* Arguments:      page number
* Range Issues:     
* Special Issues:  
*****************************************************************************/


unsigned long LongReg;

ssize_t flashFastWrite(handle_t hndl, const void * pBuffer, size_t n_bytes)
{
#if __m56800E_lmm__ 
    asm(move.w A0,A);
    asm(move.w  A1,Y0);
#else
	asm(nop);
	asm(nop);
#endif
    asm( move.w #0x0002, X:(BSP_PERIPH_BASE + archoff_Flash_ModuleControlReg) ); /* select P Flash register bank */
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionReg) ); /* unsecure data flash */ 
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionBootReg) );

    asm( clr A );
    asm( clr B );
    asm( move.w Y0, B0 );

    asm( tfra R2, R1 );
    asm( move.l X:(R1 + sFlash_offset_seek), R4 ); /* B - byte address */
    asm( move.w R4, X0 );  /* dword address */
    asm( asra R4 );  asm( asra R4 );  asm( asla R4 ); /*  word address */

    asm( and.w #3, X0 );
    asm( move.l #LongReg, R2 );
    asm( move.w X0, R0 );
    asm( asla R2 );
    asm( adda R2, R0, N );
    asm( tfra N, R0 );

    asm( moveu.w M01, N );
    asm( moveu.w #0x3, M01 );
   
    //asm( brclr #0x0003, X0, skip_filling ); /* first step */
    asm( bra  odd_filling ); /* first enter */
loop1:
    asm( brclr #0x0003, X0, do_filling );
    asm( bra skip_filling );
    do_filling:
        asm( move.l LongReg, Y ); /* load -1 */

        inc_loop:  /* check if CBEIF is set */
            asm( brclr #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), inc_loop  );

        repeat_loop:
            asm(bfset #0x0300,SR); /* dis isrs!!! */
            asm( tfra R4, R2 );
            asm( move.w Y0, P:(R2)+ ); /* select data flash programming */
            asm( move.w Y1, P:(R2)+ ); /* select data flash programming(2) */
            asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_CommandReg) ); /* program command */
            asm( move.w #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear CBEIF flag */
            asm( move.w X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), Y0 );
            asm(bfclr #0x0300,SR);

            asm( brclr #0x20, Y0, end_PVIOL ); /* wait until PVIOL flag is set */ 
            asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear PVIOL flag */
            asm( bra repeat_loop );
        end_PVIOL:

            asm( brclr #0x10, Y0, end_ACCERR );  /* wait until ACCERR flag is set */ 
            asm( move.w #0x10, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear ACCERR flag */
            asm( bra repeat_loop );
        end_ACCERR: 

        wait_CCIF:  /* check if CCIF is set */
            asm( brclr #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CCIF  );

        asm( adda #2, R4 );
    odd_filling:
        asm( cmp.l A, B );
        asm( ble end_loop );
        asm( tfra R4, R2 );
        asm( move.w P:(R2)+, Y0 );
        asm( move.w P:(R2)+, Y1 );
        asm( move.l Y, LongReg ); /* load old value */
    skip_filling:
    asm( cmp.l A, B );
    asm( ble skip_copy );
        asm( moveu.bp  X:(R3)+, Y0 );
        asm( move.bp  Y0, X:(R0)+ );
        asm( inc.l A );         /* use len as actual */
    skip_copy:
        asm( inc.w  X0 );
    asm( bra loop1 );
end_loop:    

    asm( move.l X:(R1 + sFlash_offset_seek), B ); /* B - byte address */
    asm( add A, B );
    asm( move.l B10, X:(R1 + sFlash_offset_seek) );
    asm( tfr A, Y );

#if __m56800E_lmm__ 
    asm(move.w Y0,A0);
    asm(clr.w  A1);
#else
    asm(nop);
	asm(nop);
#endif


    asm( moveu.w N, M01 );  /* restore M0 */
}



/*****************************************************************************
* Module:         copyToBuf
* Description:    copy to flash driver internal buffer
* Returns:        modified user buffer pointer
* Arguments:      offset in flash driver internal buffer
*                 user buffer pointer
*                 user buffer length to copy
* Range Issues:     
* Special Issues:  
*****************************************************************************/
#if 0

char* copyToBuf( struct sflashDesc* hndl, char* buf, unsigned int len )
{
    int lim = hndl->toffs + len;
    int i;

    int foundfirst = 0; 
    char tmp;
    int lastdif = 0;

    hndl->tdiflen = 0;
    hndl->tdifoff = 0;
    hndl->flags &= ~O_ERASE_FLAG;

    for( i = hndl->toffs; i < lim; i++ )
    {
        if( flash_tmp_buf.buf8[i] != *buf )
        {
            tmp = flash_tmp_buf.buf8[i] ^ *buf; /* transition */
            if( *buf & tmp ) /* only flash(0->1) transition required */
            {
                hndl->flags |= O_ERASE_FLAG;
            }
            if( !foundfirst )
            {
                foundfirst = 1;
                hndl->tdifoff = i;
            }
            lastdif = i + 1;
        }
        flash_tmp_buf.buf8[i] = *buf++;
    }
    if( lastdif != 0 )
    {
        hndl->tdiflen = lastdif - hndl->tdifoff; /* calculate real length */
    }

    return buf;
}

#else

char* copyToBuf( unsigned int off, char* buf, unsigned int len )
{
    asm( asr  Y0 );      /* shift off to first words */
    asm( adda #flash_tmp_buf.buf32, Y0, R3 );  /* load buffer address, c not changed */
    asm( bcc  skip_odd_first );
        asm( move.w  X:(R3)+, Y0 );
        asm( bfclr  #0xFF00, Y0 );
        asm( deca R3 );
        asm( move.bp X:(R2)+, X0 );
        asm( asll.w   #8, X0 );
        asm( or.w   X0, Y0 );
        asm( move.w Y0, X:(R3)+ );      /* move first byte to word array */
        asm( dec.w  Y1 );               /* update length */
skip_odd_first:
    asm( asr  Y1 );                 /* shift off to first words */
    asm( rol.w  A );                /* C ->B1:0 */
    asm( do Y1, enddo );
        asm( moveu.bp  X:(R2)+, X0 );
        asm( bfclr  #0xFF00, X0 );
        asm( moveu.bp  X:(R2)+, Y0 );
        asm( asll.w   #8, Y0 );
        asm( or.w   X0, Y0 );
        asm( move.w Y0, X:(R3)+ );
enddo:
    asm( brclr #0x0001, A1, skip_odd_last );
        asm( move.w X:(R3)+, X0 );
        asm( bfclr  #0x00FF, X0 );
        asm( deca R3 );
        asm( moveu.bp X:(R2)+, Y0 );
        asm( bfclr  #0xFF00, Y0 );
        asm( or.w   X0, Y0 );
        asm( move.w Y0, X:(R3)+ );
skip_odd_last:
    ;
}
#endif //0

/*****************************************************************************
* Module:         copyFromBuf
* Description:    copy from flash driver internal buffer
* Returns:        modified user buffer pointer
* Arguments:      offset in flash driver internal buffer
*                 user buffer pointer
*                 user buffer length to copy
* Range Issues:     
* Special Issues:  
*****************************************************************************/
char* copyFromBuf( unsigned int off, char* buf, unsigned int len )
{
    asm( asr  Y0 );     /* shift off to first words */
    asm( adda #flash_tmp_buf.buf32, Y0, R3 );  /* load buffer address, c not changed */
    asm( bcc  skip_odd_first );
        asm( move.w  X:(R3)+, X0 );
        asm( asrr.w   #8, X0 );
        asm( dec.w  Y1 );               /* update length */
        asm( move.bp X0, X:(R2)+ );
skip_odd_first:
    asm( asr  Y1 );                 /* shift off to first words */
    asm( rol.w  A );                /* C ->B1:0 */
    asm( do Y1, enddo );
        asm( move.w  X:(R3)+, X0 );
        asm( move.w  X0, Y0 );
        asm( asrr.w   #8, X0 );
        asm( move.bp  Y0, X:(R2)+ );
        asm( move.bp  X0, X:(R2)+ );
        asm( nop ); /* needed nops ? */
        asm( nop );
enddo:
    asm( brclr #0x0001, A1, skip_odd_last );
        asm( move.w X:(R3)+, X0 );
        asm( nop );
        asm( nop );
        asm( move.bp  X0, X:(R2)+ );
skip_odd_last:
    ;
}

/*****************************************************************************
* Module:         copyFromBuf
* Description:    copy from flash driver internal buffer
* Returns:        modified user buffer pointer
* Arguments:      offset in flash driver internal buffer
*                 user buffer pointer
*                 user buffer length to copy
* Range Issues:     
* Special Issues:  
*****************************************************************************/
char* compareWithBuf( unsigned int off, char* buf, unsigned int len )
{
    asm( asr  Y0 );     /* shift off to first words */
    asm( adda #flash_tmp_buf.buf32, Y0, R3 );  /* load buffer address, c not changed */
    asm( bcc  skip_odd_first );
        asm( move.w  X:(R3)+, X0 );
        asm( asrr.w   #8, X0 );
        asm( dec.w  Y1 );               /* update length */
        asm( moveu.bp X:(R2)+, Y0 );
        asm( cmp.b X0, Y0 );
        asm( bne no_compare );
skip_odd_first:
    asm( asr  Y1 );                 /* shift off to first words */
    asm( rol.w  A );                /* C ->B1:0 */
    asm( do Y1, enddo );
        asm( moveu.bp  X:(R2)+, X0 );
        asm( moveu.bp  X:(R2)+, Y0 );
        asm( asll.w   #8, Y0 );
        asm( or.w   X0, Y0 );
        asm( move.w  X:(R3)+, X0 );
        asm( cmp.w X0, Y0 );
        asm( beq continue_compare );
            asm( enddo );
            asm( bra  no_compare );
continue_compare:
        asm( nop ); /* 1 nop needed ? */
        asm( nop );
enddo:
    asm( brclr #0x0001, A1, skip_odd_last );
        asm( move.w X:(R3)+, X0 );
        asm( moveu.bp X:(R2)+, Y0 );
        asm( cmp.b X0, Y0 );
        asm( bne no_compare );
skip_odd_last:
    asm( rts);
no_compare:
    asm( move.l #0, R2 );
}



/*****************************************************************************
* Module:         ioctlFLASH_RAW_MASSERASE_PMEM
* Description:    erase page
* Arguments:      page number
* Range Issues:     
* Special Issues:  for bootloader only 
*****************************************************************************/
void ioctlFLASH_RAW_MASSERASE_PMEM(handle_t hndl, unsigned long params )
{
    asm( move.l  #0, R0 );
    
    asm( move.w #0x0002, X:(BSP_PERIPH_BASE + archoff_Flash_ModuleControlReg) ); /* select P Flash register bank */
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionReg) ); /* unsecure data flash */ 
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionBootReg) );
   
    asm( wait_CBEIF: ); 
    asm( brclr #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CBEIF  );

    asm( repeat_loop: ); 
    /* dis isrs!!! */
    asm( tfra R0, R1 );
    asm( clr.w X0 );
    asm( move.w X0, P:(R1)+ ); /* select data flash programming */
    
    asm( move.w #0x41, X:(BSP_PERIPH_BASE + archoff_Flash_CommandReg) ); /* page erase command */
    asm( move.w #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear CBEIF flag */

    asm( move.w X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), Y0 );

    asm( brclr #0x20, Y0, end_PVIOL ); /* wait until PVIOL flag is set */ 
    asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear PVIOL flag */
    asm( bra repeat_loop );
    asm( end_PVIOL: );

    asm( brclr #0x10, Y0, end_ACCERR );  /* wait until ACCERR flag is set */ 
    asm( move.w #0x10, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear ACCERR flag */
    asm( bra repeat_loop );
    asm( end_ACCERR: );

    asm( wait_CCIF: ); /* check if CCIF is set */
    asm( brclr #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CCIF  );
    
}



/*****************************************************************************
* Module:         ioctlFLASH_RAW_ERASE_PMEM
* Description:    erase page
* Arguments:      page number
* Range Issues:     used Y, R0, R1
* Special Issues:  
*****************************************************************************/
void ioctlFLASH_RAW_ERASE_PMEM(unsigned int page)
{
    asm( clr.w Y1 );
    asm( asll.l  #PAGE_SIZE_P_LOG, Y);          /* convert page to lenght */
    asm( move.l  Y, R0 );
    
    asm( move.w #0x0002, X:(BSP_PERIPH_BASE + archoff_Flash_ModuleControlReg) ); /* select P Flash register bank */
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionReg) ); /* unsecure data flash */ 
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionBootReg) );
   
    asm( wait_CBEIF: ); 
    asm( brclr #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CBEIF  );

    asm( repeat_loop: ); 
    /* dis isrs!!! */
    asm( tfra R0, R1 );
    asm( clr.w Y1 );
    asm( move.w Y1, P:(R1)+ ); /* select data flash programming */
    
    asm( move.w #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_CommandReg) ); /* page erase command */
    asm( move.w #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear CBEIF flag */

    asm( move.w X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), Y0 );

    asm( brclr #0x20, Y0, end_PVIOL ); /* wait until PVIOL flag is set */ 
    asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear PVIOL flag */
    asm( bra repeat_loop );
    asm( end_PVIOL: );

    asm( brclr #0x10, Y0, end_ACCERR );  /* wait until ACCERR flag is set */ 
    asm( move.w #0x10, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear ACCERR flag */
    asm( bra repeat_loop );
    asm( end_ACCERR: );

    asm( wait_CCIF: ); /* check if CCIF is set */
    asm( brclr #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CCIF  );
    
}

/*****************************************************************************
* Module:         ioctlFLASH_RAW_ERASE_XMEM
* Description:    erase page
* Arguments:      page number
* Range Issues:     used Y, R0, [R1]
* Special Issues:  
*****************************************************************************/
void ioctlFLASH_RAW_ERASE_XMEM(unsigned int page )
{
    asm( asll.w  #PAGE_SIZE_LOG, Y0);           /* convert page to lenght */
    asm( adda # XMEM_FLASH_OFF, Y0, R0 );                 /* 0x2000 - data address page */
    
    asm( move.w #0x0001, X:(BSP_PERIPH_BASE + archoff_Flash_ModuleControlReg) ); /* select P Flash register bank */
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionReg) ); /* unsecure data flash */ 
  

    asm( wait_CBEIF: ); 
    asm( brclr #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CBEIF  );

    asm( repeat_loop: ); 
    /* dis isrs!!! */

    asm( clr.w Y1 );
    asm( move.w Y1, X:(R0) ); /* select data flash programming */
    
    asm( move.w #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_CommandReg) ); /* page erase command */
    asm( move.w #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear CBEIF flag */

    asm( move.w X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), Y0 );

    asm( brclr #0x20, Y0, end_PVIOL ); /* wait until PVIOL flag is set */ 
    asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear PVIOL flag */
    asm( bra repeat_loop );
    asm( end_PVIOL: );

    asm( brclr #0x10, Y0, end_ACCERR );  /* wait until ACCERR flag is set */ 
    asm( move.w #0x10, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear ACCERR flag */
    asm( bra repeat_loop );
    asm( end_ACCERR: );

    asm( wait_CCIF: ); /* check if CCIF is set */
    asm( brclr #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CCIF  );
}


/*****************************************************************************
* Module:         ioctlFLASH_RAW_ERASE_BOOT
* Description:    erase page
* Arguments:      page number
* Range Issues:     used Y, R0, R1
* Special Issues:  
*****************************************************************************/
void ioctlFLASH_RAW_ERASE_BOOT(unsigned int page )
{

    asm( asll.w  #PAGE_SIZE_LOG, Y0);           /* convert page to lenght */
    asm( adda 0x20000, Y0, R0 );                 /* 0x20000 - P: boot address page */
    
    
    asm( move.w #0x0002, X:(BSP_PERIPH_BASE + archoff_Flash_ModuleControlReg) ); /* select P Flash register bank */
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionBootReg) );
   

    asm( wait_CBEIF: ); 
    asm( brclr #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CBEIF  );

    asm( repeat_loop: ); 
    /* dis isrs!!! */
    asm( tfra R0, R1 );
    asm( clr.w Y1 );
    asm( move.w Y1, P:(R1)+ ); /* select data flash programming */
    
    asm( move.w #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_CommandReg) ); /* page erase command */
    asm( move.w #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear CBEIF flag */

    asm( move.w X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), Y0 );

    asm( brclr #0x20, Y0, end_PVIOL ); /* wait until PVIOL flag is set */ 
    asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear PVIOL flag */
    asm( bra repeat_loop );
    asm( end_PVIOL: );

    asm( brclr #0x10, Y0, end_ACCERR );  /* wait until ACCERR flag is set */ 
    asm( move.w #0x10, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear ACCERR flag */
    asm( bra repeat_loop );
    asm( end_ACCERR: );

    asm( wait_CCIF: ); /* check if CCIF is set */
    asm( brclr #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CCIF  );

}


/*****************************************************************************
* Module:         ioctlFLASH_RAW_READ_PMEM
* Description:    read page to internal buffer
* Arguments:      page number
* Range Issues:     
* Special Issues:  
*****************************************************************************/
void ioctlFLASH_RAW_READ_PMEM(unsigned int page )
{
    asm( move.l  # flash_tmp_buf.buf32, R2 );   /* load modified address */
    asm( clr.w Y1 );
    asm( asll.l  #PAGE_SIZE_P_LOG, Y);          /* convert page to lenght */
    asm( move.l  Y, R0 );
    
    asm( move.w  #PAGE_SIZE_P_IN_WORDS, X0 );    /* load page size */ 
    asm( do  X0, enddo );                       /* copy page to memory location */
        asm( move.w P:(R0)+, Y1 );
        asm( nop );
        asm( nop );
        asm( move.w Y1, X:(R2)+ );
    asm( enddo: );
    asm( nop );
}

/*****************************************************************************
* Module:         ioctlFLASH_RAW_READ_XMEM
* Description:    read page to internal buffer
* Arguments:      page number
* Range Issues:     
* Special Issues:  
*****************************************************************************/
void ioctlFLASH_RAW_READ_XMEM(unsigned int page )
{
    asm( move.l  # flash_tmp_buf.buf32, R2 );   /* load modified address */
    asm( asll.w  #PAGE_SIZE_LOG, Y0);           /* convert page to lenght */
    asm( adda # XMEM_FLASH_OFF, Y0, R3 );                 /* 0x2000 - X: data address page */
    asm( move.w  #PAGE_SIZE_IN_WORDS, X0 );     /* load page size */ 
    asm( do  X0, enddo );                       /* copy page to memory location */
        asm( move.w X:(R3)+, X0 );
        asm( move.w X0, X:(R2)+ );
    asm( enddo: );
    asm( nop );
}

/*****************************************************************************
* Module:         ioctlFLASH_RAW_READ_BOOT
* Description:    read page to internal buffer
* Arguments:      page number
* Range Issues:     
* Special Issues:  
*****************************************************************************/
void ioctlFLASH_RAW_READ_BOOT(unsigned int page )
{
    asm( move.l  # flash_tmp_buf.buf32, R2 );   /* load modified address */
    asm( asll.w  #PAGE_SIZE_LOG, Y0);           /* convert page to lenght */
    asm( adda 0x20000, Y0, R0 );                 /* 0x2000 - P: boot address page */
    asm( move.w  #PAGE_SIZE_IN_WORDS, X0 );     /* load page size */ 
    asm( do  X0, enddo );                       /* copy page to memory location */
        asm( move.w P:(R0)+, Y1 );
        asm( nop );
        asm( nop );
        asm( move.w Y1, X:(R2)+ );
    asm( enddo: );
    asm( nop );
}
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************
* Module:         ioctlFLASH_RAW_WRITE_PMEM
* Description:    write page from internal buffer
* Arguments:      page number
* Range Issues:     
* Special Issues:  
*****************************************************************************/
void ioctlFLASH_RAW_WRITE_PMEM(unsigned int page )
{
    asm( clr.w Y1 );
    asm( tfr Y, A );
//    asm( jsr ioctlFLASH_RAW_ERASE_PMEM );   /* Y, R0, R1 changed */

    asm( asll.l  #PAGE_SIZE_P_LOG, A);          /* convert page to lenght */
    asm( move.l  A10, R0 );

 //   asm( move.w  #PAGE_SIZE_P_IN_WORDS, X0 );    /* load page size */ 
    asm( move.w  #PAGE_SIZE_IN_WORDS, X0 );    /* load page size in double words */ 
    asm( move.l  # flash_tmp_buf.buf32, R1 );   /* load modified address */


    asm( tfra R0, R4 );     /* R2 - dst */
    asm( tfra R1, R3 );     /* R3 - src */

#if 1    
    asm( do X0, end_cmp_loop );
        asm( move.w P:(R0)+, A1 );
        asm( move.w A1, A0 );
        asm( move.l X:(R1)+, B );
        asm( move.w P:(R0)+, A1 );
        asm( eor.l B, A );
        asm( and.l B, A );
        asm( beq continue_loop );
            asm( enddo );
            /* Y0 - required page */
            asm( jsr ioctlFLASH_RAW_ERASE_PMEM );   /* Y, R0, R1 changed */
            asm( bra endloop );

continue_loop:
        asm( nop );
        asm( nop );
end_cmp_loop:
        asm( nop );
        asm( nop );
endloop:        
#endif  //0
    
    asm( move.w #0x0002, X:(BSP_PERIPH_BASE + archoff_Flash_ModuleControlReg) ); /* select P Flash register bank */
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionReg) ); /* unsecure data flash */ 
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionBootReg) );
   
    asm( do X0, end_data_loop );

        asm( move.l X:(R3), Y );
#if 1
        asm( tfra R4, R2 );
        asm( move.w P:(R2)+, A1 );
        asm( move.w A1, A0 );
        asm( move.w P:(R2)+, A1 );
        asm( cmp.l A, Y );
        asm( beq  continue_data_loop );
#endif //0
    inc_loop:  /* check if CBEIF is set */
        asm( brclr #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), inc_loop  );

    repeat_loop:
        
        asm(bfset #0x0300,SR); /* dis isrs!!! */
        asm( tfra R4, R2 );
        asm( move.w Y0, P:(R2)+ ); /* select data flash programming */
        asm( move.w Y1, P:(R2)+ ); /* select data flash programming(2) */
        asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_CommandReg) ); /* program command */
        asm( move.w #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear CBEIF flag */
        asm( move.w X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), Y0 );
        asm(bfclr #0x0300,SR);

        asm( brclr #0x20, Y0, end_PVIOL ); /* wait until PVIOL flag is set */ 
        asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear PVIOL flag */
        asm( bra repeat_loop );
    end_PVIOL:

        asm( brclr #0x10, Y0, end_ACCERR );  /* wait until ACCERR flag is set */ 
        asm( move.w #0x10, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear ACCERR flag */
        asm( bra repeat_loop );
    end_ACCERR: 

    wait_CCIF:  /* check if CCIF is set */
        asm( brclr #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CCIF  );
    continue_data_loop:
        asm( adda #2, R3);
        asm( adda #2, R4);  /* update array pointers */
        asm( nop );
        asm( nop );
        asm( nop );
end_data_loop:
        ;
}

/*****************************************************************************
* Module:         ioctlFLASH_RAW_WRITE_XMEM
* Description:    write page from internal buffer
* Arguments:      page number
* Range Issues:     
* Special Issues:  
*****************************************************************************/
void ioctlFLASH_RAW_WRITE_XMEM(unsigned int page )
{
    asm( move.w Y0, X0 );   /* save page */
//    asm( jsr ioctlFLASH_RAW_ERASE_XMEM );   /* Y, R0, R1 changed */

    asm( asll.w  #PAGE_SIZE_LOG, X0);           /* convert page to lenght */
    asm( adda # XMEM_FLASH_OFF, X0, R0 );        /* X address page */
    asm( move.w  #PAGE_SIZE_IN_WORDS, X0 );     /* load page size */ 
    asm( move.l  # flash_tmp_buf.buf32, R1 );   /* load modified address */

    asm( tfra R0, R4 );     /* R2 - dst */
    asm( tfra R1, R3 );     /* R3 - src */

#if 1    
    asm( do X0, end_cmp_loop );
        asm( move.w X:(R0)+, A1 );
        asm( move.w X:(R1)+, B1 );
        asm( eor.w B, A );
        asm( and.w B, A );
        asm( beq continue_loop );
            asm( enddo );
            /* Y0 - required page */
            asm( jsr ioctlFLASH_RAW_ERASE_XMEM );   /* Y, R0, R1 changed */
            asm( bra endloop );

continue_loop:
        asm( nop );
        asm( nop );
end_cmp_loop:
        asm( nop );
        asm( nop );
endloop:        
#endif  //0


    
    asm( move.w #0x0001, X:(BSP_PERIPH_BASE + archoff_Flash_ModuleControlReg) ); /* select P Flash register bank */
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionReg) ); /* unsecure data flash */ 
   
    asm( do X0, end_data_loop );

        asm( move.w X:(R3), Y1 );
#if 1
        asm( move.w X:(R4), A1 );
        asm( cmp.w A, Y1 );
        asm( beq  continue_data_loop );
#endif //0

inc_loop:  /* check if CBEIF is set */
        asm( brclr #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), inc_loop  );

    repeat_loop: 
        asm(bfset #0x0300,SR); /* dis isrs!!! */
        asm( move.w Y1, X:(R4) ); /* select data flash programming */
        asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_CommandReg) ); /* program command */
        asm( move.w #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear CBEIF flag */
        asm( move.w X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), Y0 );
        asm(bfclr #0x0300,SR);

        asm( brclr #0x20, Y0, end_PVIOL ); /* wait until PVIOL flag is set */ 
        asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear PVIOL flag */
        asm( bra repeat_loop );
    end_PVIOL: 

        asm( brclr #0x10, Y0, end_ACCERR );  /* wait until ACCERR flag is set */ 
        asm( move.w #0x10, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear ACCERR flag */
        asm( bra repeat_loop );
    end_ACCERR: 

    wait_CCIF:  /* check if CCIF is set */
        asm( brclr #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CCIF  );

    continue_data_loop:
        asm( adda #1, R3);
        asm( adda #1, R4);  /* update array pointers */
        asm( nop ); /* nop is needed */
        asm( nop );
        asm( nop );
end_data_loop:
    ;
}

/*****************************************************************************
* Module:         ioctlFLASH_RAW_WRITE_BOOT
* Description:    write page from internal buffer
* Arguments:      page number
* Range Issues:     
* Special Issues:  
*****************************************************************************/
void ioctlFLASH_RAW_WRITE_BOOT(unsigned int page )
{

    asm( move.w Y0, X0 );   /* save page */
    asm( jsr ioctlFLASH_RAW_ERASE_BOOT );   /* Y, R0, R1 changed */

    asm( asll.w  #PAGE_SIZE_LOG, X0);           /* convert page to lenght */
    asm( adda 0x20000, X0, R0 );                 /* 0x20000 - P: boot address page */
    asm( move.w  #PAGE_SIZE_IN_WORDS, X0 );     /* load page size */ 
    asm( move.l  # flash_tmp_buf.buf32, R1 );   /* load modified address */
    
    asm( move.w #0x0002, X:(BSP_PERIPH_BASE + archoff_Flash_ModuleControlReg) ); /* select P Flash register bank */
    asm( move.w #0, X:(BSP_PERIPH_BASE + archoff_Flash_ProtectionBootReg) );
   
    inc_loop:  /* check if CBEIF is set */

        asm( brclr #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), inc_loop  );

    repeat_loop:  
        /* dis isrs!!! */
        asm( tfra R0, R3 );
        asm( move.w X:(R1), Y1 );
        asm( move.w Y1, P:(R3)+ ); /* select data flash programming */
        asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_CommandReg) ); /* program command */
        asm( move.w #0x80, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear CBEIF flag */

        asm( move.w X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), Y0 );

        asm( brclr #0x20, Y0, end_PVIOL ); /* wait until PVIOL flag is set */ 
        asm( move.w #0x20, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear PVIOL flag */
        asm( bra repeat_loop );
    end_PVIOL:

        asm( brclr #0x10, Y0, end_ACCERR );  /* wait until ACCERR flag is set */ 
        asm( move.w #0x10, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg) ); /* clear ACCERR flag */
        asm( bra repeat_loop );
    end_ACCERR:

    wait_CCIF:  /* check if CCIF is set */
        asm( brclr #0x40, X:(BSP_PERIPH_BASE + archoff_Flash_UserStatusReg ), wait_CCIF  );
 
        asm( adda #1, R1);
        asm( adda #1, R0);  /* update array pointers */

    asm( dec.w X0 );
    asm( bne inc_loop );
    
}
/*****************************************************************************
* Module:         mainFlashConfig()
* Description:    Function to initialize flash device.
* Range Issues:   critical for chip  
* Special Issues:  see configguration for dependences 
*****************************************************************************/
void mainFlashConfig()
{
    extern const unsigned int flash_divider;
    /*  Clock Divider Register, check "Clock Divider Loaded" */
    if( 1 /*!( periphMemRead(&ArchIO.Flash.ClockDividerReg) & 0x80) */ ) 
    { 
         /* initialize with prescaler, PRDIV8 + FLASH_DIVIDER /
         150[kHz] < (system_clk/(2*8))/(EEP_nwDIV+1) < 200[kHz] 
         then system_clk = 52Mhz and EEP_nwDIV = 16 then HFM_clk = 191.2[kHz]
         */
        periphMemWrite( flash_divider, &ArchIO.Flash.ClockDividerReg ); 

    }
    else  
    { 
        /* Note that DSP is reseted only during software debugging by CodeWarrior. 
           The reset must be performed here in order to modify HFMCLKD register to 
           reflect new IPBus frequency set by the PLL. If the DSP wasn't reseted 
           than the HFCLKD register could be modified - it is write once register */  
    
        /* periphMemWrite( 0x10 , &ArchIO.Sim.ControlReg); */
    } 
} 


/*****************************************************************************
* Module(s):      flashPmemOpen, flashXmemOpen, flashBootOpen
* Description:    Open specified Flash device for user. 
* Returns:        Device descriptor if success
*                 -1 if specified device name is not Flash device name
* Arguments:      pName - BSP Flash device name
*                 OFlags - open mode flags. 
*                          O_NONBLOCK mode is not supported.
*                          ther flags are ignored
* Range Issues:   None
* Special Issues: For executive loop only 
*****************************************************************************/
handle_t flashPmemOpen(const char * pName, int OFlags, ...)
{
   // assert(!((UWord16)OFlags & O_NONBLOCK)); /* NonBlocking Mode is not supported for Flash Driver */

    mainFlashConfig();
    PmemFlash.flags = OFlags;
    return (handle_t)&PmemFlash;
}

handle_t flashXmemOpen(const char * pName, int OFlags, ...)
{
   // assert(!((UWord16)OFlags & O_NONBLOCK)); /* NonBlocking Mode is not supported for Flash Driver */

    mainFlashConfig();
    XmemFlash.flags = OFlags;
    return (handle_t)&XmemFlash;
}

handle_t flashBootOpen(const char * pName, int OFlags, ...)
{
   // assert(!((UWord16)OFlags & O_NONBLOCK)); /* NonBlocking Mode is not supported for Flash Driver */
    mainFlashConfig();
    BootFlash.flags = OFlags;
    return (handle_t)&BootFlash;
}




/*****************************************************************************
* Module:         flashClose()
* Description:    Close specified Flash device
* Returns:        0 
* Arguments:      pHangle - device descriptor
* Range Issues:   None
* Special Issues:  For executive loop only, Just change device state to 0.
*****************************************************************************/
int flashClose(handle_t FileDesc)
{
   return 0;
}




/*****************************************************************************
* Module:         UpdatePageContex
* Description:    get length for next operation
* Returns:        available length for operation
* Arguments:      pHangle - device descriptor, n_bytes - user buffer size
* Range Issues:   
*  ***|********|********|********|********|********|**      
* Special Issues:  optimize on asm in future
*****************************************************************************/
unsigned int UpdatePageContex( handle_t hndl, size_t n_bytes )
{
    unsigned int res_len;

    res_len = 0;
    ((struct sflashDesc*)hndl)->tpage = ((struct sflashDesc*)hndl)->seek >> ((struct sflashDesc*)hndl)->pagesizelog; 

    if( ((struct sflashDesc*)hndl)->tpage < ((struct sflashDesc*)hndl)->lastpage )
    {
        res_len = 1 << ((struct sflashDesc*)hndl)->pagesizelog;
        
        ((struct sflashDesc*)hndl)->toffs = 
              ( res_len - 1 ) & ((struct sflashDesc*)hndl)->seek ;

        res_len = res_len - ((struct sflashDesc*)hndl)->toffs;

        if( n_bytes < res_len )
        {
            res_len = n_bytes;
        }
    }
    return res_len;
}




/*****************************************************************************
*
* Module:         flashRead()
*
* Description:    Read or verify flash context,. 
* Returns:        Actual processed size
* Arguments:      pHandle - device descriptor
*                 pBuffer - pointer to users buffer
*                 nbytes - User buffer size
* Special Issues:  For executive loop only. 
*****************************************************************************/
ssize_t flashRead(handle_t hndl, void * pBuffer, size_t nbytes)
{
    unsigned int len;
    ssize_t num = 0;
    
    do{
        len = UpdatePageContex(hndl, nbytes);    /* get new lenght */
        if( len == 0) 
            break;
        /* raw read from flash, 1 - read offset */
        ((struct sflashDesc*)hndl)->rawRead( ((struct sflashDesc*)hndl)->tpage ); 
        if( ((struct sflashDesc*)hndl)->flags & O_READ_COMPARE )
        {
            pBuffer = compareWithBuf( ((struct sflashDesc*)hndl)->toffs,
                                    (char*)pBuffer, len );
            if( pBuffer == 0 )
                return -1;      /* !!! seek unpredictable */
        }
        else
        {
            pBuffer = copyFromBuf( ((struct sflashDesc*)hndl)->toffs,
                                    (char*)pBuffer, len );
        }
        num += len;
        ((struct sflashDesc*)hndl)->seek += len;
    }while( nbytes -= len );

    return num;
}
    

/*****************************************************************************
*
* Module:         flashWrite()
*
* Description:    Write data from user buffer into flash. 
*
*                 Determine page to write, 
*                 Check is it empty page or not
*                 If not empty, 
*                    save data from this page into driver buffer and erase page                    
*                 Merge saved data and data from user buffer
*                 Program data from driver buffer into flash page
*                 Continue while end of user buffer
*
* Returns:        Actual processed size
*
* Arguments:      pHandle - device descriptor
*                 pBuffer - pointer to users buffer
*                 Size - User buffer size
*        
* Special Issues:  For executive loop only
*****************************************************************************/
ssize_t flashWrite(handle_t hndl, const void * pBuffer, size_t n_bytes)
{
    unsigned int len;
    ssize_t num = 0;
    
    do{
        len = UpdatePageContex(hndl, n_bytes);    /* get new lenght */
        if( len == 0) 
            break;
        /* raw read from flash */
        ((struct sflashDesc*)hndl)->rawRead( ((struct sflashDesc*)hndl)->tpage ); 
        pBuffer = copyToBuf( ((struct sflashDesc*)hndl)->toffs, (char*)pBuffer, len );
//        ((struct sflashDesc*)hndl)->rawErase( ((struct sflashDesc*)hndl)->tpage ); 
        ((struct sflashDesc*)hndl)->rawWrite( ((struct sflashDesc*)hndl)->tpage ); 
        num += len;
        ((struct sflashDesc*)hndl)->seek += len;
    }while( n_bytes -= len);

    return num;
}
    



/*****************************************************************************
* Module:         ioctlFLASH_CMD_SEEK()
* Description:    Function to change device modes.
* Returns:        0
* Arguments:      pHandle  - device descriptor
*                 pParams  - pointer to optional command parameters
* Range Issues:   Casting of long to pointer, impact to P flash  
* Special Issues:  Looking for decision for long value 
*****************************************************************************/
UWord16 ioctlFLASH_CMD_SEEK(handle_t hndl, unsigned long seek )
{
    ((struct sflashDesc*)hndl)->seek = seek;
    return 0;
}

UWord16 ioctlFLASH_RESET(handle_t hndl, unsigned long params )
{
    ((struct sflashDesc*)hndl)->seek = 0;
    return 0;
}

UWord16 ioctlFLASH_MODE_VERIFY(handle_t hndl, unsigned long params )
{

}

#if 0
/*****************************************************************************/

UWord16 ioctlFLASH_ERASE_PMEM(handle_t hndl, unsigned long params )
{
//    ioctlFLASH_RAW_ERASE_PMEM(((struct sflashDesc*)hndl)->page );
}
UWord16 ioctlFLASH_ERASE_XMEM(handle_t hndl, unsigned long params )
{
//    ioctlFLASH_RAW_ERASE_XMEM(((struct sflashDesc*)hndl)->page );
}
UWord16 ioctlFLASH_ERASE_BOOT(handle_t hndl, unsigned long params )
{
//    ioctlFLASH_RAW_ERASE_BOOT(((struct sflashDesc*)hndl)->page );
}

UWord16 ioctlFLASH_READ_PMEM(handle_t hndl, unsigned long params )
{
//    ioctlFLASH_RAW_READ_PMEM(((struct sflashDesc*)hndl)->page );
}
UWord16 ioctlFLASH_READ_XMEM(handle_t hndl, unsigned long params )
{
//    ioctlFLASH_RAW_READ_XMEM(((struct sflashDesc*)hndl)->page );
}
UWord16 ioctlFLASH_READ_BOOT(handle_t hndl, unsigned long params )
{
//    ioctlFLASH_RAW_READ_BOOT(((struct sflashDesc*)hndl)->page );
}

UWord16 ioctlFLASH_WRITE_PMEM(handle_t hndl, unsigned long params )
{
//    ioctlFLASH_RAW_WRITE_PMEM(((struct sflashDesc*)hndl)->page );
}
UWord16 ioctlFLASH_WRITE_XMEM(handle_t hndl, unsigned long params )
{
//    ioctlFLASH_RAW_WRITE_XMEM(((struct sflashDesc*)hndl)->page );
}
UWord16 ioctlFLASH_WRITE_BOOT(handle_t hndl, unsigned long params )
{
//    ioctlFLASH_RAW_WRITE_PMEM(((struct sflashDesc*)hndl)->page );
}


#endif