/******************************************************************************
*
*       COPYRIGHT 2003 MOTOROLA, ALL RIGHTS RESERVED
*
*       The code is the property of Motorola St.Petersburg Software Development
*       and is Motorola Confidential Proprietary Information.
*
*       The copyright notice above does not evidence any
*       actual or intended publication of such source code.
*
* Functions:    
*
* Description:     
*
* Notes:        
*
******************************************************************************/

#include "bsp.h"
#include "periph.h"

#include "flexcan.h"
#include "stdarg.h"

can_sBuffer IOBuffer[16];  /* supported buffers */

can_sFD      CanFDs;

int nBottomRead = 16; /* bottom pointer */
int nTopWrite = 0;
int CANErrno;
unsigned int nBusyFlags = 0;

extern const unsigned int FlexCANFlags;  /* FlexCAN configuration word */
extern const io_sFLEXCANInterface canInterfaceVT;
extern const can_pRawCallBack CANRcvPrepare;  /*  RAW CallBack pointer */

/*******************************************************************************
*
* NAME:             FC_Init
*
* DESCRIPTION:      Initialize CAN driver device 
*              
********************************************************************************
* PARAMETERS:       
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   device initialization.
*
* DEPENDENCIES:     None
*******************************************************************************/
void FC_Init( )
{       
    unsigned int i, j;
    extern const can_sInitialState canInitialState;

    periphMemWrite(0x0200,  &ArchIO.CAN.ConfigReg );
    i = 0;
    periphMemWrite(15,  &ArchIO.CAN.MaxMBReg );

    periphMemWrite( canInitialState.regFCMCR,  &ArchIO.CAN.ConfigReg );
    periphMemWrite( canInitialState.regFCCTL0, &ArchIO.CAN.Control0Reg );
    periphMemWrite( canInitialState.regFCCTL1, &ArchIO.CAN.Control1Reg );

#if 0 /* default mask state is OK */
    periphMemWrite( 0xFFE7, &ArchIO.CAN.RxGlobalMaskHiReg );
    periphMemWrite( 0xFFFE, &ArchIO.CAN.RxGlobalMaskLoReg );
    periphMemWrite( 0xFFE7, &ArchIO.CAN.Rx14MaskHiReg );
    periphMemWrite( 0xFFFE, &ArchIO.CAN.Rx14MaskLoReg );
    periphMemWrite( 0xFFE7, &ArchIO.CAN.Rx15MaskHiReg );
    periphMemWrite( 0xFFFE, &ArchIO.CAN.Rx15MaskLoReg );
#endif
    periphMemWrite(0,   &ArchIO.CAN.IntMask1Reg );

    for( ; i < 16; i++ ) 
    {
        /* reset all elements in all Message Buffers Control Register */
        periphMemWrite( FC_MB_NOTREADY, &ArchIO.CAN.Buffer[i].CtlStatusReg );
    }

    periphBitClear( 0xF000, &ArchIO.CAN.ConfigReg);

}

/*******************************************************************************
*
* NAME: CheckCANID
*
* DESCRIPTION: check CAN ID to protect from reopen
*              
********************************************************************************
* PARAMETERS:   CAN ID, mode for loopback
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
int CheckCANID( CANADDRESSTYPE canID, unsigned int mode )
{
    register int i;

    if( !( mode & (O_LOOP_BACK) >> 1 )   )    
        for( i = nBottomRead;  i  < 16; i++ )
    {
        if( IOBuffer[i].canID == canID)  /* checking receive buffers */
        {
            CANErrno = CAN_ERR_REOPEN;
            return 0;  
        }
    }
    
    if( !( mode & (O_LOOP_BACK)  )   )    
        for( i = 0;  i < nTopWrite; i++ )
    {
        if( IOBuffer[i].canID == canID ) /* checking send buffers */
        {
            CANErrno = CAN_ERR_REOPEN;
            return 0;  /* error: re-open is not possible */
        }
    }

    return 1;

}


/*******************************************************************************
*
* NAME: SetCANID
*
* DESCRIPTION: Decompose 11/29-bit CAN ID into the registers 
*
********************************************************************************
* PARAMETERS:   Address of the registers, CAN ID
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
void SetCANID( volatile unsigned short* pAddr, CANADDRESSTYPE canID, unsigned long addr_flags )
{
    asm( brclr #CAN_SHORT_IDE_MASK, B1, can2a );
    asm( asl  A );

    asm( move.w A1, X0 );
    asm( bfclr #0xFFF8, X0 );  /* ID[18-17] */
    asm( or.w X0, B );

    asm( move.w A1, X0 );     /* ID[29-18] */
    asm( asll.w  #0x2, X0 );
    asm( bfclr #0x001F, X0 );
    asm( or.w X0, B );

    asm( move.w  B1, X:(R2)+ );
    asm( move.w  A0, X:(R2)+ );

    asm( rts );

can2a:
    asm( asll.l  #0x5, A );
    asm( move.w  A0, X:(R2)+ );
    asm( clr.w   X:(R2)+ );

}

/*******************************************************************************
*
* NAME: GetCANID
*
* DESCRIPTION: Compose 11/29-bit CAN ID from the registers values
*
********************************************************************************
* PARAMETERS:   Raw ID 
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
CANADDRESSTYPE GetCANID( unsigned long id )
{
    asm( brclr # CAN_SHORT_IDE_MASK, A1, can2a );

    asm( bfclr    # (CAN_SHORT_IDE_MASK|CAN_SHORT_SRR_MASK), A1);  /* clear srr, ide */
    asm( brclr    #0x0002, A1, Bit_16 ); /* bit 16th of id */
    asm( bfset    #CAN_SHORT_IDE_MASK, A1);
Bit_16: 
    asm( brclr    # 0x0004, A1, Bit_17 );  /* bit 17th of id */
    asm( bfset    # CAN_SHORT_SRR_MASK, A1);
Bit_17: 
    asm( asr  A );  /* bit_15 -> A0  */
    asm( move.w A0, B0 );
    asm( lsrr.w  #2, A ); /* skip bit 16,17 */
    asm( move.w B0, A0 );
    asm( rts );
can2a:
    asm( lsrr.l  #21, A );
}


/*******************************************************************************
*
* NAME: PrepareCANID
*
* DESCRIPTION:  prepare buffer for reading or write
*              
********************************************************************************
* PARAMETERS:       buffer descriptor
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:  
*
* DEPENDENCIES:     None
*******************************************************************************/
void PrepareCANID( can_sBuffer* pbuf, unsigned int cmd )
{
    asm( move.w X:(R2 + #sFlexCAN_offset_flags), X0 );
    asm( move.w  X0, Y1 );
    asm( bfclr #0xFFF0, X0 );    /* clear flags */
    asm( asll.w  #0x3, X0 ); /* x8, convert to buffer index */

    asm( adda #( BSP_PERIPH_BASE + archoff_FlexCAN_Buffer_0 ), X0, R4 ); 
    
    asm( move.w #FC_MB_NOTACTIVE, X:(R4 + devoff_FlexCANBuffer_CtlStatusReg) );

    asm( move.l  X:( R2 + sFlexCAN_offset_canID), A );
    asm( adda   # devoff_FlexCANBuffer_IdRegHi, R4, R2 );

    asm( clr B );   /* form flags on B register */
    asm( brset    # CAN_BUF_READ_HANDLE, Y1, Bit_17 ); 
    asm( bfset    # CAN_SHORT_SRR_MASK, B1);  /* only for write handle */
Bit_17: 
    asm( brclr    # CAN_BUF_2B, Y1, Bit_16 ); 
    asm( bfset    #CAN_SHORT_IDE_MASK, B1); /* only for long handles */
Bit_16: 

    asm( jsr SetCANID ); /* used A,B, X0 */

    asm( move.w Y0, X:(R4 + devoff_FlexCANBuffer_CtlStatusReg) );
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_TimerReg), X0 );
}


/*******************************************************************************
*
* NAME: flexcanOpen
*
* DESCRIPTION: driver Low Open procedure: initialize buffer
*              
********************************************************************************
* PARAMETERS:   device name, open flags, CAN ID
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
handle_t  flexcanOpen(const char * pName, int oFlags,  CANADDRESSTYPE canID )
{
    can_sBuffer* pbufRx;
    can_sBuffer* pbufTx;
    unsigned int cmdRx;
    unsigned int cmdTx;


    nBottomRead--; 
    pbufRx = &IOBuffer[nBottomRead];
    pbufRx->flags = (unsigned int)nBottomRead | CAN_BUF_READ_HANDLE;
    cmdRx = FC_MB_ACTIVE;
    periphBitSet( (1 << nBottomRead),  &ArchIO.CAN.IntMask1Reg );

    pbufTx = &IOBuffer[nTopWrite]; /* CAN_BUF_WRITE_HANDLE */
    pbufTx->flags = (unsigned int)nTopWrite;
    cmdTx = FC_MB_NOTREADY;
    nTopWrite++;  /* next buffer */
    
    if(!(oFlags & O_NONBLOCK))
    {
	  pbufRx->flags |= CAN_BUF_SYNCHRONOUS; 
	  pbufTx->flags |= CAN_BUF_SYNCHRONOUS;
    }
    
    if(FlexCANFlags & O_CAN_2B)
    {
        pbufRx->flags |= CAN_BUF_2B;
        pbufTx->flags |= CAN_BUF_2B;
    }
    
    pbufRx->canID = 1; 
    pbufTx->canID = 2; 

    PrepareCANID(pbufRx, cmdRx);
    PrepareCANID(pbufTx, cmdTx);

	CanFDs.RxFD = (handle_t)pbufRx;
	CanFDs.TxFD = (handle_t)pbufTx;

    return (handle_t)&CanFDs; 
}


/*******************************************************************************
*
* NAME: Write
*
* DESCRIPTION: Driver write procedure
*              
********************************************************************************
* PARAMETERS:   file descriptor, buffer and size
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: handler is pointer to the buffer, 
*                 truncation is possible when porting
*
* DEPENDENCIES: None 
*******************************************************************************/
ssize_t flexcanWrite(handle_t handle,  const void *pBuffer, size_t size)
{
	int   i    = ((can_sBuffer*)((can_sFD *)handle)->TxFD)->flags & 0x000F;    
	char* pSrc = (char*)pBuffer;
    int   dummy_int = 0;
    char  TempData[8];
    int   j;
   	int   Iterations  = size / 8;
	int   NumberToPad = ((Iterations + 1) * 8) - size;
	int   Index       = 0;
    
    volatile  arch_sFlexCANBuffer * mb = &ArchIO.CAN.Buffer[i];
	
	if(size <= 8)
	{
		for(j = 0; j < size; j++)
		{
			TempData[j] = pSrc[j];
		}

		for(j = size; j < 8; j++)
		{
			TempData[j] = 0;
		}
		
	    periphMemWrite(( TempData[0] << 8 )| TempData[1], &mb->DataSegReg[0] );
	    periphMemWrite(( TempData[2] << 8 )| TempData[3], &mb->DataSegReg[1] );
	    periphMemWrite(( TempData[4] << 8 )| TempData[5], &mb->DataSegReg[2] );
	    periphMemWrite(( TempData[6] << 8 )| TempData[7], &mb->DataSegReg[3] );

	    periphBitClear( 0x000F, &mb->CtlStatusReg ); 
	    periphBitSet( (FC_MB_SEND | size), &mb->CtlStatusReg );
	    dummy_int = periphMemRead(&ArchIO.CAN.TimerReg);     
		
		return size;      
	}
	else
	{
		for(i = 0; i < Iterations; i++)
		{	
			for(j = 0; j < 8; j++)
			{
				TempData[j] = pSrc[Index];
				Index += 1;
			}
			
		    periphMemWrite(( TempData[0] << 8 )| TempData[1], &mb->DataSegReg[0] );
		    periphMemWrite(( TempData[2] << 8 )| TempData[3], &mb->DataSegReg[1] );
		    periphMemWrite(( TempData[4] << 8 )| TempData[5], &mb->DataSegReg[2] );
		    periphMemWrite(( TempData[6] << 8 )| TempData[7], &mb->DataSegReg[3] );

		    periphBitClear( 0x000F, &mb->CtlStatusReg ); 
		    periphBitSet( (FC_MB_SEND | 8), &mb->CtlStatusReg );
		    dummy_int = periphMemRead(&ArchIO.CAN.TimerReg);  
		
			FlexCanWriteDelay();
		}
		
		FlexCanWriteDelay();
		
		for(j = 0; j < (size - (Iterations * 8)); j++)
		{
			TempData[j] = pSrc[Index];
			Index += 1;
		}
	
		for(j = (size - (Iterations * 8)); j < 8; j++)
		{
			TempData[j] = 0;
		}
		
	    periphMemWrite(( TempData[0] << 8 )| TempData[1], &mb->DataSegReg[0] );
	    periphMemWrite(( TempData[2] << 8 )| TempData[3], &mb->DataSegReg[1] );
	    periphMemWrite(( TempData[4] << 8 )| TempData[5], &mb->DataSegReg[2] );
	    periphMemWrite(( TempData[6] << 8 )| TempData[7], &mb->DataSegReg[3] );

	    periphBitClear( 0x000F, &mb->CtlStatusReg ); 
	    periphBitSet( (FC_MB_SEND | (8-NumberToPad)), &mb->CtlStatusReg );
	    dummy_int = periphMemRead(&ArchIO.CAN.TimerReg);  
		
		return size; 
	}
}

/*******************************************************************************
*
* NAME: Read
*
* DESCRIPTION: Driver read procedure
*              
********************************************************************************
* PARAMETERS:   file descriptor, buffer and size
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: handler is pointer to the CAN ID descriptor, 
*               
* DEPENDENCIES: None
*******************************************************************************/
UInt8          CANInputBuffer[8];
UWord16        CANDataLeft;
UWord16        CANDataIndex; 

UWord16 GetCANData(handle_t handle);

UWord16 GetCANData(handle_t handle)
{
	int    Temp1 = 0;
	int    Temp2 = 0;
	int    i     = ((can_sBuffer*)((can_sFD *)handle)->RxFD)->flags & 0x000F;
	volatile arch_sFlexCANBuffer * mb = &ArchIO.CAN.Buffer[i];
	
	Temp1 = periphMemRead(&mb->CtlStatusReg);
	Temp2 = Temp1;
	
	Temp1 = Temp1 & 0x00F0;
	Temp2 = Temp2 & 0x000F;

	if((Temp1 == FC_MB_FULL) || (Temp1 == FC_MB_OVERRUN))
	{
		Temp1 = Temp2;
		
		if(Temp2 != 0)
		{
			CANInputBuffer[0] = periphMemRead(&mb->DataSegReg[0]) >> 8;
			Temp2 -= 1;
		}
		if(Temp2 != 0)
		{
			CANInputBuffer[1] = periphMemRead(&mb->DataSegReg[0]) & 0x00FF;
			Temp2 -= 1;
		}
		if(Temp2 != 0)
		{
			CANInputBuffer[2] = periphMemRead(&mb->DataSegReg[1]) >> 8;
			Temp2 -= 1;
		}
		if(Temp2 != 0)
		{
			CANInputBuffer[3] = periphMemRead(&mb->DataSegReg[1]) & 0x00FF;
			Temp2 -= 1;
		}
		if(Temp2 != 0)
		{
			CANInputBuffer[4] = periphMemRead(&mb->DataSegReg[2]) >> 8;
			Temp2 -= 1;
		}			
		if(Temp2 != 0)
		{
			CANInputBuffer[5] = periphMemRead(&mb->DataSegReg[2]) & 0x00FF;
			Temp2 -= 1;
		}			
		if(Temp2 != 0)
		{
			CANInputBuffer[6] = periphMemRead(&mb->DataSegReg[3]) >> 8;
			Temp2 -= 1;
		}			
		if(Temp2 != 0)
		{
			CANInputBuffer[7] = periphMemRead(&mb->DataSegReg[3]) & 0x00FF;
		}		
 
 		periphMemWrite(0x8000, &ArchIO.CAN.IntFlags1Reg);
		periphMemWrite(0x8000, &ArchIO.CAN.IntMask1Reg);

		periphBitSet(FC_MB_ACTIVE, &mb->CtlStatusReg);
		Temp2 = periphMemRead( &ArchIO.CAN.TimerReg);   
 		
 		return Temp1;
	}
	else
	{
		return 0;	
	}
}

ssize_t flexcanRead(handle_t handle,  void *pBuffer, size_t Size)
{
	char * pDst        = (char*)pBuffer;
	int    ReturnValue = Size;
	
	while(Size != 0)
	{
		if(CANDataLeft == 0)
		{
			while(ioctlCANID_GET_STATUS(handle, 0) == 0)
			{
			}
		
			CANDataLeft  = GetCANData(handle);
			CANDataIndex = 0;
		}
        
		*pDst = CANInputBuffer[CANDataIndex++];
		
		pDst        += 1;
		CANDataLeft -= 1;
		Size        -= 1;
    }
	
	return ReturnValue;
}

/*******************************************************************************
*
* NAME: ioctlCANID_GET_STATUS
*
* DESCRIPTION: Driver i/o control procedure - Gets status of message buffer
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   returned status value can be one of the following:
*                           CANID_EMPTY - no messages in the message buffer
*                           CANID_FULL - a message is in the message buffer
*                           CANID_OVERFLOW - a  message buffer is full
*                                      but some received messages were lost 
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCANID_GET_STATUS(handle_t handle, unsigned long params )      
{
    asm(move.w X:(R2), X0);
    asm(move.w X0, R2);
        
    asm( move.w X:(R2 + #sFlexCAN_offset_flags), X0 );
    asm( bfclr #0xFFF0, X0 );    /* clear flags */
    asm( move.w    #1, B );
    asm( asll.w    X0, B );      /* get mask for buffer */

    asm( move.w # CANID_EMPTY,  Y0 ); /* default status */
    asm( brclr CAN_BUF_READ_HANDLE, X:(R2 + #sFlexCAN_offset_flags), tx_case );

    asm( asll.w  #0x3, X0 ); /* x8, convert to buffer index */
    asm( adda #( BSP_PERIPH_BASE + archoff_FlexCAN_Buffer_0 ), X0, R4 ); 
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntFlags1Reg ), X0 );

    asm( and.w X0, B );         /* check bit in flag register */
    asm( beq end_status );      /* empty buffer if was no reception */

    asm( move.w X:(R4 + devoff_FlexCANBuffer_CtlStatusReg), Y1 );
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_TimerReg), X0 );
    asm( bfclr #0xFF0F, Y1 );    /* left only raw status */

    asm( move.w # CANID_FULL, Y0 );
    asm( cmp.w # FC_MB_FULL,  Y1 );
    asm( beq end_status );

    asm( move.w # CANID_OVERFLOW,  Y0 );
    asm( cmp.w  # FC_MB_OVERRUN,  Y1 ); 
    asm( beq end_status );

    asm( move.w # CANID_EMPTY,  Y0 ); /* for case of interim code */
    asm( bra end_status );

tx_case:
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ), X0 );
    asm( and.w X0, B );    /* check bit in flag register */
    asm( beq end_status );
    asm( move.w # CANID_FULL ,  Y0 );

end_status:
    ;
}


/*******************************************************************************
*
* NAME: ioctlCAN_RESET
*
* DESCRIPTION: Driver i/o control procedure - Reset CAN Driver
*              
********************************************************************************
* PARAMETERS:       file descriptor,  parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:  
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCAN_RESET(handle_t FileDesc, unsigned long params )
{
    register int i;

    FC_Init();

    for( i = nBottomRead;  i  < 16; i++ )
    {
         /* MB is active for receive data */
        periphMemWrite( FC_MB_ACTIVE,   &ArchIO.CAN.Buffer[i].CtlStatusReg );   
    }

    return 0;
}


/*******************************************************************************
*
* NAME: ioctlCAN_SET_SLEEP
*
* DESCRIPTION: Driver i/o control  - Sets CAN into SLEEP(power save) mode
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCAN_SET_SLEEP(handle_t FileDesc, unsigned long params )
{
    periphBitSet(CAN_STOP, &ArchIO.CAN.ConfigReg ); /* goto low power sleep mode */
    return (unsigned int)periphBitTest( CAN_STOP_ACK, &ArchIO.CAN.ConfigReg ) ;
}

/*******************************************************************************
*
* NAME: ioctlCAN_SET_WAKEUP
*
* DESCRIPTION: Driver i/o control procedure - Wakeups CAN
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCAN_SET_WAKEUP(handle_t FileDesc, unsigned long params )
{
    periphBitSet(CAN_SELF_WAKE, &ArchIO.CAN.ConfigReg ); /* goto wake up mode */
    return 0;
}

/*******************************************************************************
*
* NAME: ioctlCAN_GET_STATUS
*
* DESCRIPTION: Driver i/o control procedure - Gets status of CAN control registers
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   returned status value is composed of CANCTL0 + CANRFLG
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCAN_GET_STATUS(handle_t FileDesc, unsigned long params )
{
    unsigned int status;

    status = periphMemRead( &ArchIO.CAN.StatusReg );
    if( !(status & 0xFC00) )
        status |= 0x0008;

    return status;
}

/*******************************************************************************
*
* NAME:             Close
*
* DESCRIPTION:      Driver close procedure
*              
********************************************************************************
* PARAMETERS:       file descriptor
*
* SIDE EFFECTS:     filters is staying open
*
* DESIGNER NOTES:   close corresponding message buffer
*
* DEPENDENCIES:     None
*******************************************************************************/
int flexcanClose(handle_t handle)
{
	unsigned int i = ((can_sBuffer*)((can_sFD *)handle)->RxFD)->flags & 0x000F;	
	unsigned int j = ((can_sBuffer*)((can_sFD *)handle)->TxFD)->flags & 0x000F;	

	if(i == nBottomRead)
	{
	    nBottomRead++;
	    ((can_sBuffer*)((can_sFD *)handle)->RxFD)->flags = 0;
	}
	else
	{
	    return -1;
	}

	if(j == nTopWrite - 1)
	{
	    nTopWrite--;
	    ((can_sBuffer*)((can_sFD *)handle)->TxFD)->flags = 0;
	}
	else
	{
	    return -1;
	}

	periphMemWrite(0, &ArchIO.CAN.Buffer[i].CtlStatusReg);
	periphBitClear((1 << i),  &ArchIO.CAN.IntMask1Reg);
	periphBitSet  ((1 << i),  &ArchIO.CAN.IntFlags1Reg);

	return 0;
}


/*******************************************************************************
*
* NAME: ioctlCANID_SET_MSAK
*
* DESCRIPTION: update mask according to the buffer
*              
********************************************************************************
* PARAMETERS:       file descriptor, mask
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   only for 14 and 15 mask
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCANID_SET_MASK(handle_t handle, unsigned long params )      
{
    unsigned int i;
    unsigned long maskflag;

    i = ((can_sBuffer*)handle)->flags & 0x000F;
	maskflag = 0;
	if( ((can_sBuffer*)handle)->flags & CAN_BUF_2B )
	{
	    maskflag =   CAN_LONG_IDE_MASK ;
    }

    if( i == 15 )
    {
        SetCANID( (unsigned short*)(BSP_PERIPH_BASE + archoff_FlexCAN_Rx15MaskHiReg ), params, maskflag );
    }
    else if( i == 14 )
    {
        SetCANID( (unsigned short*)(BSP_PERIPH_BASE + archoff_FlexCAN_Rx14MaskHiReg ), params, maskflag );
    }
    else
    {
        SetCANID( (unsigned short*)(BSP_PERIPH_BASE + archoff_FlexCAN_RxGlobalMaskHiReg ), params, maskflag );
    }
    return 0;
}


/*******************************************************************************
*
* NAME: CANReceiveISR
*
* DESCRIPTION: receive ISR handler
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: Slowest handler in the driver, impacts to the interrupt lattency
*
* DESIGNER NOTES: copy is not effective on C, recomended to use asm
*
* DEPENDENCIES: None
*******************************************************************************/
#pragma interrupt called on
void CANReceiveISR(void)
{
    register  can_sData* pCrd;

    pCrd = (*CANRcvPrepare)( 
                GetCANID(
                        ((unsigned long)periphMemRead( &ArchIO.CAN.Buffer[15].IdRegHi)) << 16
                        | periphMemRead( &ArchIO.CAN.Buffer[15].IdRegLo)
                )   
           ); /* CANStdRcvPrepare */
    if( pCrd ) 
    {   
        pCrd->length =  flexcanRead( (handle_t)&IOBuffer[15], 
                        &pCrd->data[0], 8 );
    }
    else 
    {   /* just release */
         flexcanRead( (handle_t)&IOBuffer[15], &pCrd->data[0], 0 ); 
    }
}
#pragma interrupt called off

/*******************************************************************************
*
* NAME:             FC_InitCallBack
*
* DESCRIPTION:      Initialize CAN driver device for callback
*              
********************************************************************************
* PARAMETERS:       
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   only 15th buffer is used
*
* DEPENDENCIES:     None
*******************************************************************************/
void FC_InitCallBack( CANADDRESSTYPE mask, CANADDRESSTYPE canID )
{
    flexcanOpen( 0, O_RDONLY, canID ); /* use last buffer with own filter */
    ioctlCANID_SET_MASK((handle_t)&IOBuffer[15], mask );
    periphMemWrite( 0x8000, &ArchIO.CAN.IntMask1Reg ); /* 15th buffer */
    IOBuffer[15].callback = CANReceiveISR;
}


/*******************************************************************************
*
* NAME: CANReceiveISR
*
* DESCRIPTION: receive ISR handler
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: Slowest handler in the driver, impacts to the interrupt lattency
*
* DESIGNER NOTES: copy is not effective on C, recomended to use asm
*
* DEPENDENCIES: None
*******************************************************************************/
void CANBufferISR(void)
{
    asm( alignsp );	/* saves needed registers only */
    asm( move.l  Y , x:(SP)+ );
    asm( move.l  X0, x:(SP)+ );
    asm( move.l  R2, x:(SP)+ );
    asm( move.l  N, x:(SP) );

    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ), Y0 );
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntFlags1Reg ), Y1 );
    asm( and.w  Y1, Y0 );           /* actual isr vector */
    asm( clr.w Y1 );
    asm( clb  Y, X0 );              /* left buffer index (Y0) */
    asm( move.w #0x1E, Y0 );
    asm( sub  X0, Y0 );      
    asm( blt end_bufisr );
    asm( move.w    #0x0001, X0 );   /* 0th mask (X0) */
    asm( asll.w    Y0, X0 );        /* actual buffer mask (X0) */
   
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ), Y1 );
    asm( not.w X0 );       /* to clear actual mask */
    asm( and.w X0, Y1 );   /* only current ISR mask clear */
    asm( move.w Y1, X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ) );   /* release mask */

    asm( move.w #sFlexCAN_word_size, Y1 ); /* get status */
    asm( impy.w Y1, Y0, X0 );          /* Y0*Y1 get contex offset */

    asm( adda #IOBuffer, X0, R2 );  /* get context buffer address */
    asm( tst.l X:(R2 + sFlexCAN_offset_callback) );
    asm( beq end_bufisr );
    asm( move.l X:(R2 + sFlexCAN_offset_callback), N ); /* get callback address */
    asm( jsr (N) );
end_bufisr:

    asm( move.l  x:(SP)-, N );
    asm( move.l  x:(SP)-,R2 );
    asm( move.l  x:(SP)-,X0 );
    asm( move.l  x:(SP)-,Y );
    asm( move.l  x:(SP)-,SP );

    asm(  rti );
}


void FlexCanWriteDelay(void)
{
	int i;
	
	for(i=0; i < 0x03FF; i++)
	{
		asm(nop);
		asm(nop);
		asm(nop);
		asm(nop);
	}
}

