/*******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
*
* Description:       source file for the 56838 SCI device driver 
*
* Notes:  
*
******************************************************************************/

#include <stdarg.h>

#include "bsp.h"
//#include "syslib.h"
#include "periph.h"
#include "scidrv.h"
#include "arch_off.h"
#include "gpioinline.h"

/*****************************************************************************/
/*                         Driver Function prototypes                        */
/*****************************************************************************/

#define scidrvHWDisableInterrupts(BaseAddress)      (periphBitClear(SCI_SCICR_TEIE | SCI_SCICR_TIIE | SCI_SCICR_RIE | SCI_SCICR_REIE, &BaseAddress->ControlReg))
#define scidrvHWEnableRxInterrupts(BaseAddress)     (periphBitSet(SCI_SCICR_RIE | SCI_SCICR_REIE, &BaseAddress->ControlReg))
#define scidrvHWDisableRxInterrupts(BaseAddress)    (periphBitClear(SCI_SCICR_RIE | SCI_SCICR_REIE, &BaseAddress->ControlReg))
#define scidrvHWEnableTxCompleteInterrupt(BaseAddress) (periphBitSet(SCI_SCICR_TIIE, &BaseAddress->ControlReg))
#define scidrvHWEnableTxReadyInterrupt(BaseAddress) (periphBitSet(SCI_SCICR_TEIE, &BaseAddress->ControlReg))
#define scidrvHWDisableTxInterrupts(BaseAddress)    (periphBitClear(SCI_SCICR_TEIE | SCI_SCICR_TIIE, &BaseAddress->ControlReg))
#define scidrvHWDisableTx(BaseAddress)              (periphBitClear(SCI_SCICR_TE, &BaseAddress->ControlReg))
#define scidrvHWDisableRx(BaseAddress)              (periphBitClear(SCI_SCICR_RE, &BaseAddress->ControlReg))
#define scidrvHWDisableDevice(BaseAddress)          (periphBitClear(SCI_SCICR_TE | SCI_SCICR_RE, &BaseAddress->ControlReg))
#define scidrvHWEnableDevice(BaseAddress)           (periphBitSet(SCI_SCICR_TE | SCI_SCICR_RE, &BaseAddress->ControlReg))

/*****************************************************************************/
/*                                 API                                       */
/*****************************************************************************/

handle_t sciPreOpen( const char * pName, struct io_sDevice* pDev, int OFlags , ... )
{
    struct  sSciDevice    * pSciDevice;
    unsigned long   BaudRate;
    unsigned long   rawBaudRate;
    va_list         Args;
    
    Args = (char *)&OFlags;
    BaudRate = va_arg(Args, unsigned long);
    va_end(Args);

    switch ((int)pName)
    {
    case (int) BSP_DEVICE_NAME_SCI_0:
         pSciDevice = &Sci0DeviceContext;

         /* CONFIGURE SCI PINS TO BE PERIPHERAL */
        gpioIoctlGPIO_SETAS_PERIPHERAL( SCI0_GPIO_BASE_ADDRESS, SCI0_GPIO_MASK );
    break;
 
    case (int) BSP_DEVICE_NAME_SCI_1:
         pSciDevice = &Sci1DeviceContext;  

         /* CONFIGURE SCI PINS TO BE PERIPHERAL */
        gpioIoctlGPIO_SETAS_PERIPHERAL( SCI1_GPIO_BASE_ADDRESS, SCI1_GPIO_MASK );
    break;
 
    default:
        return (handle_t)-1;
    }
    scidrvHWDisableInterrupts( pSciDevice->Base );
    scidrvHWDisableDevice( pSciDevice->Base );

    if(OFlags & O_NONBLOCK)
    {
      extern const struct io_sSCIInterface scidrvIONonBlockInterfaceVT;
      pSciDevice->drvInterface = (io_sInterface *)&scidrvIONonBlockInterfaceVT;
    }
    else
    {
      extern const struct io_sSCIInterface scidrvIOBlockInterfaceVT;
      pSciDevice->drvInterface = (io_sInterface *)&scidrvIOBlockInterfaceVT;
    }


    rawBaudRate = sci_getBaudRate( BaudRate );

    return sciOpen( (const char*)pSciDevice,  OFlags, rawBaudRate );

}




/*****************************************************************************
*
* Module:         sciOpen
*
* Description:    Open SCI device.
*                 Initialize device context
*
* Returns:        Device handle if successful,
*                 -1 if device name is not supported
*
* Arguments:      pName - BSP SCI device name
*                 OFlags - open mode (for detail see sci.h file)
*                           O_NONBLOCK 
*                           O_BLOCK
*                           O_SCI_DISABLE_IN_WAIT   
*                           O_SCI_ENABLE_IN_WAIT    
*                           O_SCI_WAKE_BY_ADDRESS   
*                           O_SCI_WAKE_BY_IDLE      
*                           O_SCI_WORD_9BIT         
*                           O_SCI_WORD_8BIT         
*                           O_SCI_PARITY_NONE       
*                           O_SCI_PARITY_ODD        
*                           O_SCI_PARITY_EVEN       
*                           O_SCI_7_DATA_BITS_ODD_PARITY 
*                           O_SCI_7_DATA_BITS_EVEN_PARITY
*                           O_SCI_8_DATA_BITS_NO_PARITY  
*                           O_SCI_8_DATA_BITS_ODD_PARITY 
*                           O_SCI_8_DATA_BITS_EVEN_PARITY
*                           O_SCI_9_DATA_BITS_NO_PARITY  
*                           O_SCI_TX_INVERTED       
*                           O_SCI_TX_NOT_INVERTED   
*                           O_SCI_INVERT_ALL_BITS        
*                           O_SCI_DONT_INVERT_ANY_BITS   
*                           O_SCI_LOOPBACK_DISABLED 
*                           O_SCI_LOOPBACK_ENABLED  
*                           O_SCI_LOOPBACK_SINGLE_WIRE_ENABLED
*                           
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
handle_t sciOpen( const char * pName, int OFlags, unsigned long rawBaudRate )
{
    struct sSciDevice* pSciDevice = (struct sSciDevice*)pName;
    unsigned int    ControlWord;
    short           Control = 0;

    if(OFlags && 0xFFF0)
    {
        Control = OFlags & 0xFFF0;
    }

    if( OFlags & O_SCI_DATA_9BIT )
    {
        if( OFlags & (O_SCI_PARITY_ODD | O_SCI_PARITY_EVEN)  )
            pSciDevice->Mask = SCI_SCIDR_8BIT_MASK; /* 8 with parity */
        else
            pSciDevice->Mask = SCI_SCIDR_9BIT_MASK;
    }
    else  /* O_SCI_DATA_8BIT */
    {
        if( OFlags & (O_SCI_PARITY_ODD | O_SCI_PARITY_EVEN)  )
            pSciDevice->Mask = SCI_SCIDR_7BIT_MASK;
        else
            pSciDevice->Mask = SCI_SCIDR_8BIT_MASK; /* 8 without parity */
    }

    periphMemWrite( rawBaudRate, &pSciDevice->Base->BaudRateReg );
    periphMemWrite( Control & SCI_SCICR_USER_MASK, &pSciDevice->Base->ControlReg  );


    /* Clear all receiver related interrupts */
    periphMemRead( &pSciDevice->Base->StatusReg );
    periphMemRead( &pSciDevice->Base->DataReg );
    periphMemWrite( 0x0000, &pSciDevice->Base->StatusReg );

    scidrvHWEnableRxInterrupts( pSciDevice->Base );
    scidrvHWEnableDevice( pSciDevice->Base );

    pSciDevice->qReceive.begin  = pSciDevice->qReceive.end  = 0;
    pSciDevice->qSend.begin     = pSciDevice->qSend.end     = 0;

    return (handle_t)pSciDevice;
}

/*****************************************************************************
*
* Module:         sciClose()
*
* Description:    Close SCI device. Uninstall ISR vectors, if used.
*
* Returns:        0
*
* Arguments:      hndl - device context
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
int sciClose( handle_t hndl )
{ 
   struct   sSciDevice   * pHandle = (struct sSciDevice *)hndl;
   volatile unsigned int   status  = 0;
   
   scidrvHWDisableRx( pHandle->Base );
   
   while(1)
   {
       status = periphMemRead( &pHandle->Base->StatusReg );
       if( status & SCI_SCISR_TIDLE )
       {
           break;
       }
   }             
   
   scidrvHWDisableTx( pHandle->Base );
   
#if defined(BSP_DEVICE_NAME_SCI_0)
   /* return GPIO pins in default state */
   if( pHandle->Base == SCI0_BASE_ADDRESS )     
   {
      gpioIoctlGPIO_SETAS_GPIO( SCI0_GPIO_BASE_ADDRESS, SCI0_GPIO_MASK );
   }
#endif
#if defined(BSP_DEVICE_NAME_SCI_1)
   if( pHandle->Base == SCI1_BASE_ADDRESS )     
   {
      gpioIoctlGPIO_SETAS_GPIO( SCI1_GPIO_BASE_ADDRESS, SCI1_GPIO_MASK );
   }
#endif /* defined(BSP_DEVICE_NAME_SCI_1) */
   return 0;
}

/*****************************************************************************
*
* Module:         sciNonBlockRead()
*
* Description:    Read data from SCI driver into user buffer
*
*                 In NonBlocking mode, "read" returns the actual data that
*                 has already been received from the SCI device. It does not
*                 wait for more data, if NBytes is greater than data size
*                 already contained in the buffer.
*
* Returns:        Actual read size
*
* Arguments:      hndl        - SCI Device descriptor returned by "sciOpen" call.
*                 pBuffer     - Char pointer to user buffer. 
*                 NBytes      - NBytes (in 8-bit bytes) of the data to be read 
*                               from SCI device; 
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
ssize_t sciNonBlockRead( handle_t hndl, void * pBuffer, size_t NBytes )
{
    asm(move.w   M01, B );  /* save modulo context in B */
  
#if __m56800E_lmm__ 
    asm(move.w A0,A);
    asm(move.w  A1,Y0);
#else
	asm(nop);
	asm(nop);
#endif
    asm(move.w      Y0,A);  /* A := size */
    
    /* load queue descriptor */
    asm(move.l      X:(R2 + #(sSciDevice_offset_qReceive + sQueue_offset_begin) ), Y);  /* Y0 := begin, Y1 := end */
    /* use space in queue */
    asm(sub         Y1,Y0); /* Y0 := begin - end */
    asm(cmp.w       Y0,A);
    asm(tgt         Y0,A);  /* if ("buffer size" < "count of ready chars in queue") then copy only "buffer size" chars */
    /* setup modulo */
    asm(move.w      X:(R2 + #(sSciDevice_offset_qReceive + sQueue_offset_mask) ) , X0);
    asm(moveu.w     X0,M01);
    /* setup buffer pointer */
    asm(and.w       Y1,X0); /* X0 := moduled begin (Y0) */
    asm(add.w       X:(R2 + #(sSciDevice_offset_qReceive + sQueue_offset_bufferOffset) ) , X0);
    asm(adda        #BASE_CIRCULAR_BUFFER_ADDRESS,X0,R0);
    
    asm(do          A,end_reading_loop);
        asm(move.w      P:(R0)+,X0);
        asm(nop);
        asm(nop);
        asm(move.bp     X0,X:(R3)+);
    asm(end_reading_loop:);
    
    /* update queue descriptor */
    asm(add         A,Y1);
    asm(move.w      Y1,X:(R2 + #(sSciDevice_offset_qReceive + sQueue_offset_end) ) );
    
    asm(move.w      A,Y0);

#if __m56800E_lmm__ 
    asm(move.w A1,A0);
    asm(clr.w  A1);
#else
    asm(nop);
	asm(nop);
#endif

    asm(moveu.w  B, M01 );  /* restore modulo context */

    /* asm(rts); */
}

/*****************************************************************************
*
* Module:         sciBlockRead()
*
* Description:    Read data from SCI driver into user buffer
*
*                 In Blocking mode, "read" waits until all data will be
*                 available.
*
*
* Returns:        Actual read size
*
* Arguments:      hndl        - SCI Device descriptor returned by "sciOpen" call.
*                 pBuffer     - Char pointer to user buffer. 
*                 NBytes      - NBytes (in 8-bit bytes) of the data to be read 
*                               from SCI device; 
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
ssize_t sciBlockRead(handle_t hndl , void * pBuffer , size_t NBytes)
{
	struct sSciDevice * pHandle = (struct sSciDevice *)hndl;
	ssize_t             n_bytes = NBytes;
	ssize_t             bytes   = 0;
	
	while(n_bytes != 0)
	{
		if(sciIoctlSCI_GET_COUNT_READ(hndl, 0) != 0)
		{
			bytes    = sciNonBlockRead(hndl, pBuffer ,n_bytes);
			n_bytes -= bytes;
			pBuffer  = (unsigned char *)pBuffer + bytes;	
		}
	}

	return NBytes;
}

/*****************************************************************************
*
* Module:         sciNonBlockWrite()
*
* Description:    Write user buffer out of SCI device.
*
*                 In NonBlocking mode, "sciWrite" starts the transfer operation
*                 and returns control to the application. It does not wait 
*                 while all data is transferred via SCI.
*
* Returns:        Actual number of bytes transmitted
*
* Arguments:      hndl     - SCI Device descriptor returned by "sciOpen" call.
*                 pBuffer  - Pointer to user buffer. 
*                 NBytes   - NBytes (in 8-bit bytes) of the data to be written
*                            out of SCI device; 
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/

ssize_t sciNonBlockWrite( handle_t hndl, const void * pBuffer, size_t NBytes )
{   
    /*
        NBytes          - size in bytes
        pBuffer (R3)    - is a byte pointer     
    */

    asm(move.w   M01, B );  /* save modulo context in B */

#if __m56800E_lmm__ 
    asm(move.w A0,A);
    asm(move.w  A1,Y0);
#else
	asm(nop);
	asm(nop);
#endif
    asm(move.w      Y0, A);

    /* transferting size*/
    asm(move.l      X:(R2 + #(sSciDevice_offset_qSend + sQueue_offset_begin)) , Y); /* Y0 : = begin, Y1 := end */
    asm(move.w      X:(R2 + #(sSciDevice_offset_qSend + sQueue_offset_mask)) , X0); 
    asm(sub         Y0, Y1);
    asm(add         X0, Y1);    /* Y1 := size - 1 - (begin - end) */
    asm(inc.w       Y1);
    asm(cmp         Y1, A);
    asm(tgt         Y1, A);
    /* set up modulo  */
    asm(moveu.w     X0, M01); /* load queue size */
    /* setup buffer pointer */
    asm(and.w       Y0, X0);    /* B := moduled begin (Y0) */
    asm(add.w       X:(R2 + #(sSciDevice_offset_qSend + sQueue_offset_bufferOffset)),X0);
    asm(adda        #BASE_CIRCULAR_BUFFER_ADDRESS, X0, R0);
    
    /* transfert data from *pBuffer to *Queue.pBuffer */
    asm(do          A, end_transferting);
        asm(moveu.bp    X:(R3)+, X0);   /* R3 is a byte pointer */
        asm(nop);
        asm(nop);
        asm(move.w      X0, P:(R0)+);
    asm(end_transferting:);
    
    /* update queue pointers */
    asm(add         A, Y0);     /* update begin */
    asm(move.w      Y0, X:(R2+ #(sSciDevice_offset_qSend + sQueue_offset_begin)));  /* save begin iterator */
#if 1
    asm(moveu.w     X:(R2 + #sSciDevice_offset_Base), R4);  /* SCI base  */
#else
    asm(move.l      X:(R2 + #sSciDevice_offset_Base), R4);  /* SCI base  */
#endif  
    /* Enable Tx interrupt, adda        #BSP_PERIPH_OFF, R4 */
    asm(bfset       #SCI_SCICR_TEIE, X:(R4 + #(BSP_PERIPH_OFF + arch_sSCI_offset_ControlReg) ));

#if __m56800E_lmm__ 
    asm(move.w A1,A0);
    asm(clr.w  A1);
#else
    asm(move.w      A, Y0);
	asm(nop);
#endif

 
    asm(moveu.w  B, M01 );  /* restore modulo context */
    
    /* asm(rts); */
};

/*****************************************************************************
*
* Module:         sciBlockWrite()
*
* Description:    Write user buffer out of SCI device.
*
*                 In Blocking mode, "sciWrite" waits while all required data
*                 is transferred.
*
* Returns:        Actual number of bytes transmitted
*
* Arguments:      hndl     - SCI Device descriptor returned by "sciOpen" call.
*                 pBuffer  - Pointer to user buffer. 
*                 NBytes   - NBytes (in 8-bit bytes) of the data to be written
*                            out of SCI device; 
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/

ssize_t sciBlockWrite( handle_t hndl , const void * pBuffer , size_t NBytes )
{
    struct  sSciDevice * pHandle = (struct sSciDevice *)hndl;
            ssize_t      n_bytes = NBytes;
            ssize_t      bytes;
   
    do
    {
        bytes   = sciNonBlockWrite(hndl , pBuffer , n_bytes);
        n_bytes -= bytes;
        pBuffer = (unsigned char * )pBuffer + bytes;
    } 
    while( n_bytes );

    return NBytes;
}

/*****************************************************************************
*  Some ioctl calls cancel current SCI driver operations.
*
*****************************************************************************/

/*****************************************************************************
*
* Module:         sciIoctlSCI_GET_COUNT_READ()
*
* Description:    return count of chars which can be read
*                 
* Returns:        count
*                 
* Arguments:      hndl - device context
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_GET_COUNT_READ( handle_t hndl , unsigned long params )
{
    struct sSciDevice * pHandle = (struct sSciDevice *)hndl;

    return pHandle->qReceive.begin - pHandle->qReceive.end;
}

/*****************************************************************************
*
* Module:         sciIoctlSCI_GET_COUNT_WRITE()
*
* Description:    return count of chars which can be written
*                 
* Returns:        count
*                 
* Arguments:      hndl - device context
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_GET_COUNT_WRITE( handle_t hndl , unsigned long params )
{
    struct sSciDevice * pHandle = (struct sSciDevice *)hndl;

    return pHandle->qSend.mask + 1 - (pHandle->qSend.begin - pHandle->qSend.end);
}

/*****************************************************************************
*
* Module:         sciIoctlSCI_GET_WRITE_BUFFER_SIZE()
*
* Description:    return write buffer size
*                 
* Returns:        count
*                 
* Arguments:      hndl - device context
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_GET_WRITE_BUFFER_SIZE( handle_t hndl , unsigned long params )
{
    struct sSciDevice * pHandle = (struct sSciDevice *)hndl;

    return pHandle->qSend.mask + 1;
}

/*****************************************************************************
*
* Module:         sciIoctlSCI_GET_READ_BUFFER_SIZE()
*
* Description:    return write buffer size
*                 
* Returns:        count
*                 
* Arguments:      hndl - device context
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_GET_READ_BUFFER_SIZE( handle_t hndl, unsigned long params )
{
    struct sSciDevice * pHandle = (struct sSciDevice *)hndl;

    return pHandle->qReceive.mask + 1;
}

/*****************************************************************************
*
* Module:         sciIoctlSCI_DEVICE_RESET()
*
* Description:    Reset SCI device and set new configuration. 
*                 
* Returns:        None
*                 
* Arguments:      hndl - device context
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_DEVICE_RESET( handle_t hndl, unsigned long params )
{
   struct sSciDevice * pHandle = (struct sSciDevice *)hndl;

   sciIoctlSCI_DEVICE_OFF( hndl, NULL );
   sciIoctlSCI_DEVICE_ON( hndl, NULL );
}

/*****************************************************************************
*
* Module:         sciIoctlSCI_GET_STATUS()
*
* Description:    Read SCI device status               
*
* Returns:        device status
*
* Arguments:      hndl - device descriptor
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_GET_STATUS( handle_t hndl, unsigned long params )
{
    struct  sSciDevice *   pHandle      = (struct sSciDevice *)hndl;
            unsigned int   ReturnValue  = 0;
            
    scidrvHWDisableRxInterrupts( pHandle->Base );
    
    ReturnValue = SCI_STATUS_EXCEPTION_EXIST | periphMemRead( &pHandle->Base->StatusReg ) &
                           (SCI_SCISR_OR | SCI_SCISR_NF | SCI_SCISR_FE | SCI_SCISR_PF);
    
    scidrvHWEnableRxInterrupts( pHandle->Base );
         
    if( periphBitTest( SCI_SCICR_TEIE, &pHandle->Base->StatusReg ) == true )
    {
        ReturnValue |= SCI_STATUS_WRITE_INPROGRESS;            
    }
    
    return ReturnValue;
}
   
/*****************************************************************************
*
* Module:         sciIoctlSCI_CMD_SEND_BREAK()
*
* Description:    Send break symbol via SCI
*
* Returns:        None
*
* Arguments:      hndl - device descriptor
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_CMD_SEND_BREAK( handle_t hndl, unsigned long params )
{
   struct sSciDevice * pHandle = (struct sSciDevice *)hndl;

   periphBitSet( SCI_SCICR_SBK, &pHandle->Base->ControlReg );
   periphBitClear( SCI_SCICR_SBK, &pHandle->Base->ControlReg );
}

/*****************************************************************************
*
* Module:         sciIoctlSCI_CMD_WAIT()
*
* Description:    Put SCI device in wait state
*
* Returns:        None
*
* Arguments:      hndl - device descriptor
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_CMD_WAIT( handle_t hndl, unsigned long params )
{
   struct sSciDevice * pHandle = (struct sSciDevice *)hndl;

   periphBitSet( SCI_SCICR_RWU, &pHandle->Base->ControlReg );
}

/*****************************************************************************
*
* Module:         sciIoctlSCI_CMD_WAKEUP()
*
* Description:    Wake up device from wait mode
*
* Returns:        None
*
* Arguments:      hndl - device descriptor
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_CMD_WAKEUP( handle_t hndl, unsigned long params )
{
   struct sSciDevice * pHandle = (struct sSciDevice *)hndl;

   periphBitClear( SCI_SCICR_RWU, &pHandle->Base->ControlReg );
}

/*****************************************************************************
*
* Module:         sciIoctlSCI_DEVICE_OFF()
*
* Description:    Switch device off, disable device related interrupts
*
* Returns:        None
*
* Arguments:      hndl - device context
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_DEVICE_OFF( handle_t hndl, unsigned long params )
{
   struct sSciDevice * pHandle = (struct sSciDevice*)hndl;

   scidrvHWDisableInterrupts( pHandle->Base );
   /* Clear all receiver related interrupts */
   periphMemRead( &pHandle->Base->StatusReg  );
   periphMemRead( &pHandle->Base->DataReg );
   periphMemWrite( 0x0000, &pHandle->Base->StatusReg );
   
   scidrvHWDisableDevice( pHandle->Base );
}
         
/*****************************************************************************
*
* Module:         sciIoctlSCI_DEVICE_ON()
*
* Description:    Switch device on, clear read and write buffers, enable 
*                 receiver related interrupts
*
* Returns:        None
*
* Arguments:      hndl - device context
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
unsigned int sciIoctlSCI_DEVICE_ON( handle_t hndl , unsigned long params )
{
   struct sSciDevice * pHandle = (struct sSciDevice *)hndl;

   pHandle->qReceive.begin = pHandle->qReceive.end = 0;
   pHandle->qSend.begin = pHandle->qSend.end = 0;

   scidrvHWEnableDevice( pHandle->Base );
   scidrvHWEnableRxInterrupts( pHandle->Base );   
}

/*****************************************************************************/
/*                                 ISRs                                      */
/*****************************************************************************/

/*****************************************************************************
*
* Module:         sciReceiverISR()
*
* Description:    ISRs for SCI receivers
*                 
* Returns:        None
*
* Arguments:      None
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
/* ISR registers allocation:

    R2 - arch
    R3 - context
    R4 - callback
*/
/*
    user callback:
        input:  A0 - value(9 bits) which has been received, A1 - error status 
        output: A0 - value which will be placed into buffer 
                A1 - error state. If clear  If A == -1 value will not be saved
*/

void sciReceiverISR( volatile register arch_sSCI * pArch , struct sSciDevice * pContext, tSciCallBackPtr callback )
{
    /*save context*/
    asm(adda        #2  , SP        );
    asm(move.l      R0  , X:(SP)+   );
    asm(move.l      X0  , X:(SP)+   );  
    asm(move.l      Y   , X:(SP)+   );
    asm(move.l      A2  , X:(SP)+   );
    asm(move.l      A10 , X:(SP)    );

   /* Clear SCI rx and errror related flags */
    asm(move.w      X:(R2 + #arch_sSCI_offset_StatusReg) , A1 );
    asm(clr.w       X:(R2 + #arch_sSCI_offset_StatusReg) );
    asm(move.w      #(SCI_SCISR_OR | SCI_SCISR_NF | SCI_SCISR_FE | SCI_SCISR_PF) , Y1 );
    asm(move.w      X:(R3 + #sSciDevice_offset_Mask) , Y0 );    /* load mask    */
    asm(move.w      X:(R2 + #arch_sSCI_offset_DataReg) , A0 );  /* read Rx reg  */
    asm(and.l       Y  , A );       /* apply mask */

    /* break symbol detecting */
    asm(brclr       #SCI_SCISR_FE , A1 , test_iterators );
    asm(tst.w       A0 );
    asm(bne         test_iterators );
    asm(bfset       #SCI_EXCEPTION_BREAK_SYMBOL , A1 );

test_iterators:
    /* load queue descriptor */
    asm(move.l      X:(R3 + #(sSciDevice_offset_qReceive + sQueue_offset_begin)) , Y ); /* Y0 := begin. Y1 := end */
    asm(move.w      X:(R3 + #(sSciDevice_offset_qReceive + sQueue_offset_mask)) , X0 );
    /* calculate free space in queue */
    asm(sub         Y0 , Y1 );  
    asm(add         X0 , Y1 );          /* Y1 := mask - (begin - end) == free space - 1*/
    asm(bge         call_RxCallBack );  /* if X0 >= Y1, mask >= (begin-end) */
    asm(bfset       #SCI_EXCEPTION_BUFFER_OVERFLOW , A1 );
    
call_RxCallBack:
    /* call callback */
    asm(jsr         (R4) );
    asm(cmp.l       #-1 , A );      /* if return "-1" in A */
    asm(beq         exit_handler );
    asm(cmp.w       Y1 , X0 );
    asm(bcs         exit_handler ); /* if overflow occured */
    /* setup buffer pointer */
    asm(and.w       Y0 , X0);               /* X0 := moduled begin */
    asm(add.w       X:(R3 + #(sSciDevice_offset_qReceive + sQueue_offset_bufferOffset)) , X0 );
    asm(adda        #BASE_CIRCULAR_BUFFER_ADDRESS , X0 , R0 );
    asm(move.w      A0 , X0 );
    asm(move.w      X0 , P:(R0)+ ); 

    /* move begin iterator */
    asm(inc.w       Y0 );   /* Y0 == begin iterator*/
    asm(move.w      Y0 , X:(R3 + #(sSciDevice_offset_qReceive + sQueue_offset_begin)) );
        
exit_handler:
    /*restore context*/
    asm(move.l      X:(SP)- , A  ); 
    asm(move.l      X:(SP)- , A2 );
    asm(move.l      X:(SP)- , Y  ); 
    asm(move.l      X:(SP)- , X0 );
    asm(move.l      X:(SP)- , R0 );
}

/*****************************************************************************
*
* Module:         sciTransmitterISR()
*
* Description:    ISRs for SCI transmitters
*                 
* Returns:        None
*
* Arguments:      None
*
* Range Issues:   None
*
* Test Method:    sci.mcp
*
*****************************************************************************/
/* ISR registers allocation:

    R2 - arch
    R3 - context
    R4 - callback
*/
/*
    user callback:
        input:  A0 - value which is going to sent. A = -1 if there is nothing to send
        output: A0 - value which will be sent, A = "-1" if stop transmission
*/
void sciTransmitterISR( volatile register arch_sSCI * pArch , struct sSciDevice * pContext, tSciCallBackPtr callback )
{
    /*save context*/
    asm(adda        #2  , SP        );
    asm(move.l      R0  , X:(SP)+   );
    asm(move.l      X0  , X:(SP)+   );  
    asm(move.l      Y   , X:(SP)+   );
    asm(move.l      A2  , X:(SP)+   );
    asm(move.l      A10 , X:(SP)    );

    /*setup buffer pointer */
    asm(move.l      X:(R3 + #(sSciDevice_offset_qSend + sQueue_offset_begin)) , Y ); /* Y0 := begin, Y1 := end */
    asm(move.w      X:(R3 + #(sSciDevice_offset_qSend + sQueue_offset_mask)) , X0 );
    asm(and.w       Y1 , X0 );
    asm(add.w       X:(R3 + #(sSciDevice_offset_qSend + sQueue_offset_bufferOffset)) , X0 );
    asm(adda        #BASE_CIRCULAR_BUFFER_ADDRESS , X0 , R0 );
    
    asm(clr         A );
    asm(move.w      P:(R0)+ , X0 ); /* read from queue*/
    asm(move.w      X0 , A0 );
    asm(cmp         Y1 , Y0 );
    asm(bne         tx_callback );
    asm(move.l      #-1 , A );      /* set "-1" */
    
tx_callback:
      /* call users callback */ 
    asm(jsr         ( R4 )  );      /* call transmit callback */
    
go_on:
    asm(cmp.l       #-1 , A );
    asm(bne         transmit_go_on );
    asm(bfclr       #SCI_SCICR_TEIE , X:(R2 + #arch_sSCI_offset_ControlReg) ); /* disable interrupt */
    /* correct queue iterators after force stopping */
    asm(move.w      X:(R3 + #(sSciDevice_offset_qSend + sQueue_offset_end)) , X0);
    asm(move.w      X0 , X:(R3 + #(sSciDevice_offset_qSend + sQueue_offset_begin)) );
    asm(bra         exit_isr);
        
transmit_go_on:
    asm(inc.w       Y1);    /* move end iterator */
    asm(move.w      Y1 , X:(R3 + #(sSciDevice_offset_qSend + sQueue_offset_end)) ); /* update queue descriptor*/    
    /* Clear SCI tx related flags */
    asm(move.w      X:(R2 + #arch_sSCI_offset_StatusReg) , X0 );
    /* be performed before write to Data register to clear SCI Transmitter Ready & Idle flags */
    asm(move.w      X:(R3 + #sSciDevice_offset_Mask) , Y0 );
    asm(and.l       Y , A);
    /* send byte */
    asm(move.w      A0 , X:(R2 + #arch_sSCI_offset_DataReg) );

exit_isr:
    /*restore context*/
    asm(move.l      X:(SP)- , A  ); 
    asm(move.l      X:(SP)- , A2 );
    asm(move.l      X:(SP)- , Y  ); 
    asm(move.l      X:(SP)- , X0 );
    asm(move.l      X:(SP)- , R0 );
}   



