/*******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
* $Element: /project/dsp568_sdk/sdk/src/dsp56838evm/nos/config/config.h $ 
* $Author: saa $ 
* $Revision: /main/2 $ 
* $VOB: /project/dsp568_sdk $ 
* $OS: solaris $ 
*
* Description:  parameters of configuration of SDK application  
*
* Notes:   
*
******************************************************************************/
#ifndef __CONFIG_H
#define __CONFIG_H

/****************************************************************************
*
* Include default and user selected SDK components
*
****************************************************************************/
#include "configdefines.h"

/****************************************************************************
*
* Board specific oscillator frequency
*
****************************************************************************/
#ifndef BSP_OSCILLATOR_FREQ
   #define BSP_OSCILLATOR_FREQ 8000000UL
#endif

#ifndef PLL_MUL
	#define PLL_MUL            30
#endif

/****************************************************************************
*
* Default PLL Initialization
*
****************************************************************************/
#ifdef INCLUDE_PLL

	#ifndef PLL_MUL
		#define PLL_MUL				30
	#endif
		
	#ifndef PLL_CONTROL_REG
		#define PLL_CONTROL_REG 	(PLL_LOCK_DETECTOR | PLL_ZCLOCK_POSTSCALER)
	#endif
					
	#ifndef PLL_DIVIDE_BY_REG
		#define PLL_DIVIDE_BY_REG	( (PLL_MUL - 1) |  PLL_CLOCK_IN_DIVIDE_BY_1 |  PLL_CLOCK_OUT_DIVIDE_BY_1)
	#endif

#endif

/****************************************************************************
*
* Default FLEXCAN driver Initialization
*
****************************************************************************/
#ifdef INCLUDE_FLEXCAN

// CAN predefined speed / bit timing section 
#ifdef CAN_SPEED

#ifndef CAN_PRESCALER
    #define CAN_OSCILLATOR_CLOCK    (BSP_OSCILLATOR_FREQ * PLL_MUL / 2  / 2 / 1000000)
#endif 

#if (CAN_SPEED == 1000000)
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      4 
    #define CAN_PHASE_SEGMENT2      4
    #define CAN_DIVIDER             1
    #define CAN_MIN_FREQUENCY       16
#endif 

#if (CAN_SPEED == 500000)
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      4
    #define CAN_PHASE_SEGMENT2      4
    #define CAN_DIVIDER             1
    #define CAN_MIN_FREQUENCY       8
#endif 

#if (CAN_SPEED == 400000)
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             1
    #define CAN_MIN_FREQUENCY       8
#endif 

#if (CAN_SPEED == 250000)  
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      4 
    #define CAN_PHASE_SEGMENT2      4
    #define CAN_DIVIDER             1
    #define CAN_MIN_FREQUENCY       4
#endif 

#if (CAN_SPEED == 125000)
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      4 
    #define CAN_PHASE_SEGMENT2      4
    #define CAN_DIVIDER             2
    #define CAN_MIN_FREQUENCY       4
#endif 

#if (CAN_SPEED == 100000)
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             2
    #define CAN_MIN_FREQUENCY       4
#endif 

#if (CAN_SPEED == 50000)
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             4
    #define CAN_MIN_FREQUENCY       4
#endif 

#if (CAN_SPEED == 25000)
    #define CAN_SYNCH_JUMP_WIDTH    2
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             8
    #define CAN_MIN_FREQUENCY       4
#endif 

#if (CAN_SPEED == 10000)
    #define CAN_SYNCH_JUMP_WIDTH    2
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             20
    #define CAN_MIN_FREQUENCY       4
#endif 

#if (CAN_OSCILLATOR_CLOCK < CAN_MIN_FREQUENCY)
    #error "incorrect CAN oscillator frequency for this CAN Speed"
#endif // incorrect PLL_MUL 

#define CAN_SAMPLING                0

#ifndef CAN_SYNCH_JUMP_WIDTH
    #define CAN_SYNCH_JUMP_WIDTH    1
#endif

#ifndef CAN_PRESCALER
    #define CAN_PRESCALER (CAN_DIVIDER  * CAN_OSCILLATOR_CLOCK / CAN_MIN_FREQUENCY)
#endif 
#endif 

#if !defined(CAN_PROP_SEGMENT) || !defined(CAN_PHASE_SEGMENT1) || !defined(CAN_PHASE_SEGMENT2) || !defined(CAN_PRESCALER)
    #error  "CAN speed parameters( time segments and prescaler) are not correct assigned"
#endif 

#if CAN_PHASE_SEGMENT1 + CAN_PROP_SEGMENT < 4 || CAN_PHASE_SEGMENT2 < 2
    #error  "time segments parameters are not supported by hardware implementation"
#endif 

#if CAN_PRESCALER > 64
    #error  "invalid prescaler value or CAN oscillator frequency too high"
#endif 

#ifndef  CAN_SAMPLING
    #define CAN_SAMPLING               0
#endif 

#ifndef  CAN_SYNCH_JUMP_WIDTH
    #define CAN_SYNCH_JUMP_WIDTH       1
#endif 

#ifndef CAN_MAX_RECEIVE_ID  
    #define CAN_MAX_RECEIVE_ID         7
#endif 

#ifndef CAN_MAX_TRANSMIT_ID  
    #define CAN_MAX_TRANSMIT_ID        7
#endif 

#ifndef CAN_STOP_IN_WAIT_MODE  
    #define CAN_STOP_IN_WAIT_MODE      0
#else 
    #undef CAN_STOP_IN_WAIT_MODE      
    #define CAN_STOP_IN_WAIT_MODE      1
#endif 

#ifndef CAN_LOOP_BACK
	#define CAN_LOOP_BACK              0
#else 
#undef  CAN_LOOP_BACK
	#define CAN_LOOP_BACK              1
#endif 

#ifndef CAN_WAKE_UP_MODE
	#define CAN_WAKE_UP_MODE           0
#else 
	#undef CAN_WAKE_UP_MODE           
	#define CAN_WAKE_UP_MODE           1
#endif 

#ifndef CAN_CUSTOM_FILTER_CODE
	#define CAN_CUSTOM_FILTER_CODE     0
#endif 

#ifndef CAN_CUSTOM_FILTER_MASK
	#define CAN_CUSTOM_FILTER_MASK     0x7FF
#endif 
#endif 

/****************************************************************************
*
* Default SCI driver Initialization
*
****************************************************************************/
#if defined( INCLUDE_SCI )
   
#if !defined(SCI0_RXBUF_SIZE)
  #define SCI0_RXBUF_SIZE 32
#endif

#if !defined(SCI0_TXBUF_SIZE)
  #define SCI0_TXBUF_SIZE 32
#endif

#if defined(BSP_DEVICE_NAME_SCI_1)

#if !defined(SCI1_RXBUF_SIZE)
  #define SCI1_RXBUF_SIZE 32
#endif

#if !defined(SCI1_TXBUF_SIZE)
  #define SCI1_TXBUF_SIZE 32
#endif

#endif

// calculate SCI boud rate macro 
#define SCI_GET_SBR(BaudRate) (((((unsigned long)(PLL_MUL) * ((unsigned long)BSP_OSCILLATOR_FREQ)) \
                            + (((unsigned long)BaudRate) * 32ul)) / \
                                (((unsigned long)BaudRate) * 64ul)) & 0x1fff)
#endif  

// buffers for modulo 
#define CUR_OFF  0

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 10
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
	#define SCI0_RXBUF_OFF  CUR_OFF
	enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif 

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
	#define SCI0_TXBUF_OFF  CUR_OFF
	enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif 

#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
	#define SCI1_RXBUF_OFF  CUR_OFF
	enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif 

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
	#define SCI1_TXBUF_OFF  CUR_OFF
	enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif 

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 9
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
	#define SCI0_RXBUF_OFF  CUR_OFF
	enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif 

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
	#define SCI0_TXBUF_OFF  CUR_OFF
	enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif

#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
	#define SCI1_RXBUF_OFF  CUR_OFF
	enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
	#define SCI1_TXBUF_OFF  CUR_OFF
	enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif 

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 8
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
	#define SCI0_RXBUF_OFF  CUR_OFF
	enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif 

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
	#define SCI0_TXBUF_OFF  CUR_OFF
	enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif 

#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
	#define SCI1_RXBUF_OFF  CUR_OFF
	enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif 

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
	#define SCI1_TXBUF_OFF  CUR_OFF
	enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 7
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
	#define SCI0_RXBUF_OFF  CUR_OFF
	enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif 

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
	#define SCI0_TXBUF_OFF  CUR_OFF
	enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif

#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
	#define SCI1_RXBUF_OFF  CUR_OFF
	enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif 

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
	#define SCI1_TXBUF_OFF  CUR_OFF
	enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif 

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 6
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
	#define SCI0_RXBUF_OFF  CUR_OFF
	enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif 

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
	#define SCI0_TXBUF_OFF  CUR_OFF
	enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif 

#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
	#define SCI1_RXBUF_OFF  CUR_OFF
	enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
	#define SCI1_TXBUF_OFF  CUR_OFF
	enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 5
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
	#define SCI0_RXBUF_OFF  CUR_OFF
	enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif 

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
	#define SCI0_TXBUF_OFF  CUR_OFF
	enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif 

#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
	#define SCI1_RXBUF_OFF  CUR_OFF
	enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
	#define SCI1_TXBUF_OFF  CUR_OFF
	enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 4
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
	#define SCI0_RXBUF_OFF  CUR_OFF
	enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif 

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
	#define SCI0_TXBUF_OFF  CUR_OFF
	enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif 

#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
	#define SCI1_RXBUF_OFF  CUR_OFF
	enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif 

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
	#define SCI1_TXBUF_OFF  CUR_OFF
	enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif 

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 3
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
	#define SCI0_RXBUF_OFF  CUR_OFF
	enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif 

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
	#define SCI0_TXBUF_OFF  CUR_OFF
	enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif 

#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
	#define SCI1_RXBUF_OFF  CUR_OFF
	enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif 

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
	#define SCI1_TXBUF_OFF  CUR_OFF
	enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif 

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 2 
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > 0 ) && !defined(SCI0_RXBUF_OFF)
	#define SCI0_RXBUF_OFF  CUR_OFF
	enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK =  (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif

#if ( SCI0_TXBUF_SIZE > 0 ) && !defined(SCI0_TXBUF_OFF)
	#define SCI0_TXBUF_OFF  CUR_OFF
	enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK =  (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif 

#if ( SCI1_RXBUF_SIZE > 0 ) && !defined(SCI1_RXBUF_OFF)
	#define SCI1_RXBUF_OFF  CUR_OFF
	enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK =  (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif 

#if ( SCI1_TXBUF_SIZE > 0 ) && !defined(SCI1_TXBUF_OFF)
	#define SCI1_TXBUF_OFF  CUR_OFF
	enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK =  (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
	#undef CUR_OFF
	#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif 

/************************ CIRCULAR BUFFER SETTING **********************************/
#if defined( SCI0_RXBUF_OFF )
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SCI0_RXBUF_MASK + 1
	#else
		enum {ENUM_SCI0_RXBUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SCI0_RXBUF_M01 + SCI0_RXBUF_MASK + 1
	#endif
#endif

#if defined( SCI0_TXBUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SCI0_TXBUF_MASK + 1
	#else
		enum{ ENUM_SCI0_TXBUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SCI0_TXBUF_M01 + SCI0_TXBUF_MASK + 1
	#endif
#endif

#if defined( SCI1_RXBUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SCI1_RXBUF_MASK + 1
	#else
		enum {ENUM_SCI1_RXBUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SCI1_RXBUF_M01 + SCI1_RXBUF_MASK + 1
	#endif
#endif

#if defined( SCI1_TXBUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SCI1_TXBUF_MASK
	#else
		enum {ENUM_SCI1_TXBUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SCI1_TXBUF_M01 + SCI1_TXBUF_MASK + 1
	#endif
#endif

#if defined( CURRENT_CIRCULAR_BUFFER_SIZE ) && ( CURRENT_CIRCULAR_BUFFER_SIZE > 0x800 )
	#error "Summary size  of the circular buffer cannot be greater than 2 K"
#endif

#endif // CONFIG_H 
