# Linker.cmd file for DSP56836EVM Flash
   
#*******************************************************************************
MEMORY {

    .pInterruptVector   (RX)  : ORIGIN = 0x40000, LENGTH = 0x00AA
    .pFlash             (RX)  : ORIGIN = 0x00000, LENGTH = 0x40000
    .pBootFlash         (RX)  : ORIGIN = 0x400AA, LENGTH = 0x3F56
    .pReserved          (RX)  : ORIGIN = 0x44000, LENGTH = 0xB800
    .pIntRAM            (RWX) : ORIGIN = 0x4F800, LENGTH = 0x0800
    .pIntRAM_Mirror     (RWX) : ORIGIN = 0x4F800, LENGTH = 0x0800
    .pExtRAM            (RWX) : ORIGIN = 0x50000, LENGTH = 0x1B0000 #??? check docs  

    .xIntRAM            (RW)  : ORIGIN = 0x0000,  LENGTH = 0x3800
    .xIntRAM_Mirror     (RWX) : ORIGIN = 0x0000,  LENGTH = 0x3800
    .xStack             (RW)  : ORIGIN = 0x3800,  LENGTH = 0x0800
    .xFlash             (R)   : ORIGIN = 0x4000,  LENGTH = 0x3FFD
    .xBootCfg           (R)   : ORIGIN = 0x7FFD,  LENGTH = 0x0003
    .xExtRAM            (RW)  : ORIGIN = 0x8000,  LENGTH = 0x7000 # SMM pRAM limit is 0x7FFF

    .xPeripherals       (RW)  : ORIGIN = 0xF000,  LENGTH = 0x1000
    .xCoreRegisters     (RW)  : ORIGIN = 0xFFFFC0,LENGTH = 0x0100 # check here

}
#*******************************************************************************

FORCE_ACTIVE {FconfigInterruptVector}

#*******************************************************************************

SECTIONS {
#*******************************************************************************    
	.ApplicationInterruptVector :
	{
	    F_vector_begin_addr = .;
        * (isr_vectors.text)
		
	} > .pInterruptVector
#*******************************************************************************
    .ApplicationCode :
    {
        # Place all code into Boot Flash
        
        F_Pcode_begin_addr_in_ROM = .;
        * (.text)
        * (rtlib.text)
        * (fp_engine.text)
        * (user.text)
        F_Pcode_end_addr_in_ROM = .;
        __Pcode_end_addr_in_ROM = .;
        
    } > .pBootFlash

#*******************************************************************************
#    .ApplicationConfiguration :
#    {
#        # Place all code into Program Flash
#        * (appconst.data)
#    } > .xBootCfg
#
#*******************************************************************************
   .ApplicationInitializedData : AT( __Pcode_end_addr_in_ROM )
    {
        # Define variables for C initialization code
        
        F_Xdata_begin_addr_in_ROM = __Pcode_end_addr_in_ROM;
        F_StackAddr               = ADDR(.xStack);
        F_StackEndAddr            = ADDR(.xStack) + SIZEOF(.xStack) / 2  - 1;
        F_Xdata_begin_addr_in_RAM = .;
        __Xdata_begin_addr_in_RAM = .;
                                    
      # Place rest of the data into Internal Data RAM
        * (.const.data.char)  # used if "Emit Separate Char Data Section" enabled
        * (.const.data)
        
        * (.data)
        * (fp_state.data)
        * (rtlib.data)

        F_Xdata_ROMtoRAM_length = . - __Xdata_begin_addr_in_RAM;

    } > .xIntRAM_Mirror
#*******************************************************************************

    .DataForDataRAM : 
    {
      # allocates space for .ApplicationInitializedData section
      . = (ADDR(.xIntRAM_Mirror) + SIZEOF(.xIntRAM_Mirror) / 2);

        # Define variables for C initialization code      
      
        F_Xbss_begin_addr = .;
        __Xbss_begin_addr = .;

      # .bss sections 
      
        * (rtlib.bss.lo)
        * (rtlib.bss)
        * (.bss)
        
        F_Xbss_length = . - __Xbss_begin_addr;
        F_Xdata_end_addr_in_RAM = .;
                        
    } > .xIntRAM
#*******************************************************************************
    .InitializedConstData :
    {
#          WRITEH      0xFFFF;   # CW erases DataFlash if some data is written.

#        * (.const.data.char)  # used if "Emit Separate Char Data Section" enabled
#        * (.const.data)

    } > .xFlash
#*******************************************************************************

	FArchIO   =  0; # ADDR(.xPeripherals);
	FArchCore =  0; # ADDR(.xCoreRegisters);

}
