/*****************************************************************************
*
* Motorola Inc.
* (c) Copyright 2003 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
******************************************************************************
*
* File Name:         bootloader.h
*
* Description:       bootloader main header file with configuration parameters
*
*****************************************************************************/

#if !defined(__BOOTLOADER_H)
#define __BOOTLOADER_H

/*****************************************************************************/
// defines
/*****************************************************************************/

#define BOOT_SCI_BAUD_RATE              115200L
#define BOOT_INDICATION_SYMBOL          '.'
#define BOOT_START_DELAY                30
#define BOOT_VERSION_PROMPT             0xFF
#define BOOT_DEFAULT_APP_START_ADDR     0L
#define BOOT_IGNORE_RAM_DATA            1
#define BOOT_INPUT_OUTPUT_NO            0
#define DATA_RAM_ADDRESS_MASK           0x02000000L
#define USER_INTERRUPT_VECTOR_ADDRESS   0x00000000L

// errors codes 
#define INDICATE_NO_ERROR                       0
#define INDICATE_ERROR_RECEIVE                  1
#define INDICATE_ERROR_CHARACTER                2
#define INDICATE_ERROR_FORMAT                   3
#define INDICATE_ERROR_CHECKSUM                 4
#define INDICATE_ERROR_OVERRUN                  5
#define INDICATE_ERROR_FLASH                    6
#define INDICATE_ERROR_INTERNAL                 7
#define INDICATE_ERROR_RESERVED                 8
#define INDICATE_ERROR_PROTECTED_BOOT_SECTION   9
#define INDICATE_ERROR_FLASH_SECURED            10
#define INDICATE_ERROR_FLASH_PROTECTED          11
#define INDICATE_ERROR_CAN                      12
#define INDICATE_ERROR_LOW_VOLTAGE              13

// warning codes
#define INDICATE_WARNINGS                       100
#define INDICATE_WARNING_DATA_IN_RAM            INDICATE_WARNINGS

// bootloader configuration placed at the end of data flash
#ifdef DSP568357EVM
	#define BOOT_START_ADDRESS_PLACE        0x00002FFEL
	#define BOOT_START_DELAY_PLACE          0x00002FFDL   // low byte 
	#define BOOT_VERSION_PROMPT_PLACE       0x00002FFDL   // high byte 

	#define BOOTFLASH_BEGIN_ADDRESS         0x00020000L
	#define BOOTFLASH_END_ADDRESS           0x00021FFFL

	#define INT_PRAM_BEGIN_ADDRESS          0x0002F800L
	#define INT_PRAM_END_ADDRESS            0x0002FFFFL

	#define EXT_PRAM_BEGIN_ADDRESS          0x00030000L
	#define EXT_PRAM_END_ADDRESS            0x001FFFFFL

	#define DATAFLASH_BEGIN_ADDRESS         0x00002000L
	#define DATAFLASH_END_ADDRESS           0x00002FFFL
	
	#define PROGRAMFLASH_BEGIN_ADDRESS      0x00000000L
	#define PROGRAMFLASH_END_ADDRESS        0x0001FFFFL
	
#elif defined (DSP56836XEVM) 

	#define BOOT_START_ADDRESS_PLACE        0x00007FFEL
	#define BOOT_START_DELAY_PLACE          0x00007FFDL   // low byte 
	#define BOOT_VERSION_PROMPT_PLACE       0x00007FFDL   // high byte 

	#define BOOTFLASH_BEGIN_ADDRESS         0x00040000L
	#define BOOTFLASH_END_ADDRESS           0x00043FFFL

	#define INT_PRAM_BEGIN_ADDRESS          0x0004F800L
	#define INT_PRAM_END_ADDRESS            0x0004FFFFL

	#define EXT_PRAM_BEGIN_ADDRESS          0x00050000L
	#define EXT_PRAM_END_ADDRESS            0x001FFFFFL

	#define DATAFLASH_BEGIN_ADDRESS         0x00004000L
	#define DATAFLASH_END_ADDRESS           0x00007FFFL
	
	#define PROGRAMFLASH_BEGIN_ADDRESS      0x00000000L
	#define PROGRAMFLASH_END_ADDRESS        0x0003FFFFL	
	
#else
	#define BOOT_START_ADDRESS_PLACE        0x00001FFEL
	#define BOOT_START_DELAY_PLACE          0x00001FFDL   // low byte 
	#define BOOT_VERSION_PROMPT_PLACE       0x00001FFDL   // high byte 
	
	#define BOOTFLASH_BEGIN_ADDRESS         0x00020000L
	#define BOOTFLASH_END_ADDRESS           0x00020FFFL
	
	#define INT_PRAM_BEGIN_ADDRESS          0x0002F800L
	#define INT_PRAM_END_ADDRESS            0x0002FFFFL
	
	#define EXT_PRAM_BEGIN_ADDRESS          0x00030000L
	#define EXT_PRAM_END_ADDRESS            0x001FFFFFL
	
	#define DATAFLASH_BEGIN_ADDRESS         0x00001000L
	#define DATAFLASH_END_ADDRESS           0x00001FFFL

#if defined(DSP56832XEVM)
	#define PROGRAMFLASH_BEGIN_ADDRESS      0x00000000L
	#define PROGRAMFLASH_END_ADDRESS        0x00003FFFL
#else
	#define PROGRAMFLASH_BEGIN_ADDRESS      0x00000000L
	#define PROGRAMFLASH_END_ADDRESS        0x0000FFFFL
#endif
#endif

#ifdef INCLUDE_CAN
	#define BOOT_CAN_REPLY_LENGTH 6
	#define BOOT_REPLY_LENGTH     6
#endif

#ifdef CAN20B
	#define BOOT_CAN_ID_CODE    0x01L  // CAN ID for code/data messages   
    #define BOOT_CAN_ID_REPLY   0x02L  // CAN ID for reply messages
#else
	#define BOOT_CAN_ID_CODE    0x01   // CAN ID for code/data messages    
	#define BOOT_CAN_ID_REPLY   0x02   // CAN ID for reply messages 
#endif 

#define XON             17       // Ctrl Q  17 
#define XOFF            19       // Ctrl S  19 
#define bootSendXON()   Temp = XON; \
                        sciBlockWrite( DriverFD, &Temp, 1 )
#define bootSendXOFF()  Temp = XOFF;  \
                        sciBlockWrite( DriverFD, &Temp, 1 )
                        
#if !defined(NULL)
	#define NULL (0)
#endif

#define LVIS27S 0x0008
#define LVIS22S 0x0004
#define LVIE27  0x0002
#define LVIE22  0x0001

#define GPIO_B0	0x0010
#define GPIO_B1	0x0020

#undef  BOOT_APP_START_ADDR_IN_PFLASH
#undef  BOOT_SCI_TO_CAN_GATE
#undef  BOOT_CAN_ACK                        
                        
/****************************************************************************
//    local function prototypes 
****************************************************************************/
bool    bootReadState(void);
void    bootTimerSet(UWord16 timeValue);
UWord16 bootTimerCheck(void);
void    bootError(UWord16 errorNumber);
UWord32 bootAsciiToHex(unsigned char * strbuf, char length);
void    bootHexToAscii(unsigned char value, unsigned char * strbuf);
void    bootMemoryWrite(UWord32 address, UInt8 * buffer, UWord16 length);
void    bootExit(void);
void    bootExit_2(void);
void    bootLoadSRecord( UInt8* srecType, UWord32* dataAddress, UWord16* dataLength, UInt8* dataBuffer );
UWord16 bootBlockRead( UInt8 * buffer, UWord16 length );
UWord16 bootBlockReadWithConvert( UInt8 * buffer, UWord16 length, bool isReverse );
void    pllInitialization(void);

#ifdef INCLUDE_CAN
	void bootCANPrepareTransmit(void);
#ifdef BOOT_SCI_TO_CAN_GATE 
	void bootCANBlockWrite( UInt8* dataBuffer, UWord16 dataLength );
#endif 
#endif

#endif /* !defined(__BOOTLOADER_H) */
