/*******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
* $Element:  $ 
* $Author: saa $ 
* $Revision: /main/2 $ 
* $VOB: /project/dsp568_sdk $ 
* $OS: solaris $ 
*
* Description:       source file for the 56838 GPIO device driver 
*
* Notes:  
*
******************************************************************************/
#ifndef __GPIOINLINE_H
#define __GPIOINLINE_H

#include "port.h"
#include "bsp.h"
#include "periph.h"
#include "arch.h"

#define gpioIoctl(Addr,Cmd,Pin) gpioIoctl##Cmd(Addr,Pin)


#if !defined(gpioPin)
#define gpioPin(Bit) ((unsigned short)(0x01 << Bit))
#endif /* !defined(gpioPin) */


/* registry offsets */
#define GPIO_PULLUP_REG_OFFSET            0x00
#define GPIO_DATA_REG_OFFSET              0x01
#define GPIO_DATA_DIRECTION_REG_OFFSET    0x02
#define GPIO_PERIPHERAL_ENABLE_REG_OFFSET 0x03
#define GPIO_INT_ASSERT_REG_OFFSET        0x04
#define GPIO_INT_ENABLE_REG_OFFSET        0x05
#define GPIO_INT_POLARITY_REG_OFFSET      0x06
#define GPIO_INT_PENDING_REG_OFFSET       0x07
#define GPIO_INT_EDGE_SENS_REG_OFFSET     0x08


/*****************************************************************************
*                             Ioctl functions
*****************************************************************************/

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_SET()
*
* Description:    Set pin to "1". 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_SET( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value; 

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 1), Value); /* Address + (data_reg offset) */
	asm(or.w    Mask, Value); 
	asm(move.w  Value,  x:(arch_gpio + BSP_PERIPH_BASE + 1));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_CLEAR()
*
* Description:    Set pin to "0". 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_CLEAR( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 1), Value); /* Address + (data_reg offset) */
	asm(not.w   Mask);
	asm(and.w   Mask, Value);
	asm(move.w  Value,  x:(arch_gpio + BSP_PERIPH_BASE + 1));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_TOGGLE()
*
* Description:    Change pin logical state. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_TOGGLE( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio+BSP_PERIPH_BASE + 1), Value); /* Address + (data_reg offset) */
	asm(eor.w   Mask, Value);
	asm(move.w  Value,  x:(arch_gpio+BSP_PERIPH_BASE + 1));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_DISABLE_PULLUP()
*
* Description:    Disable pull-up in PUR register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_DISABLE_PULLUP( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio+BSP_PERIPH_BASE + 0), Value); /* Address + (pullup_reg offset) */
	asm(not.w   Mask);
	asm(and.w   Mask, Value);
	asm(move.w  Value,  x:(arch_gpio+BSP_PERIPH_BASE + 0));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_ENABLE_PULLUP()
*
* Description:    Enable pull-up in PUR register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_ENABLE_PULLUP( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;
	
	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 0), Value); /* Address + (pullup_reg offset) */
	asm(or.w    Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 0));
   
}
        
/*****************************************************************************
*
* Module:         gpioIoctlGPIO_SETAS_INPUT()
*
* Description:    Sets a gpio pin as an input in DDR register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_SETAS_INPUT( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 2), Value); /* Address + (data_direction_reg offset) */
	asm(not.w   Mask);
	asm(and.w   Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 2));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_SETAS_OUTPUT()
*
* Description:    Sets a gpio pin as an output in DDR register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_SETAS_OUTPUT( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;
	
	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 2), Value); /* Address + (data_direction_reg offset) */
	asm(or.w    Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 2));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_SETAS_GPIO()
*
* Description:    When peripheral disabled, 
*                 DDR determines direction of data flow in PER register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_SETAS_GPIO( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 3), Value); /* Address + (peripheral_enable_reg offset) */
	asm(not.w   Mask);
	asm(and.w   Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 3));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_SETAS_PERIPHERAL()
*
* Description:    A peripheral masters the gpio pin in PER register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_SETAS_PERIPHERAL( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 3), Value); /* Address + (peripheral_enable_reg offset) */
	asm(or.w    Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 3));
}
    
/*****************************************************************************
*
* Module:         gpioIoctlGPIO_INTERRUPT_ASSERT_DISABLE()
*
* Description:    Disables an interrupt assert in IAR register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_INTERRUPT_ASSERT_DISABLE( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 4), Value); /* Address + (interrupt_assert_reg offset) */
	asm(not.w   Mask);
	asm(and.w   Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 4));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_INTERRUPT_ASSERT_ENABLE()
*
* Description:    Enables an interrupt assert, 
*                 used only in software testing, in IAR register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_INTERRUPT_ASSERT_ENABLE( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 4), Value); /* Address + (interrupt_assert_reg offset) */
	asm(or.w    Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 4));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_INTERRUPT_DISABLE()
*
* Description:    Disables edge detection for any 
*                 incoming interrupt in IENR register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_INTERRUPT_DISABLE( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 5), Value); /* Address + (interrupt_enable_reg offset) */
	asm(not.w   Mask);
	asm(and.w   Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 5));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_INTERRUPT_ENABLE()
*
* Description:    Enables edge detection for any 
*                 incoming interrupt in IENR register. 
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_INTERRUPT_ENABLE( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value,  *Addr;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 5), Value); /* Address + (interrupt_enable_reg offset) */
	asm(or.w    Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 5));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_INTERRUPT_DETECTION_ACTIVE_HIGH()
*
* Description:    The interrupt seen at the PAD is active high.
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_INTERRUPT_DETECTION_ACTIVE_HIGH( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 6), Value); /* Address + (interrupt_polarity_reg offset) */
	asm(not.w   Mask);
	asm(and.w   Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 6));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_INTERRUPT_DETECTION_ACTIVE_LOW()
*
* Description:    The interrupt seen at the PAD is active low.
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_INTERRUPT_DETECTION_ACTIVE_LOW( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 6), Value); /* Address + (interrupt_polarity_reg offset) */
	asm(or.w    Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 6));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_CLEAR_INTERRUPT_PEND_REGISTER()
*
* Description:    By writing zeros (???) to this register, the IPR is cleared.
*                 
* Returns:        None
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline void gpioIoctlGPIO_CLEAR_INTERRUPT_PEND_REGISTER( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned short Value;

	asm(move.w  x:(arch_gpio + BSP_PERIPH_BASE + 8), Value); /* Address + (interrupt_edge_sens_reg offset) */
	asm(or.w    Mask, Value);
	asm(move.w  Value, x:(arch_gpio + BSP_PERIPH_BASE + 8));
}

/*****************************************************************************
*
* Module:         gpioIoctlGPIO_READ()
*
* Description:    Reads the value of an input pin.
*                 
* Returns:        Value of an input pin; 0 or 1 value
*                 
* Arguments:      arch_gpio -  arch hardware device offset
*                 Mask - pin bit mask
*
* Range Issues:   None
*
* Test Method:    gpio.mcp
*
*****************************************************************************/
inline unsigned short gpioIoctlGPIO_READ( register volatile arch_sPort * arch_gpio, register unsigned short Mask )
{
	register unsigned int Value;
	asm(move.w  X:(arch_gpio + BSP_PERIPH_BASE + 1), Value);
	return (Value & Mask) ? 1 : 0;
}
                        

#ifdef __cplusplus
}
#endif

#endif /* __GPIOINLINE_H */
