/*******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
* $Element: /project/dsp568_sdk/sdk/src/dsp56838evm/nos/config/config.h $ 
* $Author: saa $ 
* $Revision: /main/2 $ 
* $VOB: /project/dsp568_sdk $ 
* $OS: solaris $ 
*
* Description:  parameters of configuration of SDK application  
*
* Notes:   
*
******************************************************************************/
#ifndef __CONFIG_H
#define __CONFIG_H


/****************************************************************************
*
* Include default and user selected SDK components
*
****************************************************************************/
#include "configdefines.h"




/****************************************************************************
*
* Board specific oscillator frequency
*
****************************************************************************/

/* DSP56838EVM oscillator frequency is 8.0MHz */
#ifndef BSP_OSCILLATOR_FREQ
   #define BSP_OSCILLATOR_FREQ 8000000UL
#endif

#ifndef PLL_MUL
	#define PLL_MUL				30
#endif
    

/****************************************************************************
*
* Default PLL Initialization
*
****************************************************************************/
#ifdef INCLUDE_PLL

	/* Default PLL configuration */
	
	#ifndef PLL_MUL
		#define PLL_MUL				30
	#endif
		
	#ifndef PLL_CONTROL_REG
		#define PLL_CONTROL_REG 	(PLL_LOCK_DETECTOR | PLL_ZCLOCK_POSTSCALER)
	#endif
					
	#ifndef PLL_DIVIDE_BY_REG
		#define PLL_DIVIDE_BY_REG	( (PLL_MUL - 1) |  PLL_CLOCK_IN_DIVIDE_BY_1 |  PLL_CLOCK_OUT_DIVIDE_BY_1)
	#endif

#endif

/****************************************************************************
*
* Default COP Initialization
*
****************************************************************************/
#ifdef INCLUDE_COP
	/* cop_prescaler = 1024, 60MHz IPBus for Dakar */
	#define COP_PRESCALER           1024UL
#endif

/****************************************************************************
*
* Default Timer Initialization
*
****************************************************************************/
#ifdef INCLUDE_TIMER

	/* default REAL_TIME_TIMER_RESOLUTION is 0xFFFF ticks, max possible value is 0x7FFF80 ticks */

	#ifndef REAL_TICK_TIMER_RESOLUTION
		#define REAL_TICK_TIMER_RESOLUTION 0xFFFF
	#endif
	
	#if (REAL_TICK_TIMER_RESOLUTION > 0x7FFF80)
		#error "REAL_TIME_TIMER_RESOLUTION cannot be more than 0x7FFF80"
	#endif
	
	
#endif

/****************************************************************************
*
* Default MSCAN driver Initialization
*
****************************************************************************/
#ifdef INCLUDE_MSCAN

#ifndef CAN_CLOCK_SOURCE        
#define CAN_CLOCK_SOURCE           1  /* IP Bus is default */
#endif /* ifndef CAN_CLOCK_SOURCE  */


/*** CAN Predefined speed / bit timing section ***/
#if defined(CAN_SPEED)

#if !defined(CAN_PRESCALER)
	#if( CAN_CLOCK_SOURCE == 0 )
		#define CAN_OSCILLATOR_CLOCK    (  BSP_OSCILLATOR_FREQ / 1000000 )
	#else /*( CAN_CLOCK_SOURCE == 0) */
		#define CAN_OSCILLATOR_CLOCK    (  BSP_OSCILLATOR_FREQ * PLL_MUL / 2  / 2 / 1000000 )
	#endif /*( CAN_CLOCK_SOURCE == 0) */
#endif /* !defined(CAN_PRESCALER) */


#if ( CAN_SPEED == 1000000 )
	#define CAN_TIME_SEGMENT1    11
	#define CAN_TIME_SEGMENT2    4
	#define CAN_DIVIDER		     1
	#define CAN_MIN_FREQUENCY    16
#endif /* CAN_SPEED == 1000000 */

#if ( CAN_SPEED == 500000 )
	#define CAN_TIME_SEGMENT1    4
	#define CAN_TIME_SEGMENT2    3
	#define CAN_DIVIDER			 1
	#define CAN_MIN_FREQUENCY    4
#endif /* CAN_SPEED == 500000 */

#if ( CAN_SPEED == 400000 )
#define CAN_TIME_SEGMENT1   6
#define CAN_TIME_SEGMENT2   3
#define CAN_DIVIDER			1
#define CAN_MIN_FREQUENCY   4
#endif /* CAN_SPEED == 400000 */

#if ( CAN_SPEED == 250000 )  
#define CAN_TIME_SEGMENT1   11
#define CAN_TIME_SEGMENT2   4
#define CAN_DIVIDER			1
#define CAN_MIN_FREQUENCY   4
#endif /* CAN_SPEED == 250000 */

#if ( CAN_SPEED == 125000 )
#define CAN_TIME_SEGMENT1   11
#define CAN_TIME_SEGMENT2   4
#define CAN_DIVIDER			2
#define CAN_MIN_FREQUENCY   4
#endif /* CAN_SPEED == 125000 */

#if ( CAN_SPEED == 100000 )
#define CAN_TIME_SEGMENT1   13
#define CAN_TIME_SEGMENT2   6
#define CAN_DIVIDER			2
#define CAN_MIN_FREQUENCY   4
#endif /* CAN_SPEED == 100000 */

#if ( CAN_SPEED == 50000 )
#define CAN_TIME_SEGMENT1   13
#define CAN_TIME_SEGMENT2   6
#define CAN_DIVIDER			4
#define CAN_MIN_FREQUENCY   4
#endif /* CAN_SPEED == 50000 */

#if ( CAN_SPEED == 25000 )
#define CAN_SYNCH_JUMP_WIDTH    2
#define CAN_TIME_SEGMENT1   13
#define CAN_TIME_SEGMENT2   6
#define CAN_DIVIDER			8
#define CAN_MIN_FREQUENCY   4
#endif /* CAN_SPEED == 25000 */

#if ( CAN_SPEED == 10000 )
#define CAN_SYNCH_JUMP_WIDTH    2
#define CAN_TIME_SEGMENT1   13
#define CAN_TIME_SEGMENT2   6
#define CAN_DIVIDER			20
#define CAN_MIN_FREQUENCY   4
#endif /* CAN_SPEED == 10000 */

#if CAN_OSCILLATOR_CLOCK < CAN_MIN_FREQUENCY
	#error "incorrect CAN oscillator frequency for this CAN Speed"
#endif /* PLL_MUL < 8 */

#define CAN_SAMPLING			0

#ifndef  CAN_SYNCH_JUMP_WIDTH
	#define CAN_SYNCH_JUMP_WIDTH    1
#endif /* CAN_SAMPLING */

#if !defined(CAN_PRESCALER)
	#define CAN_PRESCALER ( CAN_DIVIDER  * CAN_OSCILLATOR_CLOCK / CAN_MIN_FREQUENCY )
#endif /* !defined(CAN_PRESCALER) */

#endif /* ifndef CAN_SPEED */

#if !defined(CAN_TIME_SEGMENT1) || !defined(CAN_TIME_SEGMENT2) || !defined(CAN_PRESCALER)
	#error  "CAN speed parameters( time segments and prescaler) are not correct assigned"
#endif /* !defined(CAN_TIME_SEGMENT1) || !defined(CAN_TIME_SEGMENT2) || !defined(CAN_PRESCALER) */

#if CAN_TIME_SEGMENT1 < 4 || CAN_TIME_SEGMENT2 < 2
	#error  "time segments parameters are not supported by hardware implementation"
#endif /* CAN_TIME_SEGMENT1 < 4 || CAN_TIME_SEGMENT2 < 2 */

#if CAN_PRESCALER > 64
	#error  "invalid prescaler value or CAN oscillator frequency too high"
#endif /* CAN_PRESCALER > 64 */



#ifndef  CAN_SAMPLING
	#define CAN_SAMPLING			0
#endif /* CAN_SAMPLING */

#ifndef  CAN_SYNCH_JUMP_WIDTH
	#define CAN_SYNCH_JUMP_WIDTH    1
#endif /* CAN_SYNCH_JUMP_WIDTH */
/*** end of CAN Predefined speed / bit timing section ***/

 
#ifndef CAN_MAX_RECEIVE_ID  
	#define CAN_MAX_RECEIVE_ID         7
#endif /* ifdef CAN_MAX_RECEIVE_ID  */

#ifndef CAN_MAX_TRANSMIT_ID  
	#define CAN_MAX_TRANSMIT_ID        7
#endif /* ifdef CAN_MAX_TRANSMIT_ID  */

#ifndef CAN_STOP_IN_WAIT_MODE  
	#define CAN_STOP_IN_WAIT_MODE      0
#else /* ifndef CAN_STOP_IN_WAIT_MODE */ 
	#undef CAN_STOP_IN_WAIT_MODE      
	#define CAN_STOP_IN_WAIT_MODE      1
#endif /* ifndef CAN_STOP_IN_WAIT_MODE */

#ifndef CAN_LOOP_BACK
#define CAN_LOOP_BACK              0
#else /* ifndef CAN_LOOP_BACK */
#undef  CAN_LOOP_BACK
#define CAN_LOOP_BACK              1
#endif /* ifndef CAN_LOOP_BACK */

#ifndef CAN_WAKE_UP_MODE
#define CAN_WAKE_UP_MODE           0
#else /* ifndef CAN_WAKE_UP_MODE */
#undef CAN_WAKE_UP_MODE           
#define CAN_WAKE_UP_MODE           1
#endif /* #ifndef CAN_WAKE_UP_MODE */


#ifndef CAN_CUSTOM_FILTER_CODE
#if defined (CAN_CUSTOM_FILTER_MASK)
#define CAN_CUSTOM_FILTER_CODE     CANIDMASK
#else /* defined (CAN_CUSTOM_FILTER_MASK) */
#define CAN_CUSTOM_FILTER_CODE     (CANIDMASK + 1)
#endif /* defined (CAN_CUSTOM_FILTER_MASK) */
#endif /* ifndef CAN_CUSTOM_FILTER_CODE */

#ifndef CAN_CUSTOM_FILTER_MASK
#if defined (CAN_CUSTOM_FILTER_CODE)
#define CAN_CUSTOM_FILTER_MASK     0
#else /* if defined (CAN_CUSTOM_FILTER_CODE) */
#define CAN_CUSTOM_FILTER_MASK     (CANIDMASK + 1)
#endif /* if defined (CAN_CUSTOM_FILTER_CODE) */
#endif /* ifndef CAN_CUSTOM_FILTER_MASK */

#if !defined(CAN_RECEIVE_ID_QUEUE_SIZE) || !defined(CAN_QUEUED_TRANSMISSION)
#undef CAN_RECEIVE_ID_QUEUE_SIZE  
#define CAN_RECEIVE_ID_QUEUE_SIZE  1 
#endif /* CAN_RECEIVE_ID_QUEUE_SIZE */

#if CAN_RECEIVE_ID_QUEUE_SIZE > 1 && !defined(CAN_QUEUED_TRANSMISSION)
#error  "Rx queue size can not be greater than 1, when CAN_QUEUED_TRANSMISSION undefined"
#endif /* CAN_RECEIVE_ID_QUEUE_SIZE > 1 && !defined(CAN_QUEUED_TRANSMISSION) */

#if  CAN_RECEIVE_ID_QUEUE_SIZE == 0 && !defined(CAN_RAW_CALLBACK)
#error  "Rx queue size can not be 0, when CAN_RAW_CALLBACK undefined"
#endif /*!defined(CAN_RAW_CALLBACK) && CAN_RECEIVE_ID_QUEUE_SIZE == 0 */

/*
#ifndef CAN_RAW_CALLBACK        
#define CAN_RAW_CALLBACK           NULL
#endif */ /* ifndef CAN_RAW_CALLBACK  */

#if !defined( CAN_TX_ERROR_ASSERT )
#define CAN_TX_ERROR_ASSERT  CAN_ASSERT_BUSOFF
#endif /* !defined( CAN_TX_ERROR_LIMIT )  */

//#if defined( CAN_NO_FAST_ISRs )
//#undef CAN_RECEIVE_HANDLER_USES_PRAGMA_INTERRUPT
//#else /* defined( CAN_NO_FAST_ISRs ) */
//#define CAN_RECEIVE_HANDLER_USES_PRAGMA_INTERRUPT
//#endif /* defined( CAN_NO_FAST_ISRs ) */

/*
	x	x	17	0x0022	MSCAN Wakeup
	x	x	16	0x0020	MSCAN Error
	x	x	15	0x001E	MSCAN Receiver Full
	x	x	14	0x001C	MSCAN Transmitter Ready
*/

#endif /* INCLUDE_MSCAN */

/****************************************************************************
*
* Default FLEXCAN driver Initialization
*
****************************************************************************/
#ifdef INCLUDE_FLEXCAN


/*** CAN Predefined speed / bit timing section ***/
#if defined(CAN_SPEED)

#if !defined(CAN_PRESCALER)
    #define CAN_OSCILLATOR_CLOCK    (  BSP_OSCILLATOR_FREQ * PLL_MUL / 2  / 2 / 1000000 )
#endif /* !defined(CAN_PRESCALER) */


#if ( CAN_SPEED == 1000000 )
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      4 /* = 11 */
    #define CAN_PHASE_SEGMENT2      4
    #define CAN_DIVIDER             1
    #define CAN_MIN_FREQUENCY       16
#endif /* CAN_SPEED == 1000000 */

#if ( CAN_SPEED == 500000 )
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      4
    #define CAN_PHASE_SEGMENT2      4
    #define CAN_DIVIDER             1
    #define CAN_MIN_FREQUENCY       8
#endif /* CAN_SPEED == 500000 */

#if ( CAN_SPEED == 400000 )
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 /* 13 */
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             1
    #define CAN_MIN_FREQUENCY       8
#endif /* CAN_SPEED == 400000 */

#if ( CAN_SPEED == 250000 )  
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      4 /* = 11 */
    #define CAN_PHASE_SEGMENT2      4
    #define CAN_DIVIDER             1
    #define CAN_MIN_FREQUENCY       4
#endif /* CAN_SPEED == 250000 */

#if ( CAN_SPEED == 125000 )
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      4 /* = 11 */
    #define CAN_PHASE_SEGMENT2      4
    #define CAN_DIVIDER             2
    #define CAN_MIN_FREQUENCY       4
#endif /* CAN_SPEED == 125000 */

#if ( CAN_SPEED == 100000 )
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 /* = 13 */
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             2
    #define CAN_MIN_FREQUENCY       4
#endif /* CAN_SPEED == 100000 */

#if ( CAN_SPEED == 50000 )
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 /* = 13 */
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             4
    #define CAN_MIN_FREQUENCY       4
#endif /* CAN_SPEED == 50000 */

#if ( CAN_SPEED == 25000 )
    #define CAN_SYNCH_JUMP_WIDTH    2
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 /* = 13 */
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             8
    #define CAN_MIN_FREQUENCY       4
#endif /* CAN_SPEED == 25000 */

#if ( CAN_SPEED == 10000 )
    #define CAN_SYNCH_JUMP_WIDTH    2
    #define CAN_PROP_SEGMENT        7  
    #define CAN_PHASE_SEGMENT1      6 /* = 13 */
    #define CAN_PHASE_SEGMENT2      6
    #define CAN_DIVIDER             20
    #define CAN_MIN_FREQUENCY       4
#endif /* CAN_SPEED == 10000 */

#if CAN_OSCILLATOR_CLOCK < CAN_MIN_FREQUENCY
    #error "incorrect CAN oscillator frequency for this CAN Speed"
#endif /* incorrect PLL_MUL */

#define CAN_SAMPLING            0

#ifndef  CAN_SYNCH_JUMP_WIDTH
    #define CAN_SYNCH_JUMP_WIDTH    1
#endif /* CAN_SAMPLING */

#if !defined(CAN_PRESCALER)
    #define CAN_PRESCALER ( CAN_DIVIDER  * CAN_OSCILLATOR_CLOCK / CAN_MIN_FREQUENCY )
#endif /* !defined(CAN_PRESCALER) */

#endif /* ifndef CAN_SPEED */

#if !defined(CAN_PROP_SEGMENT) || !defined(CAN_PHASE_SEGMENT1) || !defined(CAN_PHASE_SEGMENT2) || !defined(CAN_PRESCALER)
    #error  "CAN speed parameters( time segments and prescaler) are not correct assigned"
#endif /* !defined(CAN_PHASE_SEGMENT1) || !defined(CAN_PHASE_SEGMENT2) || !defined(CAN_PRESCALER) */

#if CAN_PHASE_SEGMENT1 + CAN_PROP_SEGMENT < 4 || CAN_PHASE_SEGMENT2 < 2
    #error  "time segments parameters are not supported by hardware implementation"
#endif /* CAN_PHASE_SEGMENT1 < 4 || CAN_PHASE_SEGMENT2 < 2 */

#if CAN_PRESCALER > 64
    #error  "invalid prescaler value or CAN oscillator frequency too high"
#endif /* CAN_PRESCALER > 64 */



#ifndef  CAN_SAMPLING
    #define CAN_SAMPLING            0
#endif /* CAN_SAMPLING */

#ifndef  CAN_SYNCH_JUMP_WIDTH
    #define CAN_SYNCH_JUMP_WIDTH    1
#endif /* CAN_SYNCH_JUMP_WIDTH */
/*** end of CAN Predefined speed / bit timing section ***/

 
#ifndef CAN_MAX_RECEIVE_ID  
    #define CAN_MAX_RECEIVE_ID         7
#endif /* ifdef CAN_MAX_RECEIVE_ID  */

#ifndef CAN_MAX_TRANSMIT_ID  
    #define CAN_MAX_TRANSMIT_ID        7
#endif /* ifdef CAN_MAX_TRANSMIT_ID  */

#ifndef CAN_STOP_IN_WAIT_MODE  
    #define CAN_STOP_IN_WAIT_MODE      0
#else /* ifndef CAN_STOP_IN_WAIT_MODE */ 
    #undef CAN_STOP_IN_WAIT_MODE      
    #define CAN_STOP_IN_WAIT_MODE      1
#endif /* ifndef CAN_STOP_IN_WAIT_MODE */

#ifndef CAN_LOOP_BACK
#define CAN_LOOP_BACK              0
#else /* ifndef CAN_LOOP_BACK */
#undef  CAN_LOOP_BACK
#define CAN_LOOP_BACK              1
#endif /* ifndef CAN_LOOP_BACK */

#ifndef CAN_WAKE_UP_MODE
#define CAN_WAKE_UP_MODE           0
#else /* ifndef CAN_WAKE_UP_MODE */
#undef CAN_WAKE_UP_MODE           
#define CAN_WAKE_UP_MODE           1
#endif /* #ifndef CAN_WAKE_UP_MODE */

#if !defined(CAN_CUSTOM_FILTER_CODE)
#define CAN_CUSTOM_FILTER_CODE     0
#endif /* ifndef CAN_CUSTOM_FILTER_CODE */

#if !defined(CAN_CUSTOM_FILTER_MASK)
#define CAN_CUSTOM_FILTER_MASK     0x7FF
#endif /* ifndef CAN_CUSTOM_FILTER_MASK */




#endif /* INCLUDE_FLEXCAN */


/****************************************************************************
*
* Default PWM driver Initialization
*
****************************************************************************/
#ifdef INCLUDE_PWM

		#if !defined(PWMA_CHANNEL_BUFFER_SIZE)
			#define PWMA_CHANNEL_BUFFER_SIZE 32
		#endif
		#if !defined(PWMB_CHANNEL_BUFFER_SIZE)
			#define PWMB_CHANNEL_BUFFER_SIZE 32
		#endif

		#define PWMA_BUF_SIZE (PWM_CLUSTER_SIZE * PWMA_CHANNEL_BUFFER_SIZE)
		#define PWMB_BUF_SIZE (PWM_CLUSTER_SIZE * PWMB_CHANNEL_BUFFER_SIZE)

#endif
/****************************************************************************/

/****************************************************************************
*
* Default SCI driver Initialization
*
****************************************************************************/
#if defined( INCLUDE_SCI )
   
    #if !defined(SCI0_RXBUF_SIZE)
      #define SCI0_RXBUF_SIZE 32
    #endif

    #if !defined(SCI0_TXBUF_SIZE)
      #define SCI0_TXBUF_SIZE 32
    #endif

  #if defined(BSP_DEVICE_NAME_SCI_1)

    #if !defined(SCI1_RXBUF_SIZE)
      #define SCI1_RXBUF_SIZE 32
    #endif

    #if !defined(SCI1_TXBUF_SIZE)
      #define SCI1_TXBUF_SIZE 32
    #endif
  
  #endif

/* calculate SCI boud rate macro */
#define SCI_GET_SBR(BaudRate) (((((unsigned long)(PLL_MUL) * ((unsigned long)BSP_OSCILLATOR_FREQ)) \
                                + (((unsigned long)BaudRate) * 32ul)) / \
                                    (((unsigned long)BaudRate) * 64ul)) & 0x1fff)


#endif  /* defined( INCLUDE_SCI ) */

/****************************************************************************
* 
* Default DAC Configuration
*
****************************************************************************/
#if defined(INCLUDE_DAC)

	
	#if !defined(DAC_CHANNEL_BUFFER_SIZE)
	    #define DAC_CHANNEL_BUFFER_SIZE 32
	#endif

	#define SPI0_BUF_SIZE (DAC_CLUSTER_SIZE * DAC_CHANNEL_BUFFER_SIZE)
	
	ArchInstallCallback(Spi0, ChipSelect, dacSlaveSelect);
	ArchInstallCallback(Spi0, ChipDeselect, dacSlaveDeselect);
	ArchInstallCallback(Spi0, Tx, dacTxCallback);
	
#endif /* INCLUDE_DAC*/

/****************************************************************************
*
* Default SPI Configuration
*
****************************************************************************/
#if defined( INCLUDE_SPI )

	/*SPI0*/
	#if !defined( SPI0_BUF_SIZE )
		#define SPI0_BUF_SIZE 32	
	#endif
		
	/*SPI1*/
	#if !defined( SPI1_BUF_SIZE )
		#define SPI1_BUF_SIZE 32 
	#endif	
	
	/* calculate SPI baud rate macro */
	#define SPI_BPS(divider) ( ( BSP_OSCILLATOR_FREQ * PLL_MUL / 2 / 2 ) / divider )
	
	#define SPI_GET_DEGREE_SBR(BaudRate) BaudRate < SPI_BPS(256) ? 7 : \
								BaudRate < SPI_BPS(128) ? 6 : \
									BaudRate < SPI_BPS(64) ? 5 : \
										BaudRate < SPI_BPS(32) ? 4 : \
											BaudRate < SPI_BPS(16) ? 3 : \
												BaudRate < SPI_BPS(8) ? 2 : \
													BaudRate < SPI_BPS(4) ? 1 : 0
	
	#define SPI_GET_SBR(BaudRate) ( (SPI_GET_DEGREE_SBR(BaudRate) ) << 13 )
	
#endif /* INCLUDE_SPI */

/****************************************************************************
*
* Default ADC Configuration
*
****************************************************************************/
#if defined( INCLUDE_ADC )
	/* ADCA*/
	#if !defined(ADCA_BUF_SIZE)
		#define ADCA_BUF_SIZE 32
	#endif

	/* ADCB*/
	#if !defined(ADCB_BUF_SIZE)
		#define ADCB_BUF_SIZE 32
	#endif
	
	#define ADC_GET_FREQUENCY(BaudRate) ( ( BSP_OSCILLATOR_FREQ * PLL_MUL / 2 / 2 ) / (2*BaudRate) - 1 )

#endif	/* INCLUDE_ADC*/

/* Base for buffers in P RAM (Dakar) */
#define P_MEMORY_BUFFER_BASE 0x02F800UL

/* buffers for modulo */

#define CUR_OFF  0

/*****************************************************************************/
/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 10
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
#define SCI0_RXBUF_OFF  CUR_OFF
enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif /* SCI0_RXBUF */

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
#define SCI0_TXBUF_OFF  CUR_OFF
enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif /* SCI0_TXBUF */


#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
#define SCI1_RXBUF_OFF  CUR_OFF
enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif /* SCI1_RXBUF */

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
#define SCI1_TXBUF_OFF  CUR_OFF
enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif /* SCI1_TXBUF */

#if ( SPI0_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI0_BUF_OFF)
#define SPI0_BUF_OFF  CUR_OFF
enum __spi0Enum_BUF_BASE {SPI0_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI0_BUF_OFFSET = SPI0_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SPI0_BUF_OFFSET + SPI0_BUF_MASK + 1  )
#endif /* SPI0_BUF */

#if ( SPI1_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI1_BUF_OFF)
#define SPI1_BUF_OFF  CUR_OFF
enum __spi1Enum_BUF_BASE {SPI1_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI1_BUF_OFFSET = SPI1_BUF_OFF};
#undef CURO_FF
#define CURO_FF (  SPI1_BUF_OFFSET + SPI1_BUF_MASK + 1  )
#endif /* SPI1_BUF */

#if ( ADCA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCA_BUF_OFF)
#define ADCA_BUF_OFF  CUR_OFF
enum __adcAEnum_BUF_BASE {ADCA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCA_BUF_OFFSET = ADCA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCA_BUF_OFFSET + ADCA_BUF_MASK + 1  )
#endif /* ADCA_BUF */

#if ( ADCB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCB_BUF_OFF)
#define ADCB_BUF_OFF  CUR_OFF
enum __adcBEnum_BUF_BASE {ADCB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCB_BUF_OFFSET = ADCB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCB_BUF_OFFSET + ADCB_BUF_MASK + 1  )
#endif /* ADCB_BUF */

#if ( PWMA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMA_BUF_OFF)
#define PWMA_BUF_OFF  CUR_OFF
enum __pwmAEnum_BUF_BASE {PWMA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMA_BUF_OFFSET = PWMA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMA_BUF_OFFSET + PWMA_BUF_MASK + 1  )
#endif /* PWMA_BUF */

#if ( PWMB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMB_BUF_OFF)
#define PWMB_BUF_OFF  CUR_OFF
enum __pwmBEnum_BUF_BASE {PWMB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMB_BUF_OFFSET = PWMB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMB_BUF_OFFSET + PWMB_BUF_MASK + 1  )
#endif /* PWMB_BUF */

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 9
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
#define SCI0_RXBUF_OFF  CUR_OFF
enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif /* SCI0_RXBUF */

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
#define SCI0_TXBUF_OFF  CUR_OFF
enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif /* SCI0_TXBUF */


#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
#define SCI1_RXBUF_OFF  CUR_OFF
enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif /* SCI1_RXBUF */

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
#define SCI1_TXBUF_OFF  CUR_OFF
enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif /* SCI1_TXBUF */

#if ( SPI0_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI0_BUF_OFF)
#define SPI0_BUF_OFF  CUR_OFF
enum __spi0Enum_BUF_BASE {SPI0_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI0_BUF_OFFSET = SPI0_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SPI0_BUF_OFFSET + SPI0_BUF_MASK + 1  )
#endif /* SPI0_BUF */

#if ( SPI1_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI1_BUF_OFF)
#define SPI1_BUF_OFF  CUR_OFF
enum __spi1Enum_BUF_BASE {SPI1_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI1_BUF_OFFSET = SPI1_BUF_OFF};
#undef CURO_FF
#define CURO_FF (  SPI1_BUF_OFFSET + SPI1_BUF_MASK + 1  )
#endif /* SPI1_BUF */

#if ( ADCA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCA_BUF_OFF)
#define ADCA_BUF_OFF  CUR_OFF
enum __adcAEnum_BUF_BASE {ADCA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCA_BUF_OFFSET = ADCA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCA_BUF_OFFSET + ADCA_BUF_MASK + 1  )
#endif /* ADCA_BUF */

#if ( ADCB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCB_BUF_OFF)
#define ADCB_BUF_OFF  CUR_OFF
enum __adcBEnum_BUF_BASE {ADCB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCB_BUF_OFFSET = ADCB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCB_BUF_OFFSET + ADCB_BUF_MASK + 1  )
#endif /* ADCB_BUF */

#if ( PWMA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMA_BUF_OFF)
#define PWMA_BUF_OFF  CUR_OFF
enum __pwmAEnum_BUF_BASE {PWMA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMA_BUF_OFFSET = PWMA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMA_BUF_OFFSET + PWMA_BUF_MASK + 1  )
#endif /* PWMA_BUF */

#if ( PWMB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMB_BUF_OFF)
#define PWMB_BUF_OFF  CUR_OFF
enum __pwmBEnum_BUF_BASE {PWMB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMB_BUF_OFFSET = PWMB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMB_BUF_OFFSET + PWMB_BUF_MASK + 1  )
#endif /* PWMB_BUF */

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 8
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
#define SCI0_RXBUF_OFF  CUR_OFF
enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif /* SCI0_RXBUF */

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
#define SCI0_TXBUF_OFF  CUR_OFF
enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif /* SCI0_TXBUF */


#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
#define SCI1_RXBUF_OFF  CUR_OFF
enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif /* SCI1_RXBUF */

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
#define SCI1_TXBUF_OFF  CUR_OFF
enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif /* SCI1_TXBUF */

#if ( SPI0_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI0_BUF_OFF)
#define SPI0_BUF_OFF  CUR_OFF
enum __spi0Enum_BUF_BASE {SPI0_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI0_BUF_OFFSET = SPI0_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SPI0_BUF_OFFSET + SPI0_BUF_MASK + 1  )
#endif /* SPI0_BUF */

#if ( SPI1_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI1_BUF_OFF)
#define SPI1_BUF_OFF  CUR_OFF
enum __spi1Enum_BUF_BASE {SPI1_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI1_BUF_OFFSET = SPI1_BUF_OFF};
#undef CURO_FF
#define CURO_FF (  SPI1_BUF_OFFSET + SPI1_BUF_MASK + 1  )
#endif /* SPI1_BUF */

#if ( ADCA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCA_BUF_OFF)
#define ADCA_BUF_OFF  CUR_OFF
enum __adcAEnum_BUF_BASE {ADCA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCA_BUF_OFFSET = ADCA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCA_BUF_OFFSET + ADCA_BUF_MASK + 1  )
#endif /* ADCA_BUF */

#if ( ADCB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCB_BUF_OFF)
#define ADCB_BUF_OFF  CUR_OFF
enum __adcBEnum_BUF_BASE {ADCB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCB_BUF_OFFSET = ADCB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCB_BUF_OFFSET + ADCB_BUF_MASK + 1  )
#endif /* ADCB_BUF */

#if ( PWMA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMA_BUF_OFF)
#define PWMA_BUF_OFF  CUR_OFF
enum __pwmAEnum_BUF_BASE {PWMA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMA_BUF_OFFSET = PWMA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMA_BUF_OFFSET + PWMA_BUF_MASK + 1  )
#endif /* PWMA_BUF */

#if ( PWMB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMB_BUF_OFF)
#define PWMB_BUF_OFF  CUR_OFF
enum __pwmBEnum_BUF_BASE {PWMB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMB_BUF_OFFSET = PWMB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMB_BUF_OFFSET + PWMB_BUF_MASK + 1  )
#endif /* PWMB_BUF */

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 7
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
#define SCI0_RXBUF_OFF  CUR_OFF
enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif /* SCI0_RXBUF */

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
#define SCI0_TXBUF_OFF  CUR_OFF
enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif /* SCI0_TXBUF */


#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
#define SCI1_RXBUF_OFF  CUR_OFF
enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif /* SCI1_RXBUF */

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
#define SCI1_TXBUF_OFF  CUR_OFF
enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif /* SCI1_TXBUF */

#if ( SPI0_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI0_BUF_OFF)
#define SPI0_BUF_OFF  CUR_OFF
enum __spi0Enum_BUF_BASE {SPI0_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI0_BUF_OFFSET = SPI0_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SPI0_BUF_OFFSET + SPI0_BUF_MASK + 1  )
#endif /* SPI0_BUF */

#if ( SPI1_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI1_BUF_OFF)
#define SPI1_BUF_OFF  CUR_OFF
enum __spi1Enum_BUF_BASE {SPI1_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI1_BUF_OFFSET = SPI1_BUF_OFF};
#undef CURO_FF
#define CURO_FF (  SPI1_BUF_OFFSET + SPI1_BUF_MASK + 1  )
#endif /* SPI1_BUF */

#if ( ADCA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCA_BUF_OFF)
#define ADCA_BUF_OFF  CUR_OFF
enum __adcAEnum_BUF_BASE {ADCA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCA_BUF_OFFSET = ADCA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCA_BUF_OFFSET + ADCA_BUF_MASK + 1  )
#endif /* ADCA_BUF */

#if ( ADCB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCB_BUF_OFF)
#define ADCB_BUF_OFF  CUR_OFF
enum __adcBEnum_BUF_BASE {ADCB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCB_BUF_OFFSET = ADCB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCB_BUF_OFFSET + ADCB_BUF_MASK + 1  )
#endif /* ADCB_BUF */

#if ( PWMA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMA_BUF_OFF)
#define PWMA_BUF_OFF  CUR_OFF
enum __pwmAEnum_BUF_BASE {PWMA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMA_BUF_OFFSET = PWMA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMA_BUF_OFFSET + PWMA_BUF_MASK + 1  )
#endif /* PWMA_BUF */

#if ( PWMB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMB_BUF_OFF)
#define PWMB_BUF_OFF  CUR_OFF
enum __pwmBEnum_BUF_BASE {PWMB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMB_BUF_OFFSET = PWMB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMB_BUF_OFFSET + PWMB_BUF_MASK + 1  )
#endif /* PWMB_BUF */

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 6
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
#define SCI0_RXBUF_OFF  CUR_OFF
enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif /* SCI0_RXBUF */

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
#define SCI0_TXBUF_OFF  CUR_OFF
enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif /* SCI0_TXBUF */


#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
#define SCI1_RXBUF_OFF  CUR_OFF
enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif /* SCI1_RXBUF */

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
#define SCI1_TXBUF_OFF  CUR_OFF
enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif /* SCI1_TXBUF */

#if ( SPI0_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI0_BUF_OFF)
#define SPI0_BUF_OFF  CUR_OFF
enum __spi0Enum_BUF_BASE {SPI0_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI0_BUF_OFFSET = SPI0_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SPI0_BUF_OFFSET + SPI0_BUF_MASK + 1  )
#endif /* SPI0_BUF */

#if ( SPI1_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI1_BUF_OFF)
#define SPI1_BUF_OFF  CUR_OFF
enum __spi1Enum_BUF_BASE {SPI1_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI1_BUF_OFFSET = SPI1_BUF_OFF};
#undef CURO_FF
#define CURO_FF (  SPI1_BUF_OFFSET + SPI1_BUF_MASK + 1  )
#endif /* SPI1_BUF */

#if ( ADCA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCA_BUF_OFF)
#define ADCA_BUF_OFF  CUR_OFF
enum __adcAEnum_BUF_BASE {ADCA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCA_BUF_OFFSET = ADCA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCA_BUF_OFFSET + ADCA_BUF_MASK + 1  )
#endif /* ADCA_BUF */

#if ( ADCB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCB_BUF_OFF)
#define ADCB_BUF_OFF  CUR_OFF
enum __adcBEnum_BUF_BASE {ADCB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCB_BUF_OFFSET = ADCB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCB_BUF_OFFSET + ADCB_BUF_MASK + 1  )
#endif /* ADCB_BUF */

#if ( PWMA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMA_BUF_OFF)
#define PWMA_BUF_OFF  CUR_OFF
enum __pwmAEnum_BUF_BASE {PWMA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMA_BUF_OFFSET = PWMA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMA_BUF_OFFSET + PWMA_BUF_MASK + 1  )
#endif /* PWMA_BUF */

#if ( PWMB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMB_BUF_OFF)
#define PWMB_BUF_OFF  CUR_OFF
enum __pwmBEnum_BUF_BASE {PWMB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMB_BUF_OFFSET = PWMB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMB_BUF_OFFSET + PWMB_BUF_MASK + 1  )
#endif /* PWMB_BUF */

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 5
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
#define SCI0_RXBUF_OFF  CUR_OFF
enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif /* SCI0_RXBUF */

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
#define SCI0_TXBUF_OFF  CUR_OFF
enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif /* SCI0_TXBUF */


#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
#define SCI1_RXBUF_OFF  CUR_OFF
enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif /* SCI1_RXBUF */

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
#define SCI1_TXBUF_OFF  CUR_OFF
enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif /* SCI1_TXBUF */

#if ( SPI0_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI0_BUF_OFF)
#define SPI0_BUF_OFF  CUR_OFF
enum __spi0Enum_BUF_BASE {SPI0_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI0_BUF_OFFSET = SPI0_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SPI0_BUF_OFFSET + SPI0_BUF_MASK + 1  )
#endif /* SPI0_BUF */

#if ( SPI1_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI1_BUF_OFF)
#define SPI1_BUF_OFF  CUR_OFF
enum __spi1Enum_BUF_BASE {SPI1_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI1_BUF_OFFSET = SPI1_BUF_OFF};
#undef CURO_FF
#define CURO_FF (  SPI1_BUF_OFFSET + SPI1_BUF_MASK + 1  )
#endif /* SPI1_BUF */

#if ( ADCA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCA_BUF_OFF)
#define ADCA_BUF_OFF  CUR_OFF
enum __adcAEnum_BUF_BASE {ADCA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCA_BUF_OFFSET = ADCA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCA_BUF_OFFSET + ADCA_BUF_MASK + 1  )
#endif /* ADCA_BUF */

#if ( ADCB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCB_BUF_OFF)
#define ADCB_BUF_OFF  CUR_OFF
enum __adcBEnum_BUF_BASE {ADCB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCB_BUF_OFFSET = ADCB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCB_BUF_OFFSET + ADCB_BUF_MASK + 1  )
#endif /* ADCB_BUF */

#if ( PWMA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMA_BUF_OFF)
#define PWMA_BUF_OFF  CUR_OFF
enum __pwmAEnum_BUF_BASE {PWMA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMA_BUF_OFFSET = PWMA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMA_BUF_OFFSET + PWMA_BUF_MASK + 1  )
#endif /* PWMA_BUF */

#if ( PWMB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMB_BUF_OFF)
#define PWMB_BUF_OFF  CUR_OFF
enum __pwmBEnum_BUF_BASE {PWMB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMB_BUF_OFFSET = PWMB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMB_BUF_OFFSET + PWMB_BUF_MASK + 1  )
#endif /* PWMB_BUF */

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 4
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
#define SCI0_RXBUF_OFF  CUR_OFF
enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif /* SCI0_RXBUF */

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
#define SCI0_TXBUF_OFF  CUR_OFF
enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif /* SCI0_TXBUF */


#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
#define SCI1_RXBUF_OFF  CUR_OFF
enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif /* SCI1_RXBUF */

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
#define SCI1_TXBUF_OFF  CUR_OFF
enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif /* SCI1_TXBUF */

#if ( SPI0_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI0_BUF_OFF)
#define SPI0_BUF_OFF  CUR_OFF
enum __spi0Enum_BUF_BASE {SPI0_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI0_BUF_OFFSET = SPI0_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SPI0_BUF_OFFSET + SPI0_BUF_MASK + 1  )
#endif /* SPI0_BUF */

#if ( SPI1_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI1_BUF_OFF)
#define SPI1_BUF_OFF  CUR_OFF
enum __spi1Enum_BUF_BASE {SPI1_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI1_BUF_OFFSET = SPI1_BUF_OFF};
#undef CURO_FF
#define CURO_FF (  SPI1_BUF_OFFSET + SPI1_BUF_MASK + 1  )
#endif /* SPI1_BUF */

#if ( ADCA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCA_BUF_OFF)
#define ADCA_BUF_OFF  CUR_OFF
enum __adcAEnum_BUF_BASE {ADCA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCA_BUF_OFFSET = ADCA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCA_BUF_OFFSET + ADCA_BUF_MASK + 1  )
#endif /* ADCA_BUF */

#if ( ADCB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCB_BUF_OFF)
#define ADCB_BUF_OFF  CUR_OFF
enum __adcBEnum_BUF_BASE {ADCB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCB_BUF_OFFSET = ADCB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCB_BUF_OFFSET + ADCB_BUF_MASK + 1  )
#endif /* ADCB_BUF */

#if ( PWMA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMA_BUF_OFF)
#define PWMA_BUF_OFF  CUR_OFF
enum __pwmAEnum_BUF_BASE {PWMA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMA_BUF_OFFSET = PWMA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMA_BUF_OFFSET + PWMA_BUF_MASK + 1  )
#endif /* PWMA_BUF */

#if ( PWMB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMB_BUF_OFF)
#define PWMB_BUF_OFF  CUR_OFF
enum __pwmBEnum_BUF_BASE {PWMB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMB_BUF_OFFSET = PWMB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMB_BUF_OFFSET + PWMB_BUF_MASK + 1  )
#endif /* PWMB_BUF */

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 3
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_RXBUF_OFF)
#define SCI0_RXBUF_OFF  CUR_OFF
enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif /* SCI0_RXBUF */

#if ( SCI0_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI0_TXBUF_OFF)
#define SCI0_TXBUF_OFF  CUR_OFF
enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif /* SCI0_TXBUF */


#if ( SCI1_RXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_RXBUF_OFF)
#define SCI1_RXBUF_OFF  CUR_OFF
enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif /* SCI1_RXBUF */

#if ( SCI1_TXBUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SCI1_TXBUF_OFF)
#define SCI1_TXBUF_OFF  CUR_OFF
enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK = (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif /* SCI1_TXBUF */

#if ( SPI0_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI0_BUF_OFF)
#define SPI0_BUF_OFF  CUR_OFF
enum __spi0Enum_BUF_BASE {SPI0_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI0_BUF_OFFSET = SPI0_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SPI0_BUF_OFFSET + SPI0_BUF_MASK + 1  )
#endif /* SPI0_BUF */

#if ( SPI1_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(SPI1_BUF_OFF)
#define SPI1_BUF_OFF  CUR_OFF
enum __spi1Enum_BUF_BASE {SPI1_BUF_MASK = (  (2 << BUF_BASE) - 1  ), SPI1_BUF_OFFSET = SPI1_BUF_OFF};
#undef CURO_FF
#define CURO_FF (  SPI1_BUF_OFFSET + SPI1_BUF_MASK + 1  )
#endif /* SPI1_BUF */

#if ( ADCA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCA_BUF_OFF)
#define ADCA_BUF_OFF  CUR_OFF
enum __adcAEnum_BUF_BASE {ADCA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCA_BUF_OFFSET = ADCA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCA_BUF_OFFSET + ADCA_BUF_MASK + 1  )
#endif /* ADCA_BUF */

#if ( ADCB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(ADCB_BUF_OFF)
#define ADCB_BUF_OFF  CUR_OFF
enum __adcBEnum_BUF_BASE {ADCB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), ADCB_BUF_OFFSET = ADCB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCB_BUF_OFFSET + ADCB_BUF_MASK + 1  )
#endif /* ADCB_BUF */

#if ( PWMA_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMA_BUF_OFF)
#define PWMA_BUF_OFF  CUR_OFF
enum __pwmAEnum_BUF_BASE {PWMA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMA_BUF_OFFSET = PWMA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMA_BUF_OFFSET + PWMA_BUF_MASK + 1  )
#endif /* PWMA_BUF */

#if ( PWMB_BUF_SIZE > (2 <<(BUF_BASE-1)) ) && !defined(PWMB_BUF_OFF)
#define PWMB_BUF_OFF  CUR_OFF
enum __pwmBEnum_BUF_BASE {PWMB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMB_BUF_OFFSET = PWMB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMB_BUF_OFFSET + PWMB_BUF_MASK + 1  )
#endif /* PWMB_BUF */

/*****************************************************************************/
#undef BUF_BASE 
#define BUF_BASE 2 /* minimal value */
/*****************************************************************************/
#if ( SCI0_RXBUF_SIZE > 0 ) && !defined(SCI0_RXBUF_OFF)
#define SCI0_RXBUF_OFF  CUR_OFF
enum __sci0RxEnum_BUF_BASE {SCI0_RXBUF_MASK =  (  (2 << BUF_BASE) - 1  ), SCI0_RXBUF_OFFSET = SCI0_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_RXBUF_OFFSET + SCI0_RXBUF_MASK + 1  )
#endif /* SCI0_RXBUF */

#if ( SCI0_TXBUF_SIZE > 0 ) && !defined(SCI0_TXBUF_OFF)
#define SCI0_TXBUF_OFF  CUR_OFF
enum __sci0TxEnum_BUF_BASE {SCI0_TXBUF_MASK =  (  (2 << BUF_BASE) - 1  ), SCI0_TXBUF_OFFSET = CUR_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI0_TXBUF_OFFSET + SCI0_TXBUF_MASK + 1  )
#endif /* SCI0_TXBUF */

#if ( SCI1_RXBUF_SIZE > 0 ) && !defined(SCI1_RXBUF_OFF)
#define SCI1_RXBUF_OFF  CUR_OFF
enum __sci1RxEnum_BUF_BASE {SCI1_RXBUF_MASK =  (  (2 << BUF_BASE) - 1  ), SCI1_RXBUF_OFFSET = SCI1_RXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_RXBUF_OFFSET + SCI1_RXBUF_MASK + 1  )
#endif /* SCI1_RXBUF */

#if ( SCI1_TXBUF_SIZE > 0 ) && !defined(SCI1_TXBUF_OFF)
#define SCI1_TXBUF_OFF  CUR_OFF
enum __sci1TxEnum_BUF_BASE {SCI1_TXBUF_MASK =  (  (2 << BUF_BASE) - 1  ), SCI1_TXBUF_OFFSET = SCI1_TXBUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SCI1_TXBUF_OFFSET + SCI1_TXBUF_MASK  + 1  )
#endif /* SCI1_TXBUF */

#if ( SPI0_BUF_SIZE > 0 ) && !defined(SPI0_BUF_OFF)
#define SPI0_BUF_OFF  CUR_OFF
enum __spi0Enum_BUF_BASE {SPI0_BUF_MASK =  (  (2 << BUF_BASE) - 1  ), SPI0_BUF_OFFSET = SPI0_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  SPI0_BUF_OFFSET + SPI0_BUF_MASK + 1  )
#endif /* SPI0_BUF */

#if ( SPI1_BUF_SIZE > 0 ) && !defined(SPI1_BUF_OFF)
#define SPI1_BUF_OFF  CUR_OFF
enum __spi1Enum_BUF_BASE {SPI1_BUF_MASK =  (  (2 << BUF_BASE) - 1  ), SPI1_BUF_OFFSET = SPI1_BUF_OFF};
#undef CURO_FF
#define CURO_FF (  SPI1_BUF_OFFSET + SPI1_BUF_MASK + 1  )
#endif /* SPI1_BUF */

#if ( ADCA_BUF_SIZE > 0 ) && !defined(ADCA_BUF_OFF)
#define ADCA_BUF_OFF  CUR_OFF
enum __adcAEnum_BUF_BASE {ADCA_BUF_MASK =  (  (2 << BUF_BASE) - 1  ), ADCA_BUF_OFFSET = ADCA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCA_BUF_OFFSET + ADCA_BUF_MASK + 1  )
#endif /* ADCA_BUF */

#if ( ADCB_BUF_SIZE > 0 ) && !defined(ADCB_BUF_OFF)
#define ADCB_BUF_OFF  CUR_OFF
enum __adcBEnum_BUF_BASE {ADCB_BUF_MASK =  (  (2 << BUF_BASE) - 1  ), ADCB_BUF_OFFSET = ADCB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  ADCB_BUF_OFFSET + ADCB_BUF_MASK + 1  )
#endif /* ADCB_BUF */

#if ( PWMA_BUF_SIZE > 0 ) && !defined(PWMA_BUF_OFF)
#define PWMA_BUF_OFF  CUR_OFF
enum __pwmAEnum_BUF_BASE {PWMA_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMA_BUF_OFFSET = PWMA_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMA_BUF_OFFSET + PWMA_BUF_MASK + 1  )
#endif /* PWMA_BUF */

#if ( PWMB_BUF_SIZE > 0 ) && !defined(PWMB_BUF_OFF)
#define PWMB_BUF_OFF  CUR_OFF
enum __pwmBEnum_BUF_BASE {PWMB_BUF_MASK = (  (2 << BUF_BASE) - 1  ), PWMB_BUF_OFFSET = PWMB_BUF_OFF};
#undef CUR_OFF
#define CUR_OFF (  PWMB_BUF_OFFSET + PWMB_BUF_MASK + 1  )
#endif /* PWMB_BUF */

/************************ CIRCULAR BUFFER SETTING **********************************/

#if defined( SCI0_RXBUF_OFF )
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SCI0_RXBUF_MASK + 1
	#else
		enum {ENUM_SCI0_RXBUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SCI0_RXBUF_M01 + SCI0_RXBUF_MASK + 1
	#endif
#endif

#if defined( SCI0_TXBUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SCI0_TXBUF_MASK + 1
	#else
		enum{ ENUM_SCI0_TXBUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SCI0_TXBUF_M01 + SCI0_TXBUF_MASK + 1
	#endif
#endif

#if defined( SCI1_RXBUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SCI1_RXBUF_MASK + 1
	#else
		enum {ENUM_SCI1_RXBUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SCI1_RXBUF_M01 + SCI1_RXBUF_MASK + 1
	#endif
#endif

#if defined( SCI1_TXBUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SCI1_TXBUF_MASK
	#else
		enum {ENUM_SCI1_TXBUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SCI1_TXBUF_M01 + SCI1_TXBUF_MASK + 1
	#endif
#endif

#if defined( SPI0_BUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SPI0_BUF_MASK
	#else
		enum {ENUM_SPI0_BUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SPI0_BUF_M01 + SPI0_BUF_MASK + 1
	#endif
#endif

#if defined( SPI1_BUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE SPI1_BUF_MASK
	#else
		enum {ENUM_SPI1_BUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_SPI1_BUF_M01 + SPI1_BUF_MASK + 1
	#endif
#endif

#if defined( ADCA_BUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE ADCA_BUF_MASK
	#else
		enum {ENUM_ADCA_BUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_ADCA_BUF_M01 + ADCA_BUF_MASK + 1
	#endif
#endif

#if defined( ADCB_BUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE ADCB_BUF_MASK
	#else
		enum {ENUM_ADCB_BUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_ADCB_BUF_M01 + ADCB_BUF_MASK + 1
	#endif
#endif

#if defined( PWMA_BUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE PWMA_BUF_MASK
	#else
		enum {ENUM_PWMA_BUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_PWMA_BUF_M01 + PWMA_BUF_MASK + 1
	#endif
#endif


#if defined( PWMB_BUF_OFF ) 
	#if !defined( CURRENT_CIRCULAR_BUFFER_SIZE )
		#define CURRENT_CIRCULAR_BUFFER_SIZE PWMB_BUF_MASK
	#else
		enum {ENUM_PWMB_BUF_M01 = CURRENT_CIRCULAR_BUFFER_SIZE };
		#undef CURRENT_CIRCULAR_BUFFER_SIZE
		#define CURRENT_CIRCULAR_BUFFER_SIZE ENUM_PWMB_BUF_M01 + PWMB_BUF_MASK + 1
	#endif
#endif

#if defined( CURRENT_CIRCULAR_BUFFER_SIZE ) && ( CURRENT_CIRCULAR_BUFFER_SIZE > 0x800 )
	#error "Summary size  of the circular buffer cannot be greater than 2 K"
#endif


#endif /* __CONFIG_H */
