/******************************************************************************
*
*       COPYRIGHT 2003 MOTOROLA, ALL RIGHTS RESERVED
*
*       The code is the property of Motorola St.Petersburg Software Development
*       and is Motorola Confidential Proprietary Information.
*
*       The copyright notice above does not evidence any
*       actual or intended publication of such source code.
*
* Functions:    
*
* Description:     
*
* Notes:        
*
******************************************************************************/

#include "bsp.h"
#include "periph.h"

#include "flexcan.h"
#include "stdarg.h"

can_sBuffer IOBuffer[16];  /* supported buffers */
int nBottomRead = 16; /* bottom pointer */
int nTopWrite = 0;
int CANErrno;
unsigned int nBusyFlags = 0;

extern const unsigned int FlexCANFlags;  /* FlexCAN configuration word */
extern const io_sFLEXCANInterface canInterfaceVT;
extern const can_pRawCallBack CANRcvPrepare;  /*  RAW CallBack pointer */


/*******************************************************************************
*
* NAME:             FC_Init
*
* DESCRIPTION:      Initialize CAN driver device 
*              
********************************************************************************
* PARAMETERS:       
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   device initialization.
*
* DEPENDENCIES:     None
*******************************************************************************/
void FC_Init( )
{       
    unsigned int i, j;
    extern const can_sInitialState canInitialState;

    periphMemWrite(0x0200,  &ArchIO.CAN.ConfigReg );
    i = 0;
    periphMemWrite(15,  &ArchIO.CAN.MaxMBReg );

    periphMemWrite( canInitialState.regFCMCR,  &ArchIO.CAN.ConfigReg );
    periphMemWrite( canInitialState.regFCCTL0, &ArchIO.CAN.Control0Reg );
    periphMemWrite( canInitialState.regFCCTL1, &ArchIO.CAN.Control1Reg );

#if 0 /* default mask state is OK */
    periphMemWrite( 0xFFE7, &ArchIO.CAN.RxGlobalMaskHiReg );
    periphMemWrite( 0xFFFE, &ArchIO.CAN.RxGlobalMaskLoReg );
    periphMemWrite( 0xFFE7, &ArchIO.CAN.Rx14MaskHiReg );
    periphMemWrite( 0xFFFE, &ArchIO.CAN.Rx14MaskLoReg );
    periphMemWrite( 0xFFE7, &ArchIO.CAN.Rx15MaskHiReg );
    periphMemWrite( 0xFFFE, &ArchIO.CAN.Rx15MaskLoReg );
#endif
    periphMemWrite(0,   &ArchIO.CAN.IntMask1Reg );

    for( ; i < 16; i++ ) 
    {
        /* reset all elements in all Message Buffers Control Register */
        periphMemWrite( FC_MB_NOTREADY, &ArchIO.CAN.Buffer[i].CtlStatusReg );
    }

    periphBitClear( 0xF000, &ArchIO.CAN.ConfigReg);

}

/*******************************************************************************
*
* NAME: CheckCANID
*
* DESCRIPTION: check CAN ID to protect from reopen
*              
********************************************************************************
* PARAMETERS:   CAN ID, mode for loopback
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
int CheckCANID( CANADDRESSTYPE canID, unsigned int mode )
{
    register int i;

    if( !( mode & (O_LOOP_BACK) >> 1 )   )    
        for( i = nBottomRead;  i  < 16; i++ )
    {
        if( IOBuffer[i].canID == canID)  /* checking receive buffers */
        {
            CANErrno = CAN_ERR_REOPEN;
            return 0;  
        }
    }
    
    if( !( mode & (O_LOOP_BACK)  )   )    
        for( i = 0;  i < nTopWrite; i++ )
    {
        if( IOBuffer[i].canID == canID ) /* checking send buffers */
        {
            CANErrno = CAN_ERR_REOPEN;
            return 0;  /* error: re-open is not possible */
        }
    }

    return 1;

}


/*******************************************************************************
*
* NAME: SetCANID
*
* DESCRIPTION: Decompose 11/29-bit CAN ID into the registers 
*
********************************************************************************
* PARAMETERS:   Address of the registers, CAN ID
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
void SetCANID( volatile unsigned short* pAddr, CANADDRESSTYPE canID, unsigned long addr_flags )
{
    asm( brclr #CAN_SHORT_IDE_MASK, B1, can2a );
    asm( asl  A );

    asm( move.w A1, X0 );
    asm( bfclr #0xFFF8, X0 );  /* ID[18-17] */
    asm( or.w X0, B );

    asm( move.w A1, X0 );     /* ID[29-18] */
    asm( asll.w  #0x2, X0 );
    asm( bfclr #0x001F, X0 );
    asm( or.w X0, B );

    asm( move.w  B1, X:(R2)+ );
    asm( move.w  A0, X:(R2)+ );

    asm( rts );

can2a:
    asm( asll.l  #0x5, A );
    asm( move.w  A0, X:(R2)+ );
    asm( clr.w   X:(R2)+ );

}

/*******************************************************************************
*
* NAME: GetCANID
*
* DESCRIPTION: Compose 11/29-bit CAN ID from the registers values
*
********************************************************************************
* PARAMETERS:   Raw ID 
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
CANADDRESSTYPE GetCANID( unsigned long id )
{
    asm( brclr # CAN_SHORT_IDE_MASK, A1, can2a );

    asm( bfclr    # (CAN_SHORT_IDE_MASK|CAN_SHORT_SRR_MASK), A1);  /* clear srr, ide */
    asm( brclr    #0x0002, A1, Bit_16 ); /* bit 16th of id */
    asm( bfset    #CAN_SHORT_IDE_MASK, A1);
Bit_16: 
    asm( brclr    # 0x0004, A1, Bit_17 );  /* bit 17th of id */
    asm( bfset    # CAN_SHORT_SRR_MASK, A1);
Bit_17: 
    asm( asr  A );  /* bit_15 -> A0  */
    asm( move.w A0, B0 );
    asm( lsrr.w  #2, A ); /* skip bit 16,17 */
    asm( move.w B0, A0 );
    asm( rts );
can2a:
    asm( lsrr.l  #21, A );
}


/*******************************************************************************
*
* NAME: PrepareCANID
*
* DESCRIPTION:  prepare buffer for reading or write
*              
********************************************************************************
* PARAMETERS:       buffer descriptor
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:  
*
* DEPENDENCIES:     None
*******************************************************************************/
void PrepareCANID( can_sBuffer* pbuf, unsigned int cmd )
{
    asm( move.w X:(R2 + #sFlexCAN_offset_flags), X0 );
    asm( move.w  X0, Y1 );
    asm( bfclr #0xFFF0, X0 );    /* clear flags */
    asm( asll.w  #0x3, X0 ); /* x8, convert to buffer index */

    asm( adda #( BSP_PERIPH_BASE + archoff_FlexCAN_Buffer_0 ), X0, R4 ); 
    
    asm( move.w #FC_MB_NOTACTIVE, X:(R4 + devoff_FlexCANBuffer_CtlStatusReg) );

    asm( move.l  X:( R2 + sFlexCAN_offset_canID), A );
    asm( adda   # devoff_FlexCANBuffer_IdRegHi, R4, R2 );

    asm( clr B );   /* form flags on B register */
    asm( brset    # CAN_BUF_READ_HANDLE, Y1, Bit_17 ); 
    asm( bfset    # CAN_SHORT_SRR_MASK, B1);  /* only for write handle */
Bit_17: 
    asm( brclr    # CAN_BUF_2B, Y1, Bit_16 ); 
    asm( bfset    #CAN_SHORT_IDE_MASK, B1); /* only for long handles */
Bit_16: 

    asm( jsr SetCANID ); /* used A,B, X0 */

    asm( move.w Y0, X:(R4 + devoff_FlexCANBuffer_CtlStatusReg) );
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_TimerReg), X0 );
}

/*******************************************************************************
*
* NAME: flexcanPreOpen
*
* DESCRIPTION: driver Open procedure: open CAN device,  receive and send CAN ID
*              
********************************************************************************
* PARAMETERS:   device name, open flags, CAN ID
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: following I/O flags accepted: 
*                   O_RDONLY - Message buffer is opened for READ, 
*                   O_WRONLY - Message buffer is opened for WRITE, 
*                               transmission mode is synchronous, 
*                   O_WRONLY | O_NON_BLOCK - Message buffer is opened for WRITE,
*                               transmission mode is asynchronous
*
* DEPENDENCIES: None
*******************************************************************************/
handle_t  flexcanPreOpen(const char * pName, int oFlags, ...)
{
	int i;
	va_list    Args;
	CANADDRESSTYPE canID;
    can_sBuffer* pBuf;

	Args = (char *)&oFlags;
	canID = va_arg(Args, CANADDRESSTYPE);
	va_end(Args);

    if( pName != BSP_DEVICE_NAME_CAN_0 )
    {
        return (handle_t ) -1;
    }

    if( nBottomRead == nTopWrite ) /* check buffers are available */
    {
        CANErrno = CAN_ERR_NO_BUFFERS;
        return (handle_t) -1; /* error: no more driver buffers */
    }

    /***    open for read ( reception )   ***/  
    if( oFlags & O_RDONLY   )
    {
        if( !CheckCANID( canID, FlexCANFlags & O_LOOP_BACK )  ) 
        {
             CANErrno = CAN_ERR_REOPEN;
             return (handle_t )-1; /* error: no more driver buffers */
        }
    }
    /***   open for write ( transmission )  ***/    
    else if( oFlags & O_WRONLY )
    {

        if( !CheckCANID( canID, (FlexCANFlags & O_LOOP_BACK) >> 1 )   ) 
        {
            CANErrno = CAN_ERR_REOPEN;
            return (handle_t )-1; /* error: ID has been opened yet */
        }
    }
    else
    {
        CANErrno = CAN_ERR_PARAMETER;
        return (handle_t ) -1; /* flag error */
    }

    pBuf = (can_sBuffer*)flexcanOpen( pName, oFlags, canID );
    pBuf->drvInterface = (io_sInterface *)&canInterfaceVT;  /* interfaceCAN_Tx; */

    return (handle_t)pBuf;
}




/*******************************************************************************
*
* NAME: flexcanOpen
*
* DESCRIPTION: driver Low Open procedure: initialize buffer
*              
********************************************************************************
* PARAMETERS:   device name, open flags, CAN ID
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: 
*
* DEPENDENCIES: None
*******************************************************************************/
handle_t  flexcanOpen(const char * pName, int oFlags,  CANADDRESSTYPE canID )
{
    can_sBuffer* pbuf;
    unsigned int cmd;

    /***    open for read ( reception )   ***/  
    if( oFlags & O_RDONLY   )
    {
        nBottomRead--; 
        pbuf = &IOBuffer[nBottomRead];
        pbuf->flags = (unsigned int)nBottomRead | CAN_BUF_READ_HANDLE;
        cmd = FC_MB_ACTIVE;
        periphBitSet( (1 << nBottomRead),  &ArchIO.CAN.IntMask1Reg );
    }
    /***   open for write ( transmission )  ***/    
    else if( oFlags & O_WRONLY )
    {
        pbuf = &IOBuffer[nTopWrite]; /* CAN_BUF_WRITE_HANDLE */
        pbuf->flags = (unsigned int)nTopWrite;
        cmd = FC_MB_NOTREADY;

        nTopWrite++;  /* next buffer */

    }
    if( !(oFlags & O_NONBLOCK) )
    {
        pbuf->flags |= CAN_BUF_SYNCHRONOUS;
    }
    pbuf->canID = canID; 

    if( FlexCANFlags /* oFlags*/ & O_CAN_2B )
    {
        pbuf->flags |= CAN_BUF_2B;
    }

    PrepareCANID( pbuf, cmd );

    return (handle_t)pbuf; 
    
}


/*******************************************************************************
*
* NAME: Write
*
* DESCRIPTION: Driver write procedure
*              
********************************************************************************
* PARAMETERS:   file descriptor, buffer and size
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: handler is pointer to the buffer, 
*                 truncation is possible when porting
*
* DEPENDENCIES: None 
*******************************************************************************/
ssize_t flexcanWrite( handle_t handle,  const void *pBuffer, size_t size )
{
#if __m56800E_lmm__ 
    asm(move.w A0,A);
    asm(move.w  A1,Y0);
#else
	asm(nop);
	asm(nop);
#endif
    asm( move.w X:(R2 + #sFlexCAN_offset_flags), X0 );
    asm( bfclr #0xFFF0, X0 );    /* clear flags */

    asm( move.w    #1, A );
    asm( asll.w    X0, A );      /* get mask for buffer */

    asm( asll.w  #0x3, X0 ); /* *8, convert to buffer index */
    asm( adda #( BSP_PERIPH_BASE + archoff_FlexCAN_Buffer_0 ), X0, R4 ); 
    /* R4 - message buffer */

    asm( brclr #4, X:( BSP_PERIPH_BASE + archoff_FlexCAN_StatusReg ), continue_1 );
    asm( bfset #4, X:( BSP_PERIPH_BASE + archoff_FlexCAN_StatusReg ) );

#if __m56800E_lmm__ 
    asm(move.l #0xFFFFFFFF,A);
#else
    asm( move.w #0xFFFF, Y0 ); 
#endif

    asm( move.w # CAN_ERR_LOST, CANErrno );
    asm( rts );
continue_1:    
    /* transmission finished ? */
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ), X0 );
    asm( and.w A, X0 );  /* compare with mask */
    asm( beq continue_2 );
#if __m56800E_lmm__ 
    asm(move.l #0xFFFFFFFF,A);
#else
    asm( move.w #0xFFFF, Y0 ); 
#endif
    asm( move.w #CAN_ERR_BUSY, CANErrno );
    asm( rts );
continue_2:

    asm( move.w # FC_MB_NOTREADY, X:(R4 + devoff_FlexCANBuffer_CtlStatusReg) );

    asm(cmp.w       #8, Y0);
    asm(blt         len_lt);    /* change by received len only */
    asm(move.w      #8, Y0);  
len_lt:
    asm( move.w  Y0, X0 );
    asm( inc.w      X0  );
    asm( ror.w      X0  );      /* words to copy */

    asm( adda     # devoff_FlexCANBuffer_DataSegReg_0, R4, R0 );

    asm( do X0, enddo );
    asm( move.bp  X:(R3)+, Y1 ); /* copy from pBuffer */
    asm( asll.w   #8, Y1 );
    asm( moveu.bp  X:(R3)+, X0 ); /* copy from pBuffer */
    asm( or.w   X0, Y1 );
    asm( move.w Y1, X:(R0)+ );  /* copy to hardware */
enddo:

    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntFlags1Reg ), X0 );
    asm( and.w A, X0 );  /* to clear isr flag */
    asm( move.w X0, X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntFlags1Reg ) );

    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ), X0 );
    asm( or.w A, X0 );  /* to update isr mask */
    asm( move.w X0, X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ) );
 
    asm( bfset # FC_MB_SEND, Y0 );
    asm( move.w Y0, X:(R4 + devoff_FlexCANBuffer_CtlStatusReg) );
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_TimerReg), X0 );

    asm( brclr CAN_BUF_SYNCHRONOUS, X:(R2 + #sFlexCAN_offset_flags), endwrite );
wait_tx:    /* wait in blocked mode only */

    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntFlags1Reg ), X0 );
    asm( and.w A, X0 );
    asm( beq  wait_tx );
    
endwrite:
    asm( bfclr # 0xFFF0, Y0 ); /* return actual length */
#if __m56800E_lmm__ 
    asm(move.w Y0,A0);
    asm(clr.w A1);
#else
    asm(nop);
    asm(nop);
#endif
  
}

/*******************************************************************************
*
* NAME: Read
*
* DESCRIPTION: Driver read procedure
*              
********************************************************************************
* PARAMETERS:   file descriptor, buffer and size
*
* SIDE EFFECTS: None Known
*
* DESIGNER NOTES: handler is pointer to the CAN ID descriptor, 
*               
* DEPENDENCIES: None
*******************************************************************************/
ssize_t flexcanRead( handle_t handle,  void *pBuffer, size_t size)
{
#if __m56800E_lmm__ 
    asm(move.w A0,A);
    asm(move.w  A1,Y0);
#else
	asm(nop);
	asm(nop);
#endif
    asm( move.w X:(R2 + #sFlexCAN_offset_flags), X0 );
    asm( bfclr #0xFFF0, X0 );    /* clear flags */

    asm( move.w    #1, C );
    asm( asll.w    X0, C );      /* get mask for buffer */

    asm( asll.w  #0x3, X0 ); /* *8, convert to buffer index */

    /* R3 - message buffer */
    asm( adda #( BSP_PERIPH_BASE + archoff_FlexCAN_Buffer_0 ), X0, R4 ); 
    asm( move.w X:(R4 + devoff_FlexCANBuffer_CtlStatusReg), X0 );


    asm( move.w  X0, Y1 );
    asm( bfclr #0xFF0F, Y1 );    /* left only len */
    asm( cmp.w # FC_MB_FULL,  Y1 );
    asm( beq continue_read );
    asm( cmp.w  # FC_MB_OVERRUN,  Y1 ); 
    asm( beq continue_read );

    asm( clr.w Y0 ); 

#if __m56800E_lmm__ 
    asm(clr.w A0);
    asm(clr.w A1);
#else
    asm(nop);
    asm(nop);
#endif

    
    asm( rts );
continue_read: /* mb full or overrun continue read */

    asm( move.w X:(R4 + devoff_FlexCANBuffer_IdRegHi), A ); /* A - ID */
    asm( move.w X:(R4 + devoff_FlexCANBuffer_IdRegLo), A0 );
    asm( jsr GetCANID );    /* A - raw CAN ID -> CAN ID */
    asm( cmp.w   #(8 + 4), Y0 );  /* compare with ext buffer len, Y0 - 12 */
    asm( blt        IDcopy);    /* change by received len only */
    asm(  adda     #8, R3, R0 );
    asm(  asra   R0 );
    asm(  move.w A0, X:(R0)+ );    /* copy ID to tail of long buffer */
    asm(  move.w A1, X:(R0)+ );
IDcopy:

    asm( bfclr #0xFFF0, X0 );    /* left only len */

    asm( cmp         X0, Y0);    /* Y0 - X0 */
    asm( blt         len_lt);    /* change by received len only */
    asm( move.w      X0, Y0);
len_lt:
    asm( adda     # devoff_FlexCANBuffer_DataSegReg_0, R4, R0 );
    asm( move.w  Y0, X0 );
    asm( asr     X0  );      /* words to copy */

    asm( do X0, enddo );
    asm( move.w  X:(R0)+, X0 );
    asm( move.w  X0, Y1 );
    asm( asrr.w   #8, X0 );
    asm( move.bp  X0, X:(R3)+ );
    asm( move.bp  Y1, X:(R3)+ );
enddo:
    asm( brclr #0x0001, Y0, skip_odd_last );
    asm( move.w X:(R0)+, X0 );
    asm( asrr.w   #8, X0 );
    asm( move.bp  X0, X:(R3)+ );
skip_odd_last:


    asm( move.l  X:( R2 + sFlexCAN_offset_canID), B );
    asm( cmp.l A, B );
    asm( beq id_compare );

    asm( move.w X:(R2 + #sFlexCAN_offset_flags), X0 ); /* all flags */
    asm( tfr  B, A );    /* needed CAN ID -> A */

    asm( clr B );   /* form flags on B register */
    asm( brclr    # CAN_BUF_2B, X:(R2 + #sFlexCAN_offset_flags), Bit_16 ); 
    asm( bfset    #CAN_SHORT_IDE_MASK, B1); /* only for long handles */
Bit_16: 

    asm( adda   # devoff_FlexCANBuffer_IdRegHi, R4, R2 ); /* prepare addr for SetCANID */
    asm( jsr SetCANID ); /*  SetCANID(R2, A, B) & used X0 */

id_compare:

    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntFlags1Reg ), X0 );
    asm( and.w C, X0 );  /* to clear isr flag */
    asm( move.w X0, X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntFlags1Reg ) );

    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ), X0 );
    asm( or.w C, X0 );  /* to update isr mask */
    asm( move.w X0, X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ) );

    asm( move.w #FC_MB_ACTIVE, X:(R4 + devoff_FlexCANBuffer_CtlStatusReg) );
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_TimerReg), X0 );
#if __m56800E_lmm__ 
    asm(move.w Y0,A0);
    asm(clr.w A1);
#else
    asm(nop);
    asm(nop);
#endif
}

/*******************************************************************************
*
* NAME: ioctlCANID_GET_STATUS
*
* DESCRIPTION: Driver i/o control procedure - Gets status of message buffer
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   returned status value can be one of the following:
*                           CANID_EMPTY - no messages in the message buffer
*                           CANID_FULL - a message is in the message buffer
*                           CANID_OVERFLOW - a  message buffer is full
*                                      but some received messages were lost 
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCANID_GET_STATUS(handle_t handle, unsigned long params )      
{
    asm( move.w X:(R2 + #sFlexCAN_offset_flags), X0 );
    asm( bfclr #0xFFF0, X0 );    /* clear flags */
    asm( move.w    #1, B );
    asm( asll.w    X0, B );      /* get mask for buffer */

    asm( move.w # CANID_EMPTY,  Y0 ); /* default status */
    asm( brclr CAN_BUF_READ_HANDLE, X:(R2 + #sFlexCAN_offset_flags), tx_case );

    asm( asll.w  #0x3, X0 ); /* x8, convert to buffer index */
    asm( adda #( BSP_PERIPH_BASE + archoff_FlexCAN_Buffer_0 ), X0, R4 ); 
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntFlags1Reg ), X0 );

    asm( and.w X0, B );         /* check bit in flag register */
    asm( beq end_status );      /* empty buffer if was no reception */

    asm( move.w X:(R4 + devoff_FlexCANBuffer_CtlStatusReg), Y1 );
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_TimerReg), X0 );
    asm( bfclr #0xFF0F, Y1 );    /* left only raw status */

    asm( move.w # CANID_FULL, Y0 );
    asm( cmp.w # FC_MB_FULL,  Y1 );
    asm( beq end_status );

    asm( move.w # CANID_OVERFLOW,  Y0 );
    asm( cmp.w  # FC_MB_OVERRUN,  Y1 ); 
    asm( beq end_status );

    asm( move.w # CANID_EMPTY,  Y0 ); /* for case of interim code */
    asm( bra end_status );

tx_case:
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ), X0 );
    asm( and.w X0, B );    /* check bit in flag register */
    asm( beq end_status );
    asm( move.w # CANID_FULL ,  Y0 );

end_status:
    ;

}


/*******************************************************************************
*
* NAME: ioctlCAN_RESET
*
* DESCRIPTION: Driver i/o control procedure - Reset CAN Driver
*              
********************************************************************************
* PARAMETERS:       file descriptor,  parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:  
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCAN_RESET(handle_t FileDesc, unsigned long params )
{
    register int i;

    FC_Init();

    for( i = nBottomRead;  i  < 16; i++ )
    {
         /* MB is active for receive data */
        periphMemWrite( FC_MB_ACTIVE,   &ArchIO.CAN.Buffer[i].CtlStatusReg );   
    }

    return 0;
}


/*******************************************************************************
*
* NAME: ioctlCAN_SET_SLEEP
*
* DESCRIPTION: Driver i/o control  - Sets CAN into SLEEP(power save) mode
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCAN_SET_SLEEP(handle_t FileDesc, unsigned long params )
{
    periphBitSet(CAN_STOP, &ArchIO.CAN.ConfigReg ); /* goto low power sleep mode */
    return (unsigned int)periphBitTest( CAN_STOP_ACK, &ArchIO.CAN.ConfigReg ) ;
}

/*******************************************************************************
*
* NAME: ioctlCAN_SET_WAKEUP
*
* DESCRIPTION: Driver i/o control procedure - Wakeups CAN
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCAN_SET_WAKEUP(handle_t FileDesc, unsigned long params )
{
    periphBitSet(CAN_SELF_WAKE, &ArchIO.CAN.ConfigReg ); /* goto wake up mode */
    return 0;
}

/*******************************************************************************
*
* NAME: ioctlCAN_GET_STATUS
*
* DESCRIPTION: Driver i/o control procedure - Gets status of CAN control registers
*              
********************************************************************************
* PARAMETERS:       file descriptor, command and parameters list
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   returned status value is composed of CANCTL0 + CANRFLG
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCAN_GET_STATUS(handle_t FileDesc, unsigned long params )
{
    unsigned int status;

    status = periphMemRead( &ArchIO.CAN.StatusReg );
    if( !(status & 0xFC00) )
        status |= 0x0008;

    return status;
}

/*******************************************************************************
*
* NAME:             Close
*
* DESCRIPTION:      Driver close procedure
*              
********************************************************************************
* PARAMETERS:       file descriptor
*
* SIDE EFFECTS:     filters is staying open
*
* DESIGNER NOTES:   close corresponding message buffer
*
* DEPENDENCIES:     None
*******************************************************************************/
int flexcanClose(handle_t handle)
{
    unsigned int i = ((can_sBuffer*)handle)->flags & 0x000F;

    if ( ((can_sBuffer*)handle)->flags & CAN_BUF_READ_HANDLE )
    {
        if( i == nBottomRead )
        {
            nBottomRead++;
            ((can_sBuffer*)handle)->flags = 0;
        }
        else
        {
            return -1;
        }
    }
    else 
    {
        if( i == nTopWrite - 1 ) /* correct */
        {
            nTopWrite--;
            ((can_sBuffer*)handle)->flags = 0;
        }
        else
        {
            return -1;
        }
    }
    periphMemWrite( 0, &ArchIO.CAN.Buffer[i].CtlStatusReg );
    periphBitClear( (1 << i),  &ArchIO.CAN.IntMask1Reg );
    periphBitSet( (1 << i),  &ArchIO.CAN.IntFlags1Reg );

    return 0;
}


/*******************************************************************************
*
* NAME: ioctlCANID_SET_MSAK
*
* DESCRIPTION: update mask according to the buffer
*              
********************************************************************************
* PARAMETERS:       file descriptor, mask
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   only for 14 and 15 mask
*
* DEPENDENCIES:     None
*******************************************************************************/
unsigned int ioctlCANID_SET_MASK(handle_t handle, unsigned long params )      
{
    unsigned int i;
    unsigned long maskflag;

    i = ((can_sBuffer*)handle)->flags & 0x000F;
	maskflag = 0;
	if( ((can_sBuffer*)handle)->flags & CAN_BUF_2B )
	{
	    maskflag =   CAN_LONG_IDE_MASK ;
    }

    if( i == 15 )
    {
        SetCANID( (unsigned short*)(BSP_PERIPH_BASE + archoff_FlexCAN_Rx15MaskHiReg ), params, maskflag );
    }
    else if( i == 14 )
    {
        SetCANID( (unsigned short*)(BSP_PERIPH_BASE + archoff_FlexCAN_Rx14MaskHiReg ), params, maskflag );
    }
    else
    {
        SetCANID( (unsigned short*)(BSP_PERIPH_BASE + archoff_FlexCAN_RxGlobalMaskHiReg ), params, maskflag );
    }
    return 0;
}


/*******************************************************************************
*
* NAME: CANReceiveISR
*
* DESCRIPTION: receive ISR handler
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: Slowest handler in the driver, impacts to the interrupt lattency
*
* DESIGNER NOTES: copy is not effective on C, recomended to use asm
*
* DEPENDENCIES: None
*******************************************************************************/
#pragma interrupt called on
void CANReceiveISR(void)
{
    register  can_sData* pCrd;

    pCrd = (*CANRcvPrepare)( 
                GetCANID(
                        ((unsigned long)periphMemRead( &ArchIO.CAN.Buffer[15].IdRegHi)) << 16
                        | periphMemRead( &ArchIO.CAN.Buffer[15].IdRegLo)
                )   
           ); /* CANStdRcvPrepare */
    if( pCrd ) 
    {   
        pCrd->length =  flexcanRead( (handle_t)&IOBuffer[15], 
                        &pCrd->data[0], 8 );
    }
    else 
    {   /* just release */
         flexcanRead( (handle_t)&IOBuffer[15], &pCrd->data[0], 0 ); 
    }
}
#pragma interrupt called off

/*******************************************************************************
*
* NAME:             FC_InitCallBack
*
* DESCRIPTION:      Initialize CAN driver device for callback
*              
********************************************************************************
* PARAMETERS:       
*
* SIDE EFFECTS:     None Known
*
* DESIGNER NOTES:   only 15th buffer is used
*
* DEPENDENCIES:     None
*******************************************************************************/
void FC_InitCallBack( CANADDRESSTYPE mask, CANADDRESSTYPE canID )
{
    flexcanOpen( 0, O_RDONLY, canID ); /* use last buffer with own filter */
    ioctlCANID_SET_MASK((handle_t)&IOBuffer[15], mask );
    periphMemWrite( 0x8000, &ArchIO.CAN.IntMask1Reg ); /* 15th buffer */
    IOBuffer[15].callback = CANReceiveISR;
}


/*******************************************************************************
*
* NAME: CANReceiveISR
*
* DESCRIPTION: receive ISR handler
*
********************************************************************************
* PARAMETERS:   
*
* SIDE EFFECTS: Slowest handler in the driver, impacts to the interrupt lattency
*
* DESIGNER NOTES: copy is not effective on C, recomended to use asm
*
* DEPENDENCIES: None
*******************************************************************************/
void CANBufferISR(void)
{
    asm( alignsp );	/* saves needed registers only */
    asm( move.l  Y , x:(SP)+ );
    asm( move.l  X0, x:(SP)+ );
    asm( move.l  R2, x:(SP)+ );
    asm( move.l  N, x:(SP) );

    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ), Y0 );
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntFlags1Reg ), Y1 );
    asm( and.w  Y1, Y0 );           /* actual isr vector */
    asm( clr.w Y1 );
    asm( clb  Y, X0 );              /* left buffer index (Y0) */
    asm( move.w #0x1E, Y0 );
    asm( sub  X0, Y0 );      
    asm( blt end_bufisr );
    asm( move.w    #0x0001, X0 );   /* 0th mask (X0) */
    asm( asll.w    Y0, X0 );        /* actual buffer mask (X0) */
   
    asm( move.w X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ), Y1 );
    asm( not.w X0 );       /* to clear actual mask */
    asm( and.w X0, Y1 );   /* only current ISR mask clear */
    asm( move.w Y1, X:( BSP_PERIPH_BASE + archoff_FlexCAN_IntMask1Reg ) );   /* release mask */

    asm( move.w #sFlexCAN_word_size, Y1 ); /* get status */
    asm( impy.w Y1, Y0, X0 );          /* Y0*Y1 get contex offset */

    asm( adda #IOBuffer, X0, R2 );  /* get context buffer address */
    asm( tst.l X:(R2 + sFlexCAN_offset_callback) );
    asm( beq end_bufisr );
    asm( move.l X:(R2 + sFlexCAN_offset_callback), N ); /* get callback address */
    asm( jsr (N) );
end_bufisr:

    asm( move.l  x:(SP)-, N );
    asm( move.l  x:(SP)-,R2 );
    asm( move.l  x:(SP)-,X0 );
    asm( move.l  x:(SP)-,Y );
    asm( move.l  x:(SP)-,SP );

    asm(  rti );
}




