/*******************************************************************************
*
* Motorola Inc.
* (c) Copyright 2002 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
* $Element: /project/dsp568_sdk/sdk/src/dsp56838evm/nos/bsp/timerdrv.c $ 
* $Author: saa $ 
* $Revision: /main/2 $ 
* $VOB: /project/dsp568_sdk $ 
* $OS: solaris $ 
*
* Description:       source file for the 56838 posix timer driver 
*
* Notes:  
*
******************************************************************************/
#include "port.h"
#include "bsp.h"
#include "arch.h"

#include "timerdrv.h"
#include "timer.h"

static unsigned long TimerISRCount; /* system timer counter */

/* Local prototypes */
static void    timerRealTimeClockISR(union sigval Params); 
 
/* QT ISR prototypes */
extern posix_tContext   posix_ctx_A_0;
extern posix_tContext   posix_ctx_A_1;
extern posix_tContext   posix_ctx_A_2;
extern posix_tContext   posix_ctx_A_3;
extern posix_tContext   posix_ctx_B_0;
extern posix_tContext   posix_ctx_B_1;
extern posix_tContext   posix_ctx_B_2;
extern posix_tContext   posix_ctx_B_3;
extern posix_tContext   posix_ctx_C_0;
extern posix_tContext   posix_ctx_C_1;
extern posix_tContext   posix_ctx_C_2;
extern posix_tContext   posix_ctx_C_3;
extern posix_tContext   posix_ctx_D_0;
extern posix_tContext   posix_ctx_D_1;
extern posix_tContext   posix_ctx_D_2;
extern posix_tContext   posix_ctx_D_3;

/*** Quadrature timer Normal ISR template ***/
#if 0
#define INSTQTISR( isr, base, device )                                  \
void isr(void)                                                          \
{																		\
    extern const union sigval sig;										\
    if(  --(device.SoftCounter) == 0 )									\
    {																	\
        device.SoftCounter = device.reloadSoftCounter;					\
        if(device.pUserFunc != 0 )										\
        device.pUserFunc( sig );										\
    }																	\
	if( (device.reloadSoftCounter == 0) && (device.SoftCounter < 2))	\
         	 periphBitSet(QTB_ONCE,(volatile unsigned short*)(base));	\
}
#else
#define POUND #
#define INSTQTISR( isr, base, device )                              \
void isr(void)                                                      \
{																	\
    asm(	adda		#0x0000002 , SP							);	\
    asm(	move.l		A10 , X:(SP)+							);	\
    asm(	move.l		R2  , X:(SP)							);	\
    asm(	move.w		device + SoftCounter_offset , A			);	\
    asm(	dec.w		A 										);	\
    asm(	bne			nextsoftstep							);	\
    asm(	tst.w 		device + pUserFunc_offset				);	\
    asm(	beq			nextsoftstep							);	\
    asm(	move.l		device + pUserFunc_offset , R2			);	\
    asm(	adda		#2 , SP									);	\
    asm(	move.w		#0 , X:(SP)								);	\
    asm(	jsr			(R2)									);	\
    asm(	suba		#2 , SP									);	\
    asm(	move.w		device + reloadSoftCounter_offset , A1	);	\
    asm(	nextsoftstep:										);	\
    asm(	move.w		A1 , device + SoftCounter_offset		);	\
    asm(	tst.w		device + reloadSoftCounter_offset		);	\
    asm(	bne 		endstep									);	\
    asm(	move.w		device + SoftCounter_offset , A1		);	\
    asm(	cmp.w 		#1 , A									);	\
    asm(	bne 		endstep									);	\
    asm(	move.l		POUND base , R2					 		);	\
    asm(	move.w		X:(R2-0x1000) , A1						);	\
    asm(	bfset		0x0040 , A								);	\
    asm(	move.w		A1 , X:( R2 - 0x1000 )					);	\
    asm(	endstep:											);	\
    asm(	move.l		X:(SP)- , R2							);	\
    asm(	move.l		X:(SP)- , A								);	\
	}
#endif

/* Quadrature timer isr service routines */
INSTQTISR( POSIXtimerISRA0, archoff_TimerA0_ControlReg, posix_ctx_A_0 )
INSTQTISR( POSIXtimerISRA1, archoff_TimerA1_ControlReg, posix_ctx_A_1 )
INSTQTISR( POSIXtimerISRA2, archoff_TimerA2_ControlReg, posix_ctx_A_2 )
INSTQTISR( POSIXtimerISRA3, archoff_TimerA3_ControlReg, posix_ctx_A_3 )

#if !defined(DSP56838EVM)
INSTQTISR( POSIXtimerISRB0, archoff_TimerB0_ControlReg, posix_ctx_B_0 )
INSTQTISR( POSIXtimerISRB1, archoff_TimerB1_ControlReg, posix_ctx_B_1 )
INSTQTISR( POSIXtimerISRB2, archoff_TimerB2_ControlReg, posix_ctx_B_2 )
INSTQTISR( POSIXtimerISRB3, archoff_TimerB3_ControlReg, posix_ctx_B_3 )
#endif /* !defined(DSP56838EVM) */

INSTQTISR( POSIXtimerISRC0, archoff_TimerC0_ControlReg, posix_ctx_C_0 )
INSTQTISR( POSIXtimerISRC1, archoff_TimerC1_ControlReg, posix_ctx_C_1 )
INSTQTISR( POSIXtimerISRC2, archoff_TimerC2_ControlReg, posix_ctx_C_2 )
INSTQTISR( POSIXtimerISRC3, archoff_TimerC3_ControlReg, posix_ctx_C_3 )

INSTQTISR( POSIXtimerISRD0, archoff_TimerD0_ControlReg, posix_ctx_D_0 )
INSTQTISR( POSIXtimerISRD1, archoff_TimerD1_ControlReg, posix_ctx_D_1 )
INSTQTISR( POSIXtimerISRD2, archoff_TimerD2_ControlReg, posix_ctx_D_2 )
INSTQTISR( POSIXtimerISRD3, archoff_TimerD3_ControlReg, posix_ctx_D_3 )

#define POSIX_TIMER_READ_PRESCALER_VALUE( bspDevice, Dummy ) \
          ( 1<< (timerStdFastStart.bits.InputSource & 0xFFF8 )   )


#define POSIX_TIMER_GET_CONTEXT(TimerID) \
((posix_tContext*)timer_qt_readCompareReg1( (arch_sTimerChannel*) TimerID ) )


extern const        qt_sQuadState  timerStdFastStart;


/*****************************************************************************
*
* Module: U64addU32
*
* Description: addition unsigned 64 and unsigned 32 bits numbers
*
* Returns: none
*
* Arguments: pFirst     - pointer to unsigned U64 value
*            second     - 32 bits unsigned number
*            pRezult    - not null pointer to U64 value
*
* Range Issues: 
*
* Special Issues:  
*           Register discription:
*                       R2 - pointer to U64 value
*                       A  - 32 bits number
*                       R3 - not null pointer to U64 value
*
*                       Module doesn't save register: B
*
* if overflow occurs then carry bit will be set
*
* Test Method: 
*
*****************************************************************************/
static void U64addU32(const U64 * pFirst, const unsigned long second, U64 * pResult)
{
    asm(adda    #1, SP      );

    asm(move.w  A0, X:(SP)+ );
    asm(move.w  A1, X:(SP)+ );
    asm(move.w  B0, X:(SP)+ );
    asm(move.w  B1, X:(SP)  );
    

    asm(move.l  X:(R2), B   );      /* Prepare data for addition least 32-bits 	*/
    asm(add     A, B        );      /* Add least 32-bits  						*/
    asm(move.l  B10, X:(R3) );      /* Save least 32-bits 						*/                               
    asm(move.l  X:(R2+2), B );      /* copy high 32-bits to result 				*/
    asm(move.l  #0, Y       );
    asm(adc     Y, B        );
    asm(move.l  B10, X:(R3+2));     /* Save least 32-bits 						*/
    
    asm(move.w  X:(SP)-, B1 );
    asm(move.w  X:(SP)-, B0 );
    asm(move.w  X:(SP)-, A1 );
    asm(move.w  X:(SP)-, A0 );
        
    asm(rts                 );
}

/*****************************************************************************
*
* Module: U64mulU32
*
* Description: multiplication unsigned 64 and unsigned 32 bits multiplier
*
* Returns: none
*
* Arguments: pFirst     - pointer to U64 value
*            second     - 32 bits multiplier
*            pRezult    - not null pointer to U64 value
*
* Range Issues: 
*
* Special Issues:  
*           Register discription:
*                       R2 - pointer to U64 value
*                       A  - 32 bits number
*                       R3 - not null pointer to U64 value
*
*                       Module doesn't save registers: B, Y
*
* Return: return higest bits of results (64..96 bits) in register A
*
* Test Method: 
*
*****************************************************************************/
static unsigned long U64mulU32(const U64 * pFirst, const unsigned long second, U64* pRezult)
{       
    asm(adda    #1, SP      );

    asm(move.w  B0, X:(SP)+ );
    asm(move.w  B1, X:(SP)+ );
    asm(move.w  Y0, X:(SP)+ );
    asm(move.w  Y1, X:(SP)  );

    asm(move.l  #0, Y       ); /* prepare register to use as carry number			*/
                
    asm(do      #4, ENDDO   ); /* multiply step by step evry 16-bits of U64 		*/
    asm(move.w  X:(R2), B0  ); /* take current 16-bits of U64 (start from least) 	*/            
    asm(impyuu  A0, B0, B   ); /* multiply least part of multiplier 				*/
    asm(add     Y, B        ); /* add carry number 									*/
    asm(move.w  B0, Y0      ); /* save 16 bits of resuls 							*/
    asm(move.w  X:(R2)+, B0 ); /* take next 16-bits of U64 							*/
    asm(move.w  Y0, X:(R3)+ ); /* save our current result to memory ( it's use to realise such possibility U64mul32(&a, b, &a)) */
    asm(impyuu  A1, B0, Y   ); /* multiply high part of multiplier 					*/
    asm(move.w  B1, B0      ); /* sum up carry number 								*/
    asm(clr.w   B1          );
    asm(add     B, Y        );
    
    asm(ENDDO:              );  

    asm(move.w  Y0, A0      ); /* if overflow occures, the higest bits will be in register A (return value) */
    asm(move.w  Y1, A1      );

    asm(adda    #-4, R2     );
    asm(adda    #-4, R3     );
    
    asm(move.w  X:(SP)-, Y1 );
    asm(move.w  X:(SP)-, Y0 );  
    asm(move.w  X:(SP)-, B1 );  
    asm(move.w  X:(SP)-, B0 );  
    
    asm(rts                 ); 
};

/*****************************************************************************
*
* Module: incU64
*
* Description: increase U64
*
* Returns: carry bit set if pU64 == 0xFFFF FFFF FFFF FFFF
*
* Arguments: pU64 - not null pointer to 64 bits dividend
*
*
* Range Issues: 
*
* Special Issues:  For executive loop only. Module doesn't save register: A
*                                           
* Test Method: 
*
*****************************************************************************/
static void incU64(U64 * pU64)
{                       
    asm(adda    #1, SP      			);

    asm(move.w  A0, X:(SP)+ 			);
    asm(move.w  A1, X:(SP)  			);
                                                            
    asm(move.l  X:(R2), A           	);  /* movel low bits to A */
    asm(inc.l   A                   	);  /* increase A */
    asm(move.l  A10, X:(R2)         	);  /* save result */
    asm(move.l  X:(R2+2), A         	);  /* movel high bits to A */
    asm(brclr   #1, SR, CarryBitClear   );  /* test carry bit ; carry bit set if overflow occurs*/
    asm(inc.l   A                   	);  /* inc hifh bits if overflow occurs */
            
    asm(CarryBitClear:              	);
    asm(move.l  A10, X:(R2+2)       	);  /* save high bits */
    
    asm(move.w  X:(SP)-, A1         	);
    asm(move.w  X:(SP)-, A0         	);

    asm(rts                         	);
}

/*****************************************************************************
*
* Module: U32divU16s
*
* Description: division unsigned 32 bits value by unsigned 16 bits value divider
*
* Returns: quotient
*
* Arguments: Dividend - not null pointer to 32 bits dividend
*            Divisor    - 16 bits unsigned divider
*
* Range Issues: 
*
* Special Issues:  For executive loop only. Module doesn't save registers: A, B, Y, X0
*
* Test Method: 
*
*****************************************************************************/
static unsigned short U32divU16s(const unsigned long Dividend, const unsigned short Divisor, unsigned long * Result)
{

    asm(	move.w  Y0, Y1      		);  /* save real divider 						*/
    asm(	brclr   #$8000, Y0, Core	);	/* if (sign bit is not set) then go core 	*/
    asm(	lsr.w   Y1          		);  /* new divider:=divider>>1 					*/

    /*			division																*/
    asm(Core:               			);  /* main part of division 					*/

    asm(	clr     B           		);  /* preparing for division 					*/
    asm(	move.w  A1, B0      		);  /* move high number position in register B 	*/
    asm(	bfclr   #$0001, SR  		);  /* clear carry flag ( for first shifting )) */

    asm(	rep     #17         		);  /* 16-bit for division + 1-bit for sign 	*/
    asm(	div     Y1, B       		);  /* Positive quotient in B0 					*/

    asm(	move.w  B0, A1      		);  /* Save quotient in A1 						*/

    asm(	add     Y1, B       		);  /* Restore remainder in B1 					*/
    asm(	asr     B           		);  /* Required for correct integer remainder 	*/

    asm(	move.w  A0, B0      		);  /* move low number position in register B 	*/
    asm(	bfclr   #$0001, SR  		);  /* clear carry flag ( for first shifting )) */

    asm(	rep     #17         		);  /* 16-bit for division + 1-bit for sign 	*/
    asm(	div     Y1, B       		);  /* Positive quotient in B0 					*/

    asm(	move.w  B0, A0      		);  /* Save quotient in A0 						*/

    asm(	add     Y1, B       		);  /* Restore remainder in B1 					*/
    asm(	asr     B           		);  /* Required for correct integer remainder 	*/

    /*					correct the result if divisor is odd number						*/
    asm(	bfclr   #$FF, A2        	);
    asm(	bfclr   #$FF, B2        	);

    asm(	brclr   #$8000, Y0, Endofdiv); 	/* if sign bit not set go Endofdiv 			*/
    asm(	brclr   #$0001, A0, EvenA   );  /* if quotient(A/[Y>>1]) is even go OddA 	*/

    asm(	add     Y1, B       		);  /* odd quotient. remainder( A / Y ) = remainder( A / [Y>>1] ) + Y >> 1 */

    asm(EvenA:              			);

    asm(	asr     A           		); 	/* quotient(A\[Y>>1])>>=1 					*/
    asm(	brclr   #$0001, Y0, Endofdiv); 	/* if divider is even go Endofdiv 			*/
    asm(	move.w  B1, B0      		); 	/* for addition operation 					*/
    asm(	clr.w   B1          		);
    asm(	clr.w   X0          		);
    asm(	clr.w   Y1          		);

    /*			while(remainder<0) do { remainder += divider; quotient += 1 } 			*/
    asm(Circle:             			);

    asm(	cmp     A, B            	);  /* no more than 4 times 					*/
    asm(	bge     GetK        		);
    asm(	add     Y, B            	);  /* Y = divider 								*/
    asm(	inc.w   X0          		);
    asm(	bra     Circle      		);

    asm(GetK:               			);

    asm(	sub     A, B            	);  /* the same: remainder += divider * k, k = {0, 1, 2, 3, 4} */
    asm(	move.w  B0, B1      		);  /* calling convention 						*/
    asm(	move.w  Y0, B0      		);
    asm(	move.w  X0, Y0      		);
    asm(	bfclr   #$0001, SR  		);
    asm(	sbc     Y, A            	);
    asm(	move.w  B0, Y0      		);

    asm(Endofdiv:           			); /* end of division 							*/

    asm(	move.l  A10, X:(R2)  		); /* save results of division 					*/
    asm(	move.w  B1, Y0      		);
    asm(	rts                 		);
}

/*****************************************************************************
*
* Module: U64divU16
*
* Description: division unsigned 64 bits value by unsigned 16 bits value divider
*
* Returns: remainder
*
* Arguments: puDividend - not null pointer to 64 bits dividend
*            uDivider   - 16 bits unsigned divider
*            puQuotient - not null pointer to 64 bits quotient
*
* Range Issues: 
*
* Special Issues:  For executive loop only. Module doesn't save registers: A, B, Y, X0, C1, D1
*                                           Use SP
* Test Method: 
*
*****************************************************************************/
static unsigned short U64divU16(const U64 * puDividend, const unsigned short uDivider, U64 * puQuotient)
{       
    asm(adda    #1, SP      		);
    asm(move.w  Y1, X:(SP)+ 		);
    asm(move.w  B0, X:(SP)+ 		);
    asm(move.w  B1, X:(SP)+ 		);
    asm(move.w  A0, X:(SP)+ 		);
    asm(move.w  A1, X:(SP)+ 		);
    asm(move.w  D1, X:(SP)+ 		);
    asm(move.w  C1, X:(SP)+ 		);
                                                                        
    asm(move.l  X:(R2), B   		);  /* save X:(R2), X:(R2+1) to X:(SP) 				*/
    asm(move.w  B0, X:(SP)+ 		);
    asm(move.w  B1, X:(SP)  		);          
    asm(move.w  Y0, C1      		);  /* save divider (Y0) to C1 						*/

    asm(move.l  X:(R2+2), A 		);	/* move 2 high number positions in register A 	*/    
    asm(jsr     U32divU16s  		);  /* divide high positions 						*/
    asm(move.l  A10, x:(R3+2)		);  /* save high number positions of results 		*/

    asm(move.w  Y0, A1      		);  /* move reminder of first division  			*/
    asm(move.w  X:(SP), A0  		);  /* prepare next 16 bit for division 			*/
    asm(move.w  C1, Y0      		);  /* restore divider 								*/
    asm(jsr     U32divU16s  		);  /* divide middle number position 				*/
    asm(move.w  A0, D1      		);  /* save middle number position of result 		*/
		                                /* not at once to R3 because if user write U64divU16(&a, b, &a) */      
        		                        /* next division destroy this result 			*/
    asm(move.w  Y0, A1      		);  /* move reminder of second division  			*/
    asm(move.w  X:(SP-1), A0		);  /* prepare next 16 bit for division 			*/
    asm(move.w  C1, Y0      		);  /* restore divider 								*/
    asm(jsr     U32divU16s  		);  /* divide low number position 					*/

    asm(move.w  X:(SP)-, B1 		);  /* restore X:(R2), X:(R2+1) 					*/
    asm(move.w  X:(SP)-, B0 		);
    asm(move.l  B10, X:(R2) 		);
    asm(move.w  D1, A1      		);  /* save results of low bits D1A0 -> X:(R3) 		*/
    asm(move.l  A10, x:(R3) 		);


    asm(move.w  X:(SP)-, C1 		);
    asm(move.w  X:(SP)-, D1 		);
    asm(move.w  X:(SP)-, A1 		);
    asm(move.w  X:(SP)-, A0 		);
    asm(move.w  X:(SP)-, B1 		);
    asm(move.w  X:(SP)-, B0 		);
    asm(move.w  X:(SP)-, Y1 		);

    asm(rts                 		);
}

#if 0
/*****************************************************************************
*
* Module: TimerCallback
*
* Description: program counter routine for POSIX timer
*
* Returns: none
*
* Arguments: QT callback type, Timer device
*
* Range Issues: 
*
* Special Issues: to be optimized in asm
*
* Test Method: 
*
*****************************************************************************/
void  TimerCallback( posix_tContext* ptr )
{
    extern const union sigval sig; /* non OS version */

 //   periphBitClear( QTB_TCF, &((arch_sTimerChannel*)ptr->bspDevice)->StatusControlReg );             

    if(  --ptr->SoftCounter == 0 )
    {
        ptr->SoftCounter = ptr->reloadSoftCounter;
        if(ptr->pUserFunc != 0 ) 
            ptr->pUserFunc( sig );
    }
}
#endif
/*****************************************************************************
*
* Module:         timerRealTimeClockISR
*
* Description:   Update CLOCK_REALTIME system timer counter
*    
* Returns: None  
*
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
#pragma interrupt called
static void timerRealTimeClockISR(union sigval Params) 
{   /* update system counter */
    timerTick();  /* increments TimerISRCount */    
}

/* No operating system (nos) version */

/* Include OS hooks for Timer ticks */

void timerSleep(long Ticks)
{
    unsigned long TargetISRCount = TimerISRCount + Ticks;
    
    while(TimerISRCount < TargetISRCount)
    {
          continue;
    }
}

void timerTick(void)
{
    TimerISRCount++;
}

/*****************************************************************************
*
* Module: timer_create
*
* Description (POSIX) :    
*   The timer_create() function creates a per-process timer using the specified
*   clock, ClockID, as the timing base. The EventParams argument, if non-NULL, 
*   points to a sigevent structure. This structure, allocated by the application, 
*   defines the asynchronous notification that will occur when the timer expires. 
*    
* Returns: 
*   If the call succeeds, timer_create() returns zero and in the 
*   location referenced by TimerID, stores a timer ID of type timer_t used to
*   identify the timer in subsequent timer requests. This timer ID will be 
*   unique within the calling process until the timer is deleted.      
*
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
int timer_create (  const clockid_t          ClockID, 
                    const struct sigevent *  EventParams, 
                    timer_t *          TimerID)
{
    extern const posix_tConfig posixDeviceMap[];
    posix_tContext* ptr = NULL;
    const posix_tConfig * cfg = &posixDeviceMap[0];

    while(1)
    {
        if(cfg->ctx == 0 )
            return -1; /* last timer */
        if( (unsigned short)cfg->base == ClockID )
            break;
        cfg++;
    }
    
    *TimerID = ClockID;
    ptr = cfg->ctx;
    ptr->pUserFunc = EventParams->sigev_notify_function; /* save user function here */
    timer_qt_writeCompareReg1( (arch_sTimerChannel*)ClockID, (unsigned short)ptr);
    
    return 0;
}

/*****************************************************************************
*
* Module:         timer_delete
*
* Description (POSIX) :    
*   The timer_delete() function deletes the specified timer, timerid, previously
*   created by the timer_create() function. If the timer is armed when 
*   timer_delete() is called, the behaviour will be as if the timer is
*   automatically disarmed before removal. The disposition of pending 
*   signals for the deleted timer is unspecified. 
*    
* Returns: If successful, the function returns a value of zero. Otherwise, 
*       the function returns a value of -1.       
*
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
int timer_delete(timer_t TimerID)
{
        timer_qt_disable( (arch_sTimerChannel*)TimerID );
}

/*****************************************************************************
*
* Module:         time2Ticks
*
* Description  :    
*   Translate time (ns) to ticks
*    
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
static void time2Ticks(const struct timespec * pTimeValue, U64 * pTicks)
{
    U64 nsTime;
    
    /*translate into nanoseconds*/
    
    nsTime.w[0] = pTimeValue->tv_sec;
    nsTime.w[1] = 0;
    
    U64mulU32(&nsTime, 1000000000UL, &nsTime);      /* nSec=Sec*10^9    */
    U64addU32(&nsTime, pTimeValue->tv_nsec, &nsTime);
    
    /* translate software counter into ticks        */
    /* tick = nanoSec * qtINPUT_FREQUENCY / 10^9    */

    U64mulU32(&nsTime, IP_BUS_FREQUENCY, &nsTime);  
    
    U64divU16(&nsTime, 1000, &nsTime);
    
    U64divU16(&nsTime, 1000, &nsTime);
    
    U64divU16(&nsTime, 1000, &nsTime);   
    pTicks->w[0] = nsTime.w[0];
    pTicks->w[1] = nsTime.w[1];
}

/*****************************************************************************
*
* Module:         ticks2ns
*
* Description  :    
*   Translate ticks to ns
*    
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
static void ticks2ns(U64* tickPeriod, U64* timePeriod)
{
    /* translate ticks to ns: ns=10^9 * ticks / FREQ  */
    U64mulU32(tickPeriod, 1000000000, timePeriod);
    U64divU16(timePeriod, IP_BUS_FREQUENCY_1, timePeriod);
    U64divU16(timePeriod, IP_BUS_FREQUENCY_2, timePeriod);
}

/*****************************************************************************
*
* Module:         ns2Time
*
* Description  :    
*   It represents time, which is specified in ns, as <sec, ns>
*    
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
static void ns2Time(U64* timePeriod, struct timespec * tp)
{
    U64     secCount;
    unsigned long nsCount;
	U64	   	tmp;

    /* timePeriod   = 10^9 * secCount  + nsCount 							*/
    /* 1:timePeriod = 10^3 * secCount1 + nsCount1 							*/
    /* 2:secCount1  = 10^3 * secCount2 + nsCount2 							*/
    /* 3:secCount   = 10^3 * secCount2 + nsCount3 							*/
    /* 4:nsCount    = 10^6 * nsCount3  + nsCount2 * 10^3 + nsCount1 		*/

    nsCount = U64divU16(timePeriod, 1000, &secCount);
	
	tmp.w[0] = U64divU16(&secCount, 1000, &secCount);	
	tmp.w[1] = 0;
	
	U64mulU32(&tmp,1000,&tmp);
		
    nsCount +=tmp.w[0];
    
    tmp.w[0]= U64divU16(&secCount, 1000, &secCount);
   	U64mulU32(&tmp,1000000,&tmp);
   	nsCount +=tmp.w[0];
     
    tp->tv_nsec = nsCount;
    tp->tv_sec  = secCount.w[0];
}

static unsigned short factorization(U64* tickPeriod, unsigned long* SWCounter)
{
	struct  /* representation nuber A as: A = divider * quot + rem, 0 < rem < divider    */
    {
        unsigned long quot;
        unsigned short rem;
    }					 	div1, div2;
    
	U64                  	SW;
    
    const unsigned short           SearchCount = 32;   /* max number of divider to try     */

                            
    unsigned short 		        rem0, rem1, minRem,
                            HWCounter,
                            foundDivider;
                            
    for( HWCounter = 0xFFFF-SearchCount, minRem = 0xFFFF;
	         HWCounter < 0xFFFF && minRem > 500;
    		     ++HWCounter)

    {
        div1.rem = rem0 = U64divU16(tickPeriod, HWCounter, &SW);
        div2.rem = rem1 = HWCounter - rem0;
            
        /* Predcondition: SWCounter.w[1] == 0 */

        if(rem0 >= SW.w[0])
        {
            div1.rem = U32divU16s((unsigned long)rem0, (unsigned short)SW.w[0], &div1.quot);

            /* prevent divider overflow */
            if(div1.quot + (unsigned long)HWCounter > 0xFFFF) div1.rem = 0xFFFF;
        };

        if(rem1 >= SW.w[0] + 1)
        {
            div2.rem = U32divU16s((unsigned long)rem1, (unsigned short)SW.w[0] + 1, &div2.quot);

            /* prevent small divider */
            if(HWCounter < 0x7FFF + (unsigned long)div2.quot) div2.rem = 0xFFFF;
        };

        if(rem0 < minRem)
        {
            minRem       = rem0;
            foundDivider = HWCounter;
        };

        if(rem1 < minRem)
        {
            minRem       = rem1;
            foundDivider = HWCounter;
        };

        if(div1.rem < minRem)
        {
            minRem       = div1.rem;
            foundDivider = HWCounter  + div1.quot;
        };

        if(div2.rem < minRem)
        {
            minRem       = div2.rem;
            foundDivider = HWCounter - div2.quot;
		};

    };/* for */
 
    HWCounter   = foundDivider;
    rem0        = U64divU16(tickPeriod, foundDivider, &SW);
        
    if	(	rem0 > (	foundDivider - rem0	)	)
    {
        U64addU32(&SW, 1, &SW);
    }
        
    *SWCounter = SW.w[0];
    return foundDivider;
}
/*****************************************************************************
*
* Module: timer_settime
*
* Description (POSIX): Set timer values
*
* Returns: A return value of 0 indicates that the call succeeded.    
*
* Arguments:  Timer device handle, assigned initial and period time.
*
* Range Issues: time precision - 10us, initial time ignored 
*
* Special Issues:  For executive loop only 
*
*****************************************************************************/
int timer_settime(      timer_t TimerID, 
                        int Flags, 
                        const struct itimerspec * pTimeValue, 
                        struct itimerspec * pOvalue
                 )

{
    extern const qt_sQuadState  timerStdFastStart;

    posix_tContext *    ptr = POSIX_TIMER_GET_CONTEXT(TimerID);

    unsigned long             SWCounter;

    unsigned short             HWCounter, reminder;

    U64                 tickPeriod;


    /*	translate into nanoseconds	*/
    if ( ( pTimeValue->it_interval.tv_sec == 0 ) && (  pTimeValue->it_interval.tv_nsec == 0 ))
   	    time2Ticks(&pTimeValue->it_value, &tickPeriod);
    else
	    time2Ticks(&pTimeValue->it_interval, &tickPeriod);

    if(tickPeriod.w[1] == 0 && tickPeriod.w[0] < 0x10000UL )
    {
        HWCounter = (unsigned short)tickPeriod.w[0];
        SWCounter = 1;
    }
    else
    {
        /* factorization */
        HWCounter = factorization(&tickPeriod, &SWCounter);
    };


    if ( ( pTimeValue->it_interval.tv_sec == 0 ) && (  pTimeValue->it_interval.tv_nsec == 0 ))
    {
	    ptr->reloadSoftCounter = 0;
    	ptr->SoftCounter = SWCounter;
    }
    else
    {
        /* save software counter */
	    ptr->reloadSoftCounter = SWCounter;

	    /*	translate first interval into nanoseconds	*/
    	time2Ticks(&pTimeValue->it_value, &tickPeriod);

	    /* Represent time in tick as rem0=HWCounter*A + rem0 */
	    reminder = U64divU16(&tickPeriod, HWCounter, &tickPeriod);

    	ptr->SoftCounter = tickPeriod.w[0]; /* inital value, there is original SWCounter at reloadSoftCounter */
    }
    /* load and start low ( hardware ) counter  */
    timer_qt_writeCompareReg2( (arch_sTimerChannel*)TimerID, 0	);
    timer_qt_writeLoadReg( (arch_sTimerChannel*)TimerID, (short)HWCounter );
    timer_qt_writeCounterReg( (arch_sTimerChannel*)TimerID, reminder );
    timer_CallbackOnCompare_On(	(arch_sTimerChannel*)TimerID	);

	if (HWCounter > 0x4B0)
   		timer_qt_start((arch_sTimerChannel*)TimerID, timerStdFastStart.value);
}


/*****************************************************************************
*
* Module: clock_settime
*
* Description (POSIX) :    
*   The clock_settime() function sets the specified clock, clock_id, 
*   to the value specified by tp. Time values that are between two consecutive
*   non-negative integer multiples of the resolution of the specified clock 
*   are truncated down to the smaller multiple of the resolution. 
*    
* Returns: A return value of 0 indicates that the call succeeded.    
*
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
int clock_settime(clockid_t ClockID, const struct timespec * tp)
{
    return 0;
}

/*****************************************************************************
*
* Module: clock_gettime
*
* Description (POSIX) :    
*   The clock_gettime() function returns the current value tp for the 
*   specified clock, clock_id.  
*    
* Returns: A return value of 0 indicates that the call succeeded.    
*
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
int clock_gettime(clockid_t ClockID, struct timespec * tp)
{
    posix_tContext* ptr  = POSIX_TIMER_GET_CONTEXT(ClockID);
	U64 timePeriod;

    timePeriod.w[0]=(unsigned long)ptr->reloadSoftCounter - (unsigned long)ptr->SoftCounter;
    timePeriod.w[1]=0;

    /*	get expired ticks	*/
    U64mulU32(&timePeriod, timer_qt_readLoadReg( (arch_sTimerChannel*)ClockID ), &timePeriod);
    U64addU32(&timePeriod, timer_qt_readCounterReg( (arch_sTimerChannel*)ClockID ), &timePeriod);
    U64mulU32(&timePeriod, POSIX_TIMER_READ_PRESCALER_VALUE((arch_sTimerChannel*)ClockID, 0), &timePeriod);

    /*	translate ticks to ns	*/
    ticks2ns(&timePeriod, &timePeriod);

    /*	factorization	*/
    ns2Time(&timePeriod, tp);

    return 0;
}



/*****************************************************************************
*
* Module:         clock_get_system_time
*
* Description:   system time for CLOCK_REALTIME timer
*    
* Returns: A return value of 0 indicates that the call succeeded.    
*
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
int clock_get_system_time( struct timespec * tp)
{
	U64 timePeriod;


    /* translate ISR count to ns */
    timePeriod.w[0] = TimerISRCount;
    timePeriod.w[1] = 0;
    U64mulU32(&timePeriod, REAL_TIME_TIMER_RESOLUTION, &timePeriod);

    ns2Time(&timePeriod, tp);

    return 0;
}

/*****************************************************************************
*
* Module:         clock_getres
*
* Description (POSIX) :    
*   The resolution of any clock can be obtained by calling clock_getres(). 
*   Clock resolutions are implementation-dependent and cannot be set by 
*   a process. If the argument res is not NULL, the resolution of the
*   specified clock is stored in the location pointed to by res. 
*    
* Returns: A return value of 0 indicates that the call succeeded.    
*
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
int clock_getres(clockid_t ClockID, struct timespec * Resolution)
{
	U64 tickPeriod;
	U64 timePeriod;

    /*	get ticks	*/
    tickPeriod.w[0] = POSIX_TIMER_READ_PRESCALER_VALUE(ptr->bspDevice, 0);
    U64mulU32(&tickPeriod, timer_qt_readLoadReg((arch_sTimerChannel*)ClockID), &tickPeriod);

    ticks2ns(&tickPeriod, &timePeriod);
    ns2Time(&timePeriod, Resolution);

    return 0;
}

/*****************************************************************************
*
* Module:         nanosleep
*
* Description (POSIX) :    
*   The nanosleep() function causes the current thread (SDK executive loop )
*   to be suspended from execution until either the time interval  specified 
*   by the rqtp argument has elapsed or a signal is delivered to the calling
*   thread and its action is to invoke a signal-catching function or 
*   to terminate the process. The suspension time may be longer than requested
*   because the argument value is rounded up to an integer multiple of the sleep
*   resolution or because of the scheduling of other activity by the system.
*   But, except for the case of being interrupted by a signal, the suspension 
*   time will not be less than the time specified by rqtp, as measured by 
*   the system clock, CLOCK_REALTIME
*    
* Returns: f the nanosleep() function returns because the requested time 
*   has elapsed, its return value is zero.     
*
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
int nanosleep(const struct timespec * rqtp, struct timespec * rmtp)
{
	U64     tickPeriod;

    time2Ticks(rqtp, &tickPeriod);

    U64divU16(&tickPeriod, REAL_TIME_TIMER_STEP, &tickPeriod);
    U64divU16(&tickPeriod, REAL_TIME_TIMER_PRESCALER, &tickPeriod);


    if(tickPeriod.w[0] == 0)
    {
        tickPeriod.w[0] = 1;
    }

    timerSleep( tickPeriod.w[0] );

    return 0;
}


/*****************************************************************************
*
* Module:         timerCreate
*
* Description:   Configuration of SDK POSIX real time timer block.
*    
* Returns:          passed value    
*
* Range Issues:   None
* Special Issues: None
* Test Method:    timers.mcp
*
*****************************************************************************/
Result timerCreate(	/*	const char * pName	*/	)
{
	timer_t      	    realTimeTimer;
	struct sigevent     realTimeEvent;
    register posix_tContext *     ptr;

    realTimeEvent.sigev_notify_function = timerRealTimeClockISR;

    timer_create(CLOCK_REALTIME, &realTimeEvent, &realTimeTimer);

	ptr = POSIX_TIMER_GET_CONTEXT(realTimeTimer);

	/*	save software counter	*/
    ptr->reloadSoftCounter  = 1;
    ptr->SoftCounter        = 1;

    timer_qt_writeCompareReg2( (arch_sTimerChannel*)realTimeTimer, 0);
    timer_qt_writeLoadReg( (arch_sTimerChannel*)realTimeTimer, REAL_TIME_TIMER_STEP );
    timer_qt_writeCounterReg( (arch_sTimerChannel*)realTimeTimer, 0 );
    timer_CallbackOnCompare_On((arch_sTimerChannel*)realTimeTimer );

    timer_qt_start((arch_sTimerChannel*)realTimeTimer, timerStdFastStart.value /* QTB_TCFIE */ );

    return PASS;
}

/****************************************************************************************
*																						*
*																						*
*																						*
*										MEASURE BLOCK									*
*																						*
*																						*
*																						*
*****************************************************************************************/

/**************************************************
*	const defenition
***************************************************/
#define HIGH_BITS_VALUE 0xFFFF0000
#define LOW_BITS_VALUE 	0x0000FFFF

static measure_t gCalibrate = 0;

volatile arch_sTimerChannel * gArchQtmerOut;
volatile arch_sTimerChannel * gArchQtmerIn;

static unsigned long InitialLoadValue = 0xFFFF;

/**************************************************
* prototypes 
***************************************************/
void   timer_get_counters(measure_t* );
short time_test_cascade(volatile arch_sTimerChannel * gTimerOut,	volatile arch_sTimerChannel * gTimerIn);

/*******************************************************************************
*
*  Function: time_test_cascade
*
*  Description: test cascede timers on belomging to one groop 
*
*  Arguments:
*			gTimerOut
*			gTimerIn  - arch of timers
*
*
*  Returns: -1 wrong cascade timers(timers cannot connect in cascade)
*			correct cascade InputSource
*  Range Issues: none
*
*  Special Issues: none
*
*******************************************************************************/
short time_test_cascade(volatile arch_sTimerChannel * gTimerOut, volatile arch_sTimerChannel * gTimerIn)
{
	unsigned short	tmpShift = (unsigned short) gTimerIn;
	
		if( gTimerOut == gTimerIn ) return -1;

		switch	(	(unsigned short)	gTimerOut - (unsigned short)	&ArchIO.TimerA.Channel0	)
		{
			case 0   : /* Cascade consist of timers from groop A */
			case 8   :
			case 16  :
			case 24  : tmpShift -= (unsigned short) &ArchIO.TimerA.Channel0; 
					   break;
#if 0 /* B timer-groop reserved */
			case 64  : /* Cascade consist of timers from groop B */
			case 72  :
			case 80  :
			case 88  : tmpShift -= (unsigned short) &ArchIO.TimerB.Channel0; 
					   break;
#endif						
			case 128 : /* Cascade consist of timers from groop C */
			case 136 :
			case 144 :
			case 152 : tmpShift -= (unsigned short) &ArchIO.TimerC.Channel0;
					   break;

			case 192 : /* Cascade consist of timers from groop D */
			case 200 :
			case 208 :
			case 216 : tmpShift -= (unsigned short) &ArchIO.TimerD.Channel0;
					   break;

    		default  : return -1;
    	}


    	switch	(	tmpShift	)
		{
			case 0  : return qtCounter0Output;break;

		    case 8  : return qtCounter1Output;break;

			case 16 : return qtCounter2Output;break;

			case 24 : return qtCounter3Output;break;

			default : return -1;
		}

	return -1;
}

/*******************************************************************************
*
*  Function: timer_Init_Measure
*
*  Description: initialize measure library
*
*  Arguments:
*			_gArchQtmer1 
*			_gArchQtmer2 - arch of timers
*
*
*  Returns: calibrate value
*
*  Range Issues: none
*
*  Special Issues: none
*
*******************************************************************************/
unsigned long timer_Init_Measure(volatile arch_sTimerChannel * gArchQtmer1,	volatile arch_sTimerChannel * gArchQtmer2)
{
	short InputSourceOut;

	if ( ( gArchQtmer1 == NULL ) && ( gArchQtmer2 == NULL ) )
	{ /* default timers for measure are C0,C1 */

			gArchQtmer1 = &ArchIO.TimerC.Channel1;
			gArchQtmer2 = &ArchIO.TimerC.Channel0;
	}

	InputSourceOut = time_test_cascade(gArchQtmer1,	gArchQtmer2);

	if ( InputSourceOut != -1 )
	{
		measure_t measureDesc;

		static qt_sQuadState gQtimerParamIn = {
			/*    unsigned int    OutputMode              :3; 1,	*/	qtDeassertOnCompare,
			/*    unsigned int    CoChannelInitialize     :1; 		*/	0,
			/*    unsigned int    CountDirection          :1; 1,	*/	qtDown,
			/*    unsigned int    CountLength             :1; 1,	*/	qtUntilCompare,
			/*    unsigned int    CountFrequency          :1; 0,	*/	qtRepeatedly,
			/*    unsigned int    SecondaryInputSource    :2; 0,	*/	qtSISCounter0Input,
			/*    unsigned int    InputSource             :4; 8,	*/	qtPrescalerDiv1,
			/*    unsigned int    Mode                    :3; 2,	*/	qtCountBothEdges,
			/*    unsigned int    OutputEnable            :1; 		*/	0,
			/*    unsigned int    OutputPolarity          :1; 0,	*/	qtNormal,
			/*    unsigned int                            :1;		*/
			/*    unsigned int                            :1;		*/
			/*    unsigned int    OutputOnMaster          :1;		*/ 	0,
			/*    unsigned int    Master                  :1;		*/ 	0,
			/*    unsigned int    CaptureMode             :2; 2,	*/ 	qtDisabled,
			/*    unsigned int                            :1;		*/
			/*    unsigned int    InputPolarity           :1; 0,	*/ 	qtNormal
			/*    unsigned int                            :6;		*/
			};

		static qt_sQuadState gQtimerParamOut = {
			/*    unsigned int    OutputMode              :3; 1,	*/	qtDeassertOnCompare,
			/*    unsigned int    CoChannelInitialize     :1; 		*/	0,
			/*    unsigned int    CountDirection          :1; 1,	*/	qtDown,
			/*    unsigned int    CountLength             :1; 1,	*/	qtUntilCompare,
			/*    unsigned int    CountFrequency          :1; 0,	*/	qtRepeatedly,
			/*    unsigned int    SecondaryInputSource    :2; 0,	*/	qtSISCounter0Input,
			/*    unsigned int    InputSource             :4; 		*/	0,
			/*    unsigned int    Mode                    :3; 7,	*/	qtCascadeCount,
			/*    unsigned int    OutputEnable            :1; 		*/	0,
			/*    unsigned int    OutputPolarity          :1; 0,	*/	qtNormal,
			/*    unsigned int                            :1;		*/
			/*    unsigned int                            :1;		*/
			/*    unsigned int    OutputOnMaster          :1;		*/ 	0,
			/*    unsigned int    Master                  :1;		*/ 	0,
			/*    unsigned int    CaptureMode             :2; 2,	*/ 	qtDisabled,
			/*    unsigned int                            :1;		*/
			/*    unsigned int    InputPolarity           :1; 0,	*/ 	qtNormal
			/*    unsigned int                            :6;		*/
			};

			gQtimerParamOut.bits.InputSource = InputSourceOut;

			/*	quad timer initialize	*/
			gArchQtmerOut = gArchQtmer1;
			gArchQtmerIn = gArchQtmer2;

			/*	run timers	*/
			timer_qt_writeCompareReg1(	gArchQtmerOut,	0);
			timer_qt_writeCompareReg1(	gArchQtmerIn ,	0);
			timer_qt_writeCompareReg2(	gArchQtmerOut,	0);
			timer_qt_writeCompareReg2(	gArchQtmerIn ,	0);
			timer_qt_writeLoadReg	 (	gArchQtmerOut,	InitialLoadValue);
			timer_qt_writeLoadReg	 (	gArchQtmerIn ,	InitialLoadValue);

			timer_qt_start	(gArchQtmerOut,	gQtimerParamOut.value);
			timer_qt_start	(gArchQtmerIn ,	gQtimerParamIn.value);

			/*	get calibrate value */
			timer_begin_measure	(	&measureDesc	);
			timer_end_measure	(	&measureDesc	);
			gCalibrate = measureDesc;

			return gCalibrate;
	}

	return -1;
}

/*******************************************************************************
*
*  Function: timer_Uninit_Measure
*
*  Description: uninitialize measure library
*
*  Arguments:
*		none
*
*  Returns: none
*
*  Range Issues: none
*
*  Special Issues: none
*
*******************************************************************************/
void timer_Uninit_Measure(void)
{
	timer_qt_disable	(	gArchQtmerOut	);
	timer_qt_disable	(	gArchQtmerIn	);
}

/*******************************************************************************
*
*  Function: timer_get_counters
*
*  Description: gets current value real-time timer
*
*  Arguments:
*		pTime - pointer to measurenment value in ticks (measure_t)
*
*  Returns: none
*
*  Range Issues: none
*
*  Special Issues: none
*
*******************************************************************************/
void timer_get_counters(measure_t* pTime)
{
	*pTime = timer_qt_readCounterReg(gArchQtmerIn);
	*pTime |= (unsigned long)	timer_qt_readHoldReg(gArchQtmerOut)	<<	16;
}

/*******************************************************************************
*
*  Function: timer_begin_measure
*
*  Description: starts measure procedure
*
*  Arguments:
*		pTime - pointer to measurenment value in ticks (measure_t)
*
*  Returns: none
*
*  Range Issues: none
*
*  Special Issues: none
*
*******************************************************************************/
void timer_begin_measure(measure_t* pTime)
{
	timer_get_counters	(	pTime	);
}

/*******************************************************************************
*
*  Function: timer_end_measure
*
*  Description: finishes measure procedure and calculate expired time
*
*  Arguments:
*			pTime - pointer to measurenment value in ticks (measure_t)
*
*  Returns: none
*
*  Range Issues: none
*
*  Special Issues: none
*
*******************************************************************************/
void timer_end_measure( measure_t* pTime)
{
	measure_t	tEnd;
	measure_t	timeRez;

	timer_get_counters(	&tEnd	);
	
	timeRez = *pTime;

	/*	calculate expired time	*/
	if(	( (	timeRez & HIGH_BITS_VALUE	) ) < ( tEnd & HIGH_BITS_VALUE ) )
	{
		timeRez += ( ( InitialLoadValue	<<	16 ) - ( tEnd & HIGH_BITS_VALUE ) + 0x10000 );
	}
	else timeRez -= tEnd & HIGH_BITS_VALUE;

	if	( ( timeRez & LOW_BITS_VALUE ) < ( tEnd & LOW_BITS_VALUE ) )
	{
		/*	validate	*/
		timeRez += ( InitialLoadValue - ( tEnd & LOW_BITS_VALUE ) + 1 );
		timeRez -= 0x10000;
	}
	else timeRez -= tEnd & LOW_BITS_VALUE;

	if ( InitialLoadValue != 0xFFFF )
	{

	U64 tmp;

		tmp.w[0] = (	( timeRez & HIGH_BITS_VALUE ) >> 16	);
		tmp.w[1] = 0;

		U64mulU32(&tmp,	InitialLoadValue,	&tmp);

		/*	get real size	*/
		timeRez =  tmp.w[0] + (timeRez & LOW_BITS_VALUE);
	}

	/*	calibrate	*/
	if( timeRez >= gCalibrate ) timeRez -= gCalibrate;
		else timeRez = 0;

	*pTime = timeRez;
}

/*******************************************************************************
*
*  Function: timer_resume
*
*  Description: continue measure procedure. This function can be used for multiple measurenment with one descriptor 
*
*  Arguments:
*			pTime - pointer to measurenment value in ticks (measure_t)
*
*  Returns: none
*
*  Range Issues: none
*
*  Special Issues: none
*
*******************************************************************************/
void timer_resume(measure_t* pTime)
{
	timer_get_counters(	pTime	);
}

/*******************************************************************************
*
*  Function: timer_measure_ticks_to_us
*
*  Description: convert ticks meaning to us meaaning
*	
*  Arguments:
*		pTime - pointer to measurenment value in ticks (measure_t)
*
*  Returns: meaning in us.
*
*  Range Issues: none
*
*  Special Issues: none
*
*******************************************************************************/
unsigned long timer_measure_ticks_to_us(measure_t* pTime)
{
	/*	us = tick * 10^6 / IP_BUS_FREQUENCY	*/
	U64 us;
	U64 tmp;
		us.w[0] = *pTime;
		us.w[1] = 0;

		U64mulU32(&us,	1000000,	&tmp);

		U64divU16(&tmp, IP_BUS_FREQUENCY_2,	&us);
		U64divU16(&us,  IP_BUS_FREQUENCY_1,	&tmp);

	return tmp.w[0];
}